#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"
#
# runabc.tcl - by seymour.shlien@crc.doc.ca  
# This is graphics user interface to the abc2midi and abc2ps programs.
# This script and the above programs are public domain.
# multiple select by ste_mi@yahoo.com
#

set runabc_version 1.561
set runabc_date "(Mar 06 2002)"

wm protocol . WM_DELETE_WINDOW {write_runabc_ini; exit}
wm resizable . 0 0
set globsel 0
global df
global globsel 

proc messages {msg} {
    global df
    if {[winfo exists .msg] != 0} {destroy .msg}
    toplevel .msg
    message .msg.out -text $msg -font $df
    pack .msg.out
}

proc platform {} {
global tcl_platform

    set plat $tcl_platform(platform)
    set os $tcl_platform(os)
    set osver $tcl_platform(osVersion)
    set machine $tcl_platform(machine)
    set welcome [format "Evidently you are running runabc.tcl for the first \
    time. You are running the script on %s with %s version %s on a \
    %s. After closing this window you should click the config button and \
    indicate the path to the other executables called by this script.\
    See the readme file and runabc.txt for more information." \
     $plat $os $osver $machine]
    messages $welcome
}



# default values for options
proc midi_init {} {
global midi df
global runabc_version

    set midi(version) $runabc_version
    set midi(font_family) [font actual . -family]
    set midi(font_family_toc) Terminal
    set midi(font_size) [font actual . -size] 
    set midi(font_weight) [font actual . -weight]
    set df [font create -family $midi(font_family) -size $midi(font_size) \
     -weight $midi(font_weight)] 
    set midi(path_abc2ps) abc2ps.exe
    set midi(path_yaps) yaps.exe
    set midi(path_abcm2ps) abcm2ps.exe
    set midi(path_abcmatch) abcmatch.exe
    set midi(path_abc2midi) abc2midi.exe
    set midi(path_abc2abc) abc2abc.exe
    set midi(path_gs) c:/gstools/gsview/gsview32
    set midi(path_midiplay) C:/Music/Winamp/Winamp.exe
    set midi(midiplay_options) ""
    set midi(gs_options) ""
    set midi(alt_path_midiplay) C:/windows/mplayer.exe
    set midi(alt_midi_options) "/play /close"
    set midi(midi_dir) "tmp"
    set midi(path_editor) ""
    set midi(player) 0
    set midi(transfer_prot_1) 0
    set midi(transfer_prot_2) 0
    set midi(red_acc) 0

    # abc2abc default parameters
    set midi(no_clipboard) 1

    # abc2midi default parameters
    set midi(channel) 1
    set midi(program) 0
    set midi(chordprog) 0
    set midi(bassprog) 0
    set midi(beat_a) 127
    set midi(beat_b) 120
    set midi(beat_c) 90
    set midi(beat_n) 4
    set midi(ratio_m) 1
    set midi(ratio_n) 2
    set midi(chordvol) 127
    set midi(bassvol) 127
    set midi(transpose) 0
    set midi(grace) 1/2
    set midi(tempo) 120
    set midi(melody) 1
    set midi(chords) 1

    # ab2ps default parameters
    set midi(ps_creator) abc2ps
    set midi(ps_scale) 0.5
    set midi(ps_width) 250
    set midi(ps_lmargin) 20
    set midi(ps_glue) fill 
    set midi(ps_shrink) 0.9
    set midi(ps_staffsep) 50
    set midi(ps_c) 0
    set midi(ps_fmt_flag) 0
    set midi(ps_fmt_file) letter.fmt
    set midi(ps_maxvent) 4
    set midi(ps_maxsent) 800
    set midi(bpsvoice) 0
    set midi(psvoice) ""
    set midi(ps_bbar) 0
    set midi(ps_bnumb) 0
    set midi(ps_bppage) 0
    set midi(ps_bxref) 0
    set midi(ps_bhist) 0
    set midi(ps_nolyric) 0
    set midi(ps_noslur) 0


    # yaps default parameters
    set midi(papersize) 0
    set midi(yaps_voice) 0
    set midi(yaps_scale) 0.7
    set midi(yaps_lmargin) 50
    set midi(yaps_tmargin) 50
    set midi(yapsx) 0
    set midi(yaps_landscape) 0

    # open/save parameters
    set midi(abc_open) open.abc
    set midi(midi_save) sample.mid
    set midi(history_length) 0
    for {set i 0} {$i < 6} {incr i} {set midi(history$i) ""}
    set midi(abc_default_file) edit.abc 
    set midi(abc_work_folder)  workfold

    for {set i 1} {$i <= 16} {incr i} {
	set midi(lvoice$i) 64
	set midi(voice$i) 0
        set midi(pvoice$i) 64
    }
    set midi(ignoreQ) 0
    set midi(ignoremidi) 0
    set midi(blank_lines) 1
#
#editor settings
    set midi(edit_body_colour) red3
    set midi(edit_field_colour) green4
    set midi(edit_comment_colour) blue2
    set midi(edit_selectbackground) orange
    set midi(startnumber) 1

#find setting
    set midi(searchdir) "../abcfiles"

#match settings
   set midi(match_timesig) 2/4
   set midi(match_keysig) G
   set midi(match_length) 1/8
   set midi(match_body) GABC|
   set midi(match_resolution) 12 
   set midi(match_any) 1
   set midi(match_contour) 1
}

# save all options, current abc file
proc write_runabc_ini {} {
global midi

    set handle [open runabc.ini w]
    foreach item [lsort [array names midi]] {
	puts $handle "$item $midi($item)"
    }
    close $handle
}

# read all options
proc read_runabc_ini {} {
global midi df
    set handle [open runabc.ini r]
    while {[gets $handle line] >= 0} {
	set error_return [catch {set n [llength $line]} error_out]
        if {$error_return} continue
	set contents "" 
	set param [lindex $line 0]
	for {set i 1} {$i < $n} {incr i} {
	  set contents [concat $contents [lindex $line $i]]
	}
#if param is not already a member of the midi array (set by midi_init),
#then we ignore it. This prevents midi array filling up with obsolete
#parameters used in older versions of the program.
        set member [array names midi $param]
        if [llength $member] { set midi($param) $contents }
    }
font configure $df -family $midi(font_family) -size $midi(font_size) \
     -weight $midi(font_weight) 
}


# init global variables
set types {{{abc files} {*.abc}}
           {{all} {*}}}
set miditype {{{midi files} {*.mid}}}
set exec_out "This window show messages produced by play and display commands."

midi_init

if [file exists runabc.ini] {
    read_runabc_ini
    set midi(version) $runabc_version
} else {
    platform
}


    #######################
    #     Tk Interface    #
    #######################

###     Control Functions    ######

frame .abc
frame .abc.file
frame .abc.functions -borderwidth 2
frame .abc.auxfunc   -borderwidth 2
frame .abc.titles

# file entry and button
entry .abc.file.entry -width 40 -relief sunken -textvariable midi(abc_open) \
 -font $df
set w .abc.file.menu
menubutton $w -text file  -relief raised  -menu $w.type -font $df
menu $w.type -tearoff 0
$w.type add radiobutton  -label "open" -command file_browser  -font $df

for {set i 0} {$i < $midi(history_length)} {incr i} {
  $w.type add radiobutton  -label $midi(history$i) \
    -value $i -variable history_index -command process_history -font $df
}   
pack .abc.file.menu -side left 
pack .abc.file.entry -side left -expand y -fill x 
pack .abc.file -side top -fill x

# first row of buttons
button .abc.functions.quit  -text quit     -command {write_runabc_ini; exit} -font $df -borderwidth 3
button .abc.functions.play  -text play     -command play_action              -font $df -borderwidth 3
button .abc.functions.disp  -text display  -command display_action           -font $df -borderwidth 3
button .abc.functions.extr  -text extract  -command {show_extract_page}      -font $df -borderwidth 3

set w .abc.functions.editmenu
menubutton $w -text edit  -relief raised  -menu $w.type -font $df -borderwidth 3
menu $w.type -tearoff 0
$w.type add radiobutton  -label "edit file" -command  {
         . config -cursor watch
          update
          abc_edit midi(abc_open)
          . config -cursor arrow
          update}  -font $df
$w.type add radiobutton  -label "edit selection" -command  {
        set outfile [extract_title_of_first_tune [title_selected] $midi(abc_open)]
        set outfile [file join $midi(abc_work_folder) $outfile.abc]
	copy_selection_to_file [title_selected] $midi(abc_open) $midi(abc_default_file) 
        file rename -force $midi(abc_default_file) $outfile
        abc_edit outfile
     } -font $df
$w.type add radiobutton  -label "my selection editor" -command  {
        set outfile [extract_title_of_first_tune [title_selected] $midi(abc_open)]
        set outfile [file join $midi(abc_work_folder) $outfile.abc]
	copy_selection_to_file [title_selected] $midi(abc_open) $midi(abc_default_file)
        file rename -force $midi(abc_default_file) $outfile
	tcl_abc_edit $outfile
     } -font $df
$w.type add radiobutton -label "new tune"       -command edit_new_tune -font $df
$w.type add radiobutton -label "abc2abc"        -command show_abc2abc_page -font $df
$w.type add radiobutton -label "view X.tmp"     -command show_tmpfile -font $df
$w.type add radiobutton -label "save midi file" -command midisave -font $df
$w.type add cascade -label "copy to file" -menu $w.type.copy -font $df
menu $w.type.copy -tearoff 0
$w.type.copy add radiobutton -label "copy" -font $df \
  -command {start_copy_selected_files w 0} 
$w.type.copy add radiobutton -label "copy and renumber" -font $df \
  -command {start_copy_selected_files w 1} 
$w.type.copy add radiobutton -label "append" -font $df \
  -command {start_copy_selected_files a 0} 
$w.type.copy add radiobutton -label "append and renumber" -font $df \
  -command {start_copy_selected_files a 1} 
$w.type.copy add radiobutton -label "combine parts" -font $df \
  -command {combine_parts} 
$w.type.copy add radiobutton -label help -font $df \
         -command {show_message_page $hlp_copy word}


button .abc.functions.console      -text console    -command {show_message_page $exec_out char} -font $df -borderwidth 3
#menubutton .abc.functions.find	   -text find	-relief raised -menu .abc.functions.find.type -font $df
#menu .abc.functions.find.type -tearoff 0
#.abc.functions.find.type add radiobutton -label "find title" -font $df  -command {find_window}
#.abc.functions.find.type add radiobutton -label "find bars"  -font $df  -command match_window


# second row of buttons

button .abc.auxfunc.toc            -text toc        -command show_titles_page  -font $df -borderwidth  3
menubutton .abc.auxfunc.find	   -text find	-relief raised -menu .abc.auxfunc.find.type -font $df -borderwidth  3
menu .abc.auxfunc.find.type -tearoff 0
.abc.auxfunc.find.type add radiobutton -label "find title" -font $df  -command {find_window}
.abc.auxfunc.find.type add radiobutton -label "find bars"  -font $df  -command match_window

set w .abc.auxfunc.midimenu
menubutton  $w  -text "abc2midi options" -relief raised -font $df -menu $w.type -borderwidth 3
menu $w.type -tearoff 0
$w.type add radiobutton -label tempo/pitch  -command {show_midi_page 1} -font $df
$w.type add radiobutton -label player       -command {show_midi_page 4} -font $df
$w.type add radiobutton -label bass       -command {show_midi_page 5} -font $df
$w.type add radiobutton -label chord      -command {show_midi_page 6} -font $df
$w.type add radiobutton -label melody       -command {show_midi_page 7} -font $df
$w.type add radiobutton -label gchord       -command {show_midi_page 10} -font $df
$w.type add radiobutton -label advanced     -command {show_midi_page 12} -font $df
$w.type add radiobutton -label voices       -command show_voice_page  -font $df


button .abc.auxfunc.yaps  -text "yaps" -command {show_ps_page yaps} -font $df -borderwidth  3
set w .abc.auxfunc.abc2ps
menubutton $w   -text "abc2ps options" -menu $w.type -font $df -relief raised -pady 6 -borderwidth  3
menu $w.type -tearoff 0
$w.type add radiobutton -label style  -command {show_ps_page psstyle} -font $df
$w.type add radiobutton -label format -command {show_ps_page psform}  -font $df

set w .abc.auxfunc.cfg
menubutton $w -text config -relief raised   -menu $w.type   -font $df -borderwidth 3
menu $w.type -tearoff 0
 $w.type add radiobutton -label "abc executables" -command {show_config_page 1} -font $df
 $w.type add radiobutton -label "player 1"        -command {show_config_page 2} -font $df
 $w.type add radiobutton -label "player 2"        -command {show_config_page 3} -font $df
 $w.type add radiobutton -label "font"            -command {show_config_page 4} -font $df

button .abc.auxfunc.help     -text help   -command contexthelp -font $df -borderwidth  3

pack .abc.functions.quit .abc.functions.play .abc.functions.disp \
.abc.functions.extr .abc.functions.editmenu -side left -fill y
pack .abc.functions.console   -fill y  -side left 

if {$midi(ps_creator) == "yaps"} {
  pack .abc.auxfunc.toc .abc.auxfunc.find .abc.auxfunc.midimenu \
   .abc.auxfunc.yaps .abc.auxfunc.cfg  -side left -fill y
} else {
  pack .abc.auxfunc.toc .abc.auxfunc.find .abc.auxfunc.midimenu \
   .abc.auxfunc.abc2ps .abc.auxfunc.cfg  -side left -fill y
  if {$midi(ps_creator) == "abcm2ps"} {
   .abc.auxfunc.abc2ps configure -text abcm2ps}
  }

pack .abc.auxfunc.help -side left -fill y

pack .abc.functions -side top 
pack .abc.auxfunc   -side top 


###    Table of Contents  - title index   ###

listbox .abc.titles.t -width 40 -borderwidth 2 -bg #f4ece0 \
  -selectmode extended -yscrollcommand {.abc.titles.ysbar set} \
  -font "$midi(font_family_toc) $midi(font_size) $midi(font_weight)"\
  -exportselection false
scrollbar .abc.titles.ysbar -bd 2 -command {.abc.titles.t yview}
pack .abc.titles.ysbar -side right -fill y 
pack .abc.titles.t  -expand y -fill both
pack .abc           -expand y -fill both
focus .abc.titles.t

if {[winfo exists .msg] != 0} {raise .msg .abc}

#####  Bindings  #####
bind . <Next>  { .abc.titles.t yview scroll +1 page }
bind . <Prior> { .abc.titles.t yview scroll -1 page }
bind . <Down>  { .abc.titles.t yview scroll +1 units }
bind . <Up>    { .abc.titles.t yview scroll -1 units }

bind . <d>     { if {$active_sheet == "titles"} {display_action} }
bind . <p>     { if {$active_sheet == "titles"} {play_action} }
bind . <space> { if {$active_sheet == "titles"} {play_action} }

#####  Functions to Play, Display, Edit #####

proc play_action {} {
global midi exec_out
global files

  set dir "[pwd]/$midi(midi_dir)"
  set cmd "file delete [glob -nocomplain $dir/X*.mid]"
  catch {eval $cmd}

  set sel [title_selected]
  set sel [tune_picked $sel $midi(abc_open)]
  set cmd "exec [list $midi(path_abc2midi)] [list $midi(midi_dir)/X.tmp]"
  catch {eval $cmd} exec_out
  set exec_out $cmd\n\n$exec_out
  set files ""
  foreach tune [lsort -integer $sel] {
    lappend files $dir/X$tune.mid
  }
  play_midis $sel
}


proc play_midis {sel} {
global midi exec_out files

set dir "[pwd]/$midi(midi_dir)"
if {$midi(player)} {
    # player 2 
    set cmd "exec [list $midi(alt_path_midiplay)] $midi(alt_midi_options) "
    switch -- $midi(transfer_prot_2) { 
      0 {set cmd [concat $cmd $dir/X[lindex $sel 0].mid]}
      1 {set cmd [concat $cmd $files]}
   }
} else {
    # player 1 
    set cmd "exec [list $midi(path_midiplay)] $midi(midiplay_options) "
    switch -- $midi(transfer_prot_1) { 
      0 {set cmd [concat $cmd $dir/X[lindex $sel 0].mid]}
      1 {set cmd [concat $cmd $files]}
   }
  }
set cmd [concat $cmd &]
set exec_out $exec_out\n\n$cmd
eval $cmd
}


set yaps_ptsize {612x792 792x1224 1224x792 612x1008 396x612 540x720 \
                 840x1189 594x840  419x594 727x1030 515x727 612x936 \
		 612x777  720x1008}


proc extractd_action {} {
global midi df yaps_ptsize exec_out type 
global extr_v
global extr_t
global extr_x
global infile
global outfile
set exec_out ""
set cmd "exec $midi(path_abc2abc) [list $midi(abc_open)] -e -r -b -t $extr_t -V $extr_v > $midi(midi_dir)/temp2.abc"
set exec_out $exec_out\n$cmd\n
catch {eval $cmd} exec_out1 
set exec_out $exec_out$exec_out1\n
if {$midi(ps_creator)=="abcm2ps" || $midi(ps_creator)=="abc2ps"} {
set type "Z"
} else {
set type "z"
}
abcmr
if {$midi(red_acc)} {
    set temp "temp4.abc"
    set infile "$midi(midi_dir)/temp3.abc"
    set outfile "$midi(midi_dir)/temp4.abc"
#    set cmd "exec redacc.tcl $midi(midi_dir)/temp3.abc > $midi(midi_dir)/temp4.abc"
#    set exec_out $exec_out\n$cmd\n
#    catch {eval $cmd} exec_out1 
#    set exec_out $exec_out$exec_out1\n
    abcra
} else { 
    set temp "temp3.abc"
}
if {$midi(ps_creator) == "yaps"} {
     # YAPS
     set M $midi(yaps_lmargin)x$midi(yaps_tmargin)
     set yapsopt "-e 1 -N -o Out.ps \
      -P [lindex $yaps_ptsize $midi(papersize)] -s $midi(yaps_scale) -M $M"
     if {$midi(yaps_landscape)} {append yapsopt " -l"}
     set cmd "exec [list $midi(path_yaps)] [list $midi(midi_dir)/$temp] $yapsopt"
#     puts $cmd
set exec_out $exec_out\n$cmd\n
     catch {eval $cmd} exec_out1 
set exec_out [concat $exec_out $exec_out1]

  } else {
    # abc2ps and abcm2ps
        switch -- $midi(ps_fmt_flag) {
      0  {set abc2psopt "-e 1  -s $midi(ps_scale) \
          -a $midi(ps_shrink) -m $midi(ps_lmargin) \
          -w $midi(ps_width)  -g $midi(ps_glue) \
          -d $midi(ps_staffsep)"}  
      1  {set abc2psopt "-p -e 1"}
      2  {set abc2psopt "-P -e 1"}
      3  {set abc2psopt "-F $midi(ps_fmt_file) -e 1"}
      4  {set abc2psopt "-l -e 1"}
    }
    if {$midi(ps_c)}     {append abc2psopt " -c"}
    if {$midi(ps_bhist)} {append abc2psopt " -n"}
    if {$midi(ps_bbar)}  {append abc2psopt " -k 1"}
    if {$midi(ps_bnumb)} {append abc2psopt " -N"}
    if {$midi(ps_bppage)} {append abc2psopt " -1"}
    if {$midi(ps_creator) == "abc2ps"} {
      if {[string length $midi(ps_maxvent)] > 0} {
         set abc2psopt [concat $abc2psopt -maxv $midi(ps_maxvent)]}
      if {[string length $midi(ps_maxsent)] > 0} {
         set abc2psopt [concat $abc2psopt -maxs $midi(ps_maxsent)]}
      }
    if {$midi(ps_creator) == "abcm2ps"} {
      if {$midi(ps_nolyric)} {
         set abc2psopt [concat $abc2psopt -M]}
      if {$midi(ps_noslur)} {
         set abc2psopt [concat $abc2psopt -G]}
      }

    if {$midi(ps_creator) == "abcm2ps"} {
      set cmd "exec [list $midi(path_abcm2ps)] \
         [list $midi(midi_dir)/$temp] $abc2psopt"} else {
      set cmd "exec [list $midi(path_abc2ps)] \
    	 [list $midi(midi_dir)/$temp] $abc2psopt -o"}

set exec_out $exec_out\n$cmd\n
catch {eval $cmd} exec_out1 
set exec_out $exec_out$exec_out1\n
    }
  set cmd "exec [list $midi(path_gs)] Out.ps &"
  set exec_out $exec_out\n\n$cmd\n
  catch {eval $cmd} exec_out1 
#  eval $cmd
  set exec_out $exec_out$exec_out1\n
  }

proc extracts_action {} {
global midi df exec_out types type
global extr_v
global extr_t
set sel [title_selected]
set exec_out ""
set cmd "exec $midi(path_abc2abc) $midi(midi_dir)/temp1.abc -t $extr_t -e > $midi(midi_dir)/temp2.abc"
set exec_out  $exec_out$cmd\n
catch {eval $cmd} exec_out1 
set exec_out $exec_out\n$exec_out1
if {$midi(ps_creator)=="abcm2ps"} {
set type "Z"
} else {
set type "z"
}
abcmr
if {$midi(red_acc)} {
    set cmd "exec redacc.tcl $midi(midi_dir)/temp3.abc > $midi(midi_dir)/temp4.abc"
    set exec_out $exec_out\n$cmd\n
    catch {eval $cmd} exec_out1 
    set exec_out $exec_out$exec_out1\n
    set temp "temp4.abc"
} else { 
    set temp "temp3.abc"
}

set exec_out $exec_out$exec_out1\n
   set filedir [file dirname $midi(abc_save)]
   set filename [tk_getSaveFile -initialdir $filedir -filetypes $types]
   if {$filename != ""} {
      set midi(abc_save) $filename
      foreach tune [lsort -integer $sel] {
        file rename -force tmp/$temp $filename
      }
   }

}

proc display_action {} {
global midi 
 
  set sel [title_selected]
  set xsel ""
  foreach i $sel {
      append xs "[lindex [.abc.titles.t get $i] 0],"
  }
  set xsel [string trimright $xs ,]
  display_tunes $midi(abc_open) $xsel
}


proc display_tunes {abcfile xsel} {
global midi 
global yaps_ptsize exec_out
  if {$midi(ps_creator) == "yaps"} {
     # YAPS
     set M $midi(yaps_lmargin)x$midi(yaps_tmargin)
     if {[string length $xsel] >0} {set yapsopt "-e $xsel "} else {set yapsopt ""}
     append yapsopt " -N -o Out.ps \
      -P [lindex $yaps_ptsize $midi(papersize)] -s $midi(yaps_scale) -M $M"
     if {$midi(yaps_voice)} {append yapsopt " -V"}
     if {$midi(yapsx)} {append yapsopt " -x"}
     if {$midi(yaps_landscape)} {append yapsopt " -l"}
     set cmd "exec [list $midi(path_yaps)] [list $abcfile] $yapsopt"
     catch {eval $cmd} exec_out 

  } else {
    # abc2ps and abcm2ps
    if {[string length $xsel] >0} {set abc2psopt "-e $xsel "} else {set abc2psopt ""}
    switch -- $midi(ps_fmt_flag) {
      0  {append abc2psopt " -s $midi(ps_scale) \
          -a $midi(ps_shrink) -m $midi(ps_lmargin) \
          -w $midi(ps_width)  \
          -d $midi(ps_staffsep)"}  
      1  {append abc2psopt " -p"}
      2  {append abc2psopt " -P"}
      3  {append abc2psopt " -F $midi(ps_fmt_file)"}
      4  {append abc2psopt " -l "}
    }

    if {$midi(ps_c)}     {append abc2psopt " -c"}
    if {$midi(ps_bxref)} {append abc2psopt " -x"}
    if {$midi(ps_bhist)} {append abc2psopt " -n"}
    if {$midi(ps_bbar)}  {append abc2psopt " -k 1"}
    if {$midi(ps_bnumb)} {append abc2psopt " -N"}
    if {$midi(ps_bppage)} {append abc2psopt " -1"}
    if {$midi(ps_creator) == "abc2ps"} {
      set abc2psopt [concat $abc2psopt " -g $midi(ps_glue)"]
      if {[string length $midi(ps_maxvent)] > 0} {
         set abc2psopt [concat $abc2psopt -maxv $midi(ps_maxvent)]}
      if {[string length $midi(ps_maxsent)] > 0} {
         set abc2psopt [concat $abc2psopt -maxs $midi(ps_maxsent)]}
      if {$midi(bpsvoice)} {append $abc2psopt " -V $midi(psvoice)"}
      }
    if {$midi(ps_creator) == "abcm2ps"} {
      if {$midi(ps_nolyric)} {
         set abc2psopt [concat $abc2psopt -M]}
      if {$midi(ps_noslur)} {
         set abc2psopt [concat $abc2psopt -G]}
      }

    if {$midi(ps_creator) == "abcm2ps"} {
      set cmd "exec [list $midi(path_abcm2ps)] \
         [list $abcfile] $abc2psopt"} else {
      set cmd "exec [list $midi(path_abc2ps)] \
         [list $abcfile] $abc2psopt -o"}

    catch {eval $cmd} exec_out 
    }
  set exec_out "$cmd\n\n$exec_out"
  set cmd "exec [list $midi(path_gs)] Out.ps &"
  set exec_out "$exec_out\n\n$cmd"
  eval $cmd
}


proc midisave {} {
global midi miditype

   set sel [title_selected]
   set sel [tune_picked $sel $midi(abc_open)]
   catch {exec $midi(path_abc2midi) tmp/X.tmp} exec_out
   set filedir [file dirname $midi(midi_save)]
   set filename [tk_getSaveFile -initialdir $filedir -filetypes $miditype]
   if {$filename != ""} {
      set midi(midi_save) $filename
      foreach tune [lsort -integer $sel] {
        file rename -force tmp/X$tune.mid $filename$tune.mid
      }
   }
}



    ################################
    #      Support Functions       # 
    ################################

proc update_history {openfile} {
global midi history_index

    #check if file is in history
    for {set i 0} {$i < $midi(history_length)} {incr i} {
      if {[string compare $midi(history$i) $openfile] ==  0} return
    } 

    if {$midi(history_length) == 0}  {
      .abc.file.menu.type add radiobutton  -value 0 \
        -variable history_index -command process_history 
    }

    # push history down open stack
    for {set i $midi(history_length)} {$i > 0} {incr i -1}  {
      set j [expr $i -1]
      set k [expr $i +1]
      set midi(history$i) $midi(history$j)
      if {$midi(history_length) < 6 && $i == $midi(history_length)} {
         .abc.file.menu.type add radiobutton  -label $midi(history$i) \
         -value $i -variable history_index -command process_history 
      } else {
        .abc.file.menu.type entryconfigure $k -label $midi(history$j)
      }
    }
    set midi(history0) $openfile
    .abc.file.menu.type entryconfigure 1 -label $midi(history0)
    if {$midi(history_length) < 6} {incr midi(history_length)}
}

proc process_history {} {
global midi history_index active_sheet

   set midi(abc_open) $midi(history$history_index)
   .abc.file.entry xview moveto 1.0   
   title_index $midi(abc_open)
   if {$active_sheet != "titles"} {show_titles_page}
   update
}

 
proc file_browser {} {
global midi types
global active_sheet

    set filedir [file dirname $midi(abc_open)]
    set openfile [tk_getOpenFile -initialdir $filedir \
        -filetypes $types]
    open_abc_file $openfile
} 


proc open_abc_file {filename} {
global midi active_sheet
    if {[string length $filename] > 0} {
       if {[string equal $filename $midi(abc_open)]} return
       set midi(abc_open) $filename
       .abc.file.entry xview moveto 1.0   
       title_index $midi(abc_open)
       update_history $filename
       if {$active_sheet != "titles"} {show_titles_page}
   }
}

#
# the function scans the entire abcfile making a list of titles and
# storing the file location of each tune. Code X: T: must begin on the
# first character position of a line.
#
proc title_index {abcfile} {
global fileseek midi 

    set srch X
    set i 0
    set pat {[0-9]+}
    .abc.titles.t delete 0 end
    set titlehandle [open $abcfile r]
    set fileseek(0) 0
    while {[gets $titlehandle line] >= 0} {
	if {!$midi(blank_lines) && [string length $line] < 1} {set srch X}
	switch -- $srch {
          X {if {[string compare -length 2 $line "X:"] == 0} {
	      regexp $pat $line  number
	      set srch T
	     } else {
	      set fileseek($i) [tell $titlehandle]}
	    }
	  T {
	    if {[string index $line 0] == "T" || [string index $line 0] == "P"} {
	     set name [string range $line 2 end]
	     set name [string trim $name]
	     set srch K
	     } 
	    }
	  K {
	    if {[string index $line 0] == "K"} {
	     set keysig [string range $line 2 end]
             set keysig [string trim $keysig]
	     set outline [format "%4s  %-6s %s" $number $keysig $name]
	     .abc.titles.t insert end $outline
	     incr i
	     set srch X
	     }
	    }
	  } 
    }  
    close $titlehandle
    if {$i < 12} {
      .abc.titles.t configure -height [expr $i + 1]
       set midi(abc_save) $midi(abc_open)
   } else {
      .abc.titles.t configure -height 12
       set midi(abc_save) edit.abc
   }
   .abc.titles.t selection set 0 
}


# returns position in list
proc title_selected {} {
  global globsel
  set index [.abc.titles.t curselection]
  if {$index == ""} {
     if {[info exist globsel] == 0} {
         show_message_page "can't read input abc file" word}
     puts "globsel = $globsel"
     set index $globsel
     foreach elem $index {
     .abc.titles.t selection set $elem 
    }
 } else {set globsel $index}
  return $index
}


proc get_nonblank_line {handle} {
  set line ""
  while {[string length $line] == 0 && [eof $handle] != 1} {
     gets $handle line
  }
  return $line 
}


# finds X: command or else returns nothing if eof
proc find_X_code {handle} {
  set line 1
  while {[string index $line 0] != "X" && [eof $handle] !=1} {
     set line [get_nonblank_line $handle]
  }
  return $line
}


proc get_title {handle} {
  set line 1
  while {[string index $line 0] != "T" && [string index $line 0] != "P" && [eof $handle] !=1} {
     set line [get_nonblank_line $handle]
  }
  return [string range $line 2 end]
}

  
#
# The function copies the selected tunes into an abc
# file called X.tmp. 
# After the X:n command, all the %%MIDI commands are
# sent as well as a Q: tempo # command. 
# Depending upon options any %%MIDI commands
# in the input file or any tempo commands will be
# ignored.  When a M: command is encountered the
# program may issue a %%MIDI gchord command.
#
# The function returns a list with the selected tune numbers
#
proc tune_picked {tunes abcfile} {
global fileseek 
global midi

  set inp_fd [open $abcfile r]
  set out_fd [open $midi(midi_dir)/X.tmp w]
  set pat {[0-9]+|C|C\|}
  set outlist ""
  
  foreach i $tunes {
    set loc $fileseek($i)
    seek $inp_fd $loc 

    for {set j 0} {$j < 17} {incr j} {set voice($j) 0}

    # The procedure expects to see an X command here. Otherwise it is
    # an error.
    set line [find_X_code $inp_fd] 
    if {[string index $line 0] == "X"} {
       puts $out_fd $line
       scan $line "X:%d" track
       lappend outlist $track
       write_midi_codes $out_fd
       puts $out_fd "Q:1/4 = $midi(tempo)"
       set line [get_nonblank_line $inp_fd]
    } else {
      title_index $midi(abc_open)
      return $outlist
    }
 
    while {[string length $line] > 0 && [string index $line 0] != "X"} {
       set code [string range $line 0 1]
       switch $code {
         M: {regexp $pat $line meter
             puts $out_fd $line
             if {$midi(bmychord)} {
               puts $out_fd "%%MIDI gchord $midi(mychord)"
             }
            }
         Q: {if {$midi(ignoreQ) == 0} {
                 puts $out_fd $line}
            }
	 %% {if {$midi(ignoremidi) == 0} {
                puts $out_fd $line}
            }
	 V: {puts $out_fd $line
	     scan $line "V:%d" vc
             if {$vc < 17} {
                if {$voice($vc) == 0} { 
                  puts $out_fd "%%MIDI program $midi(voice$vc)"
                  puts $out_fd "%%MIDI control 7 $midi(lvoice$vc)"
                  puts $out_fd "%%MIDI control 10 $midi(pvoice$vc)"
                  set voice($vc) 1
                  }
                }
             }
         default { puts $out_fd $line }
       } 
       set line [get_nonblank_line $inp_fd] 
     }
   puts $out_fd "\n"
   }
   close $inp_fd
   close $out_fd
   return $outlist
}


proc write_midi_codes {outfd} {
global midi

    set mid "%%MIDI"
    puts $outfd "$mid channel $midi(channel)"
    puts $outfd "$mid chordprog $midi(chordprog)"
    puts $outfd "$mid bassprog $midi(bassprog)"
    puts $outfd "$mid program $midi(program)"
    if {$midi(melody)} {
	puts $outfd "$mid beat $midi(beat_a) $midi(beat_b) \
	$midi(beat_c) $midi(beat_n)"
    } else {
	puts $outfd "$mid beat 0 0 0 4"
    }
    puts $outfd "$mid ratio $midi(ratio_n) $midi(ratio_m)"
    puts $outfd "$mid chordvol $midi(chordvol)"
    puts $outfd "$mid bassvol $midi(bassvol)"
    puts $outfd "$mid transpose $midi(transpose)"
    puts $outfd "$mid grace $midi(grace)"
    if {$midi(chords) == 0} {
	puts $outfd "$mid gchordoff"
    }
}



# Tcl Editor and Support Functions



# supertext.tcl v1.01a
# This is a modified version of Bryan Oakley's original supertext.
# There is a new command insert_bypass_undo which acts like
# a regular insert but does not update the undo list. It is used
# when you are loading a file into the text window and do not want
# to include the loading operation into the undo list.
#
# The change was introduced by Seymour.Shlien.crc.ca
# on November 25 2000
#
#
# supertext.tcl v1.01
#
# Copyright (c) 1998 Bryan Oakley
# All Rights Reserved
#
# this code is freely distributable, but is provided as-is with
# no waranty expressed or implied.

# send comments to oakley@channelpoint.com

# What is this?
# 
# This is a replacement for (or superset of , or subclass of, ...) 
# the tk text widget. Its big feature is that it supports unlimited
# undo. It also has two poorly documented options: -preproc and 
# -postproc. 

# The entry point to this widget is supertext::text; it takes all of
# the same arguments as the standard text widget and exhibits all of
# the same behaviors.  The proc supertext::overrideTextCommand may be
# called to have the supertext widget be used whenever the command
# "text" is used (ie: it imports supertext::text as the command "text"). 
# Use at your own risk...

# To access the undo feature, use ".widget undo". It will undo the
# most recent insertion or deletion. On windows and the mac
# this command is bound to <Control-z>; on unix it is bound to
# <Control-_>

# if you are lucky, you might find documentation here:
# http://www1.clearlight.com/~oakley/tcl/supertext.html

package provide supertext 1.01

namespace eval supertext {

    variable undo
    variable undoIndex
    variable text "::text"
    variable preProc
    variable postProc

    namespace export text
}

# this proc is probably attempting to be more clever than it should...
# When called, it will (*gasp*) rename the tk command "text" to "_text_", 
# then import our text command into the global scope. 
#
# Use at your own risk!
#
#proc supertext::overrideTextCommand {} {
#    variable text
#
#    set text "::_text_"
#    rename ::text $text
#    uplevel #0 namespace import supertext::text
#}

proc supertext::text {w args} {
    variable text
    variable undo
    variable undoIndex
    variable preProc
    variable postProc

    # this is what we will rename our widget proc to...
    set original __$w

    # do we have any of our custom options? If so, process them and 
    # strip them out before sending them to the real text command
    if {[set i [lsearch -exact $args "-preproc"]] >= 0} {
	set j [expr $i + 1]
	set preProc($original) [lindex $args $j]
	set args [lreplace $args $i $j]
    } else {
	set preProc($original) {}
    }

    if {[set i [lsearch -exact $args "-postproc"]] >= 0} {
	set j [expr $i + 1]
	set postProc($original) [lindex $args $j]
	set args [lreplace $args $i $j]
    } else {
	set postProc($original) {}
    }

    # let the text command create the widget...
    eval $text $w $args

    # now, rename the resultant widget proc so we can create our own
    rename ::$w $original

    # here's where we create our own widget proc.
    proc ::$w {command args} \
        "namespace eval supertext widgetproc $w $original \$command \$args"
    
    # set up platform-specific binding for undo; the only one I'm
    # really sure about is winders; the rest will stay the same for
    # now until someone has a better suggestion...
    switch $::tcl_platform(platform) {
	unix 		{event add <<Undo>> <Control-z>}
	windows 	{event add <<Undo>> <Control-z>}
	macintosh 	{event add <<Undo>> <Control-z>}
    }
    bind $w <<Undo>> "$w undo"

    set undo($original)	{}
    set undoIndex($original) -1
    set clones($original) {}

    return $w
}

# this is the command that we associate with a supertext widget. 
proc supertext::widgetproc {this w command args} {

    variable undo
    variable undoIndex
    variable preProc
    variable postProc

    # these will be the arguments to the pre and post procs
    set originalCommand $command
    set originalArgs $args

    # is there a pre-proc? If so, run it. If there is a problem,
    # die. This is potentially bad, because once there is a problem
    # in a preproc the user must fix the preproc -- there is no
    # way to unconfigure the preproc. Oh well. The other choice
    # is to ignore errors, but then how will the caller know if
    # the proc fails?
    if {[info exists preProc($w)] && $preProc($w) != ""} {
	if {[catch "$preProc($w) command args" error]} {
	    return -code error "error during processing of -preproc: $error"
	}
    }


    # if the command is "undo", we need to morph it into the appropriate
    # command for undoing the last item on the stack
    if {$command == "undo"} {

	if {$undoIndex($w) == ""} {
	    # ie: last command was anything _but_ an undo...
	    set undoIndex($w) [expr [llength $undo($w)] -1]
	}

	# if the index is pointing to a valid list element, 
	# lets undo it...
	if {$undoIndex($w) < 0} {
	    # nothing to undo...
	    bell

	} else {
	    
	    # data is a list comprised of a command token
	    # (i=insert, d=delete) and parameters related 
	    # to that token
	    set data [lindex $undo($w) $undoIndex($w)]

	    if {[lindex $data 0] == "d"} {
		set command "delete"
	    } else {
		set command "insert"
	    }
	    set args [lrange $data 1 end]

	    # adjust the index
	    incr undoIndex($w) -1

	}
    }

    # now, process the command (either the original one, or the morphed
    # undo command
    switch $command {

	configure {
	    # we have to deal with configure specially, since the
	    # user could try to configure the -preproc or -postproc
	    # options...
	    
	    if {[llength $args] == 0} {
		# first, the case where they just type "configure"; lets 
		# get it out of the way
		set list [$w configure]
		lappend list [list -preproc preproc Preproc {} $preProc($w)]
		lappend list [list -postproc postproc Postproc {} $postProc($w)]
		set result $list
		
		
	    } elseif {[llength $args] == 1} {
		# this means they are wanting specific configuration 
		# information
		set option [lindex $args 0]
		if {$option == "-preproc"} {
		    set result [list -preproc preproc Preproc {} $preProc($w)]

		} elseif {$option == "-postproc"} {
		    set result [list -postproc postproc Postproc {} $postProc($w)]
		    
		} else {
		    if {[catch "$w $command $args" result]} {
			regsub $w $result $this result
			return -code error $result
		    }
		}

	    } else {
		# ok, the user is actually configuring something... 
		# we'll deal with our special options first
		if {[set i [lsearch -exact $args "-preproc"]] >= 0} {
		    set j [expr $i + 1]
		    set preProc($w) [lindex $args $j]
		    set args [lreplace $args $i $j]
		    set result {}
		}

		if {[set i [lsearch -exact $args "-postproc"]] >= 0} {
		    set j [expr $i + 1]
		    set postProc($w) [lindex $args $j]
		    set args [lreplace $args $i $j]
		    set result {}
		}

		# now, process any remaining args
		if {[llength $args] > 0} {
		    if {[catch "$w $command $args" result]} {
			regsub $w $result $this result
			return -code error $result
		    }
		}
	    }
	}

	undo {
	    # if an undo command makes it to here, that means there 
	    # wasn't anything to undo; this effectively becomes a
	    # no-op
	    set result {}
	}

	insert {

	    if {[catch {set index  [text_index $w [lindex $args 0]]}]} {
		set index [lindex $args 0]
	    }

	    # since the insert command can have an arbitrary number
	    # of strings and possibly tags, we need to ferret that out
	    # now... what a pain!
	    set myargs [lrange $args 1 end]
	    set length 0
	    while {[llength $myargs] > 0} {
		incr length [string length [lindex $myargs 0]]
		if {[llength $myargs] > 1} {
		    # we have a tag...
		    set myargs [lrange $myargs 2 end]
		} else {
		    set myargs [lrange $myargs 1 end]
		}
	    }

	    # now, let the real widget command do the dirty work
	    # of inserting the text. If we fail, do some munging 
	    # of the error message so the right widget name appears...

	    if {[catch "$w $command $args" result]} {
		regsub $w $result $this result
		return -code error $result
	    }

	    # we need this for the undo stack; index2 couldn't be
	    # computed until after we inserted the data...
	    set index2 [text_index $w "$index + $length chars"]

	    if {$originalCommand == "undo"} {
		# let's do a "see" so what we just did is visible;
		# also, we'll move the insertion cursor to the end
		# of what we just did...
		$w see $index2
		$w mark set insert $index2
		
	    } else {
		# since the original command wasn't undo, we need
		# to reset the undoIndex. This means that the next
		# time an undo is called for we'll start at the 
		# end of the stack
		set undoIndex($w) ""
	    }

	    # add a delete command on the undo stack.
	    lappend undo($w) "d $index $index2"

	}

	delete {

	    # this converts the insertion index into an absolute address
	    set index [text_index $w [lindex $args 0]]

	    # lets get the data we are about to delete; we'll need
	    # it to be able to undo it (obviously. Duh.)
	    set data [eval $w get $args]

	    # add an insert on the undo stack
	    lappend undo($w) [list "i" $index $data]

	    if {$originalCommand == "undo"} {
		# let's do a "see" so what we just did is visible;
		# also, we'll move the insertion cursor to a suitable
		# spot
		$w see $index
		$w mark set insert $index

	    } else {
		# since the original command wasn't undo, we need
		# to reset the undoIndex. This means that the next
		# time an undo is called for we'll start at the 
		# end of the stack
		set undoIndex($w) ""
	    }

	    # let the real widget command do the actual deletion. If
	    # we fail, do some munging of the error message so the right
	    # widget name appears...
	    if {[catch "$w $command $args" result]} {
		regsub $w $result $this result
		return -code error $result
	    }
	}
	
        insert_bypass_undo {
            # does an insert without updating the undo stack; for 
            # example when you are loading a file into the text window.
	    if {[catch "$w insert $args" result]} {
		regsub $w $result $this result
		return -code error $result
	    }
        }


	default {
	    # if the command wasn't one of the special commands above,
	    # just pass it on to the real widget command as-is. If
	    # we fail, do some munging of the error message so the right
	    # widget name appears...
	    if {[catch "$w $command $args" result]} {
		regsub $w $result $this result
		return -code error $result
	    }
	}
    }

    # is there a post-proc? If so, run it. 
    if {[info exists postProc($w)] && $postProc($w) != ""} {
	if {[catch "$postProc($w) originalCommand originalArgs" error]} {
	    return -code error "error during processing of -postproc: $error"
	}
    }


    # we're outta here! (I think this is faster than a 
    # return, though I'm not 100% sure on this...)
    set result $result
}

# this returns a normalized index (ie: line.column), with special
# handling for the index "end"; to undo something we pretty much
# _have_ to have a precise row and column number.
proc supertext::text_index {w i} {
    if {$i == "end"} {
	set index [$w index "end-1c"]
    } else {
	set index [$w index $i]
    }

    return $index
}


proc copy_selection_to_file {tunes abcfile outfile} {
global fileseek midi

    if {[string compare [file tail $abcfile] [file tail $outfile]] == 0} return
    set edithandle [open $abcfile r]
    set outhandle [open $outfile w]

    foreach i $tunes {
	set loc $fileseek($i)
	seek $edithandle $loc 
	set line [find_X_code $edithandle] 
	puts $outhandle $line
	while {[string length $line] > 0 } {
	    set line  [get_nonblank_line $edithandle] 
	    if {[string index $line 0] == "X"} break;
	    puts $outhandle $line
	}
	puts $outhandle ""
    }

    close $edithandle
    close $outhandle
}




proc extract_title_of_first_tune {tunes abcfile} {
global fileseek midi

    set edithandle [open $abcfile r]
    set i [lindex $tunes 0]
    set loc $fileseek($i)
    seek $edithandle $loc
    set title [get_title $edithandle]
    close $edithandle 
    set title [string trimleft $title]
    if {[string length $title] <1} {set title notitle}
    set pat \[\"\/\\\*:\;\?\.\^\]
    regsub -all $pat $title "" result
    set title [string map {\040 _ \\ ""} $result]
    set title [string trimright $title]
}




proc edit_new_tune {} {
global midi

    set outfd [open $midi(abc_default_file) w]
    puts $outfd "X: 1"
    puts $outfd "T: Title"
    puts $outfd "C: Composer"
    puts $outfd "M: 2/4"
    puts $outfd "L: 1/8"
    puts $outfd "K: G"
    close $outfd
    abc_edit midi(abc_default_file)
}


proc edit_empty_file {} {
global midi
set outfd [open $midi(abc_default_file) w]
close $outfd
tcl_abc_edit $midi(abc_default_file)
}


proc start_copy_selected_files {access renumber} {
 global midi
 set sel [title_selected]
 if {$sel == -1} return
 copy_selected_files $sel $access $renumber
 }

proc copy_message {} {
set w .abc.copy
toplevel $w
focus $w
message $w.msg -text "The program will renumber the x ref sequentially \
starting from" -width 200
frame $w.fr
entry $w.fr.ent -width 5 -textvariable midi(startnumber)
button $w.fr.ok -text ok -command {destroy .abc.copy}
pack $w.msg -side top
pack $w.fr.ent $w.fr.ok -side left
pack $w.fr
grab $w
tkwait window $w
}


proc copy_selected_files {sel access renumber} {
#copies or appends all selected tunes to an output file
global fileseek midi
set types {{{abc files} {*.abc}}
           {{all} {*}}}
set filedir [file dirname $midi(abc_default_file)]
if {$renumber} copy_message 
set filename [tk_getSaveFile -initialdir $filedir \
      -filetypes $types]
if {[string length $filename] == 0} return
if {[string compare  $midi(abc_open) $filename] == 0} {
  tk_messageBox -message "do not even think of writing over the input file" \
          -type ok
   return
   }
set edithandle [open $midi(abc_open) r]
set outhandle [open $filename $access]
set n $midi(startnumber)
foreach i $sel {
  set loc $fileseek($i)
  seek $edithandle $loc 
  set line [find_X_code $edithandle] 
  if {$renumber} {puts $outhandle "X: $n"} else {puts $outhandle $line}
  incr n
  while {[string length $line] > 0 } {
     set line  [get_nonblank_line $edithandle] 
     if {[string index $line 0] == "X"} break;
     puts $outhandle $line
     }
  puts $outhandle "\n"
  }
close $edithandle
close $outhandle
}


# These are special procedures designed to take
# Laura Conrads renaissance music in her allparts.abc
# files and reformat them so each part is a separate voice
# instead of a separate tune. The procedure eliminates
# multiple copies of the lyrics, replaces the X:
# with a V: and eliminates and T: fields occurring
# in the other voices. Unfortunately, the functions
# do not do everything and some hand editing may
# necessary.

proc concat_parts {tunes abcfile outfile} {
global fileseek midi
   set body 0
   set first_tune 1
   set edithandle [open $midi(abc_open) r]
   set outhandle [open $outfile w]
   foreach i $tunes {

#   either put a X:n field or a V:n field for the tune depending
#   on whether this is the first selection or not. The variable
#   body signals the first time we reach the body of one of
#   the abc tunes.
        set loc $fileseek($i)
        seek $edithandle $loc
        set line [find_X_code $edithandle]
        if {$first_tune} {puts $outhandle $line} else {
          puts $outhandle V:[expr 1+$i]}

#   for the rest of the tune we strip out the lyrics (w:) and
#   title, if it is not the first part. For the first selection
#   we find the start of the body, (prior to K:) and insert a
#   V:1 field
#
        while {[string length $line] > 0 } {
            set line  [get_nonblank_line $edithandle]
            if {[string index $line 0] == "X"} break;
            if {$first_tune} {
#           first part
              if {[string index $line 0] == "K" && $body == 0} {
                puts $outhandle "V:1"
                set body 1}
              puts $outhandle $line} else   {
#           remaining parts
              if {[string index $line 0] == "w" || \
                  [string index $line 0] == "T" } {continue} else {
                puts $outhandle $line}
              }
        }
     set first_tune 0
     }
    puts $outhandle ""
    close $edithandle
    close $outhandle
}


proc combine_parts {} {
global midi
set sel [title_selected]
if {$sel == -1} return
set types {{{abc files} {*.abc}}
           {{all} {*}}}
set filedir [file dirname $midi(abc_default_file)]
set filename [tk_getSaveFile -initialdir $filedir \
      -filetypes $types]
concat_parts $sel $midi(abc_open) $filename 
}

proc abc_edit {varname} {
upvar #0 $varname abcfile
global midi

    if {[string length $midi(path_editor)] > 1} {
	set cmd "exec [list $midi(path_editor)] $abcfile &"
	eval $cmd
    } else {
	tcl_abc_edit $abcfile
    }
}

set hlp_editor \
"The program edits either the selected tune or the entire abc file. If \
your computer is slow and the abc file is very large, it is recommended\
that you limit your selection to one or a few tunes. The editor highlights\
the different abc components, (field commands like X:, comments and\
body) using different colour schemes. When doing inserts, some of the lines\
may not be coloured correctly.  You can restore the correct colours by\
clicking the clean/retag button. (If for some reason, the choice of colours is\
not ideal for your monitor, you may change them by editing the\
variables edit_body_colour, edit_field_colour etc in the runabc.ini\
file.)  The undo button or the <Cntl z> key will undo\
any of your changes. You have unlimited undo's provided there is sufficient\
memory in your computer.

When you edit selected tunes, the program makes a copy \
of these tunes in a temporary file specified by the variable abc_default_file \
in your runabc.ini file. (By default it is called edit.abc but you are able to\
change it by editing the runabc.ini file.) The file edit.abc is then renamed to\
a new file name based on the title of the first tune  and placed in your\
abc_work_folder directory also specified in runabc.ini. (Initially it is\
set to the name workfold.) Note that if a file of the same name already\
exists in that work folder, it will be overwrittn without warning.


The play button is designed to play a small \
portion of the file. You can either play a single text line positioned \
at your insert cursor or else you play a selected area. To designate a \
selected area,  hold the left mouse button down and sweep an \
area with your cursor in the edit window.  Avoid selecting an area \
starting with a line feed.  The tcl/tk script will search backwards \
for the X: and other field commands occurring before your highlighted \
area and also use this  information for creating the midi file.

It is recommended that you use a midi player which either \
automatically closes when the selection completes playing or one which \
stays open and automatically captures other selections when you click \
the play button again. On Microsoft Windows, I use either Winamp or \
the new Windows Media Player. On Unix, I use timidity without any \
user interface. If for some reason the play button does not produce \
audio output but normally works when you do not run the editor, check \
the console window or the X.tmp file to determine what has happened.

If you save over an existing abc file, it is a good idea to ensure \
that you have a backup of this file since I cannot guarantee the software \
is bug free.

The editor provides several  other features specificly designed for \
abc files. These are accessible through the menu items clean and tools \
for which there are other help messages. In particular there is a find \
tool for searching for particular strings in editor window starting from
the current cursor position. Enter the string in the entry box and search \
either forwards  or backwards. This search is case sensitive. To find \
a particular barline, enter its sequence number starting from 1 in the \
entry box and press the barline button. Barlines or double barlines are \
counted from the first X: reference number preceding the insert cursor \
position.



Text Widget standard binding.
<Button-1> Set the insert point, clear the selection, set focus.
<B1-Motion> Sweep out a selection from insert point.
<Shift-Button-1> Adjust the end of the selection closest to the mouse.
<Shift-Left> Move cursor and extend selection.
<Button-2> Paste the selection, or set the scrolling anchor.
<Button-2 motion> Scroll the window.
<Arrow keys> Shift insert point.
<Shift Arrow keys> Shift insert point and extend or clear selection.
<Cntl n> Shift insert point to next line.
<Cntl p> Shift insert point to previous line.
<Cntl f> Shift insert point one character to the right.
<Cntl b> Shift insert point one character to the left.
<Alt f>  Shift insert point one bar line to the left.
<Alt b>  Shift insert point one bar line to the right.
<Cntl d> Deletes character to right of the cursor.
<Cntl k> Deletes from cursor to end of the line.
<Cntl o> Inserts a new line but does not advance cursor.
<Cntl t> Transposes two characters.
<Cntl z> Undo last insert or delete. Unlimited undo's available.
<Cntl slash> Selects everything in the text widget
<Cntl backslash> Clears the selection.
<Shift arrow keys> Alter the selection.
<Delete> Delete selection if any, otherwise delete character to the right.
<Backspace> Delete selection if any, otherwise delete character to the left.

Many more bindings... see Tcl documentation for text widget. There is\
now unlimited undo's thanks to Bryan Oakley.

On Windows 95/98/00 you can use <cntl-c>, <cntl-x> <cntl-v> to copy, cut and \
paste the selections. For other systems use <cntl-y> instead of <cntl-v>. \
On some unix systems use <Cntl _> for undo."

set hlp_copy "The copy function will copy the selected tunes in the table \
of contents to a designated abc file. If the file does not already exist, \
it will create a new file; otherwise, it will destroy the existing file \
and overwrite it with the selected tunes. The X: numbers will be \
preserved.\n\nCopy and renumber will do the same as above but it will \
renumber the selected tunes increasing sequentially from a selected \
number.\n\n Append will append the selected tunes to an existing file, \
preserving the original numbering.\n\nAppend and renumber will do the \
same but renumber the selected tunes.\n\nThe copy combine parts function is \
specificly designed to reformat Laura Conrads renaissance music in \
her allparts.abc files. Each part is written as a separate tune rather \
than a separate voice. Therefore it is not possible to create a midi \
file with all parts playing simultaneously. This function combines all \
the parts into one tune, giving each part a separate voice. \
Duplicate titles, and lyrics are removed. When using this function, you must \
first select (highlight) all the tunes that you wish to combine. Like other \
copy functions, you will prompted for the name of an output file.\n\n\
Caution: do not try to copy over the source file already displayed in \
the table of contents."


proc tcl_abc_edit {abcfile} {
global midi df

    if {[winfo exists .abcedit] != 0} {
	destroy .abcedit
    }
    toplevel .abcedit 
    frame [set w .abcedit.func] 
    set midi(abc_save) $abcfile
    supertext::text .abcedit.t -width 80 -wrap none \
     -yscrollcommand {.abcedit.ysbar set} \
     -xscrollcommand {.abcedit.xsbar set} \
     -font "$midi(font_family_toc) $midi(font_size) $midi(font_weight)"
    entry .abcedit.file  -textvariable midi(abc_save) -width 100 -font $df

    menubutton $w.file -text file -font $df -menu $w.file.actions
    menu $w.file.actions -tearoff 0
    $w.file.actions add radiobutton -label save -font $df \
      -command {
       if {[string compare $midi(abc_save) $midi(abc_default_file)] == 0 \
           || [llength $midi(abc_save)] == 0} set_abc_save
       if {[llength $midi(abc_save)] >0} {Text_Dump .abcedit.t}
       }
    $w.file.actions add radiobutton -label "save as" -font $df \
      -command {
            set_abc_save
            if {[llength $midi(abc_save)] >0} {Text_Dump .abcedit.t}
            }
    $w.file.actions add radiobutton -label "quit" -font $df -command {destroy .abcedit}


    menubutton $w.clean -text clean -font $df \
      -menu $w.clean.type 
    menu $w.clean.type -tearoff 0
    $w.clean.type add radiobutton -label retag -font $df \
      -command {. config -cursor watch
                update
                tag_text
               . config -cursor arrow
               update}
    $w.clean.type add radiobutton -label "erase all" -font $df \
       -command erase_all
    $w.clean.type add radiobutton -label "remove redundant guitar chords" \
       -font $df -command {process_buffer remove_redundant_guitar_chords 0}
    $w.clean.type add radiobutton -label "remove all guitar chords" \
       -font $df -command {process_buffer remove_guitar_chords 0}
    $w.clean.type add radiobutton -label "remove all grace notes" \
       -font $df -command {process_buffer remove_grace_notes 0}
    $w.clean.type add radiobutton -label "help" -font $df\
         -command {show_message_page $hlp_clean word
                   focus .abc
                   raise .abc .abcedit
                  }

    menubutton  $w.play  -text play -font $df \
     -menu $w.play.type 
    menu $w.play.type -tearoff 0
    $w.play.type add radiobutton -label line -accelerator <Alt-p><Alt-l> \
        -command {play_from_edit_window line} -font $df
    $w.play.type add radiobutton -label selection -accelerator <Alt-p><Alt-s> \
        -command {play_from_edit_window sel} -font $df
    bind .abcedit <Alt-p><Alt-l> {play_from_edit_window line}
    bind .abcedit <Alt-p><Alt-s> {play_from_edit_window sel}

    button $w.undo -text undo -relief flat -font $df \
     -command {.abcedit.t undo}
    menubutton $w.tools -text tools -font $df -menu $w.tools.type
    menu $w.tools.type -tearoff 0
    $w.tools.type add cascade     -label transpose  -menu $w.tools.transpose -font $df
    $w.tools.type add cascade -label chords -menu $w.tools.chords -font $df
    $w.tools.type add radiobutton -label guitar -command guitar_chord -font $df
    $w.tools.type add radiobutton -label grace  -command grace_toolbar -font $df
    $w.tools.type add radiobutton -label drum   -command drum_editor -font $df

    menu $w.tools.transpose -tearoff 0
    $w.tools.transpose add radiobutton -label "octave up" -font $df\
         -command {process_buffer shift_all_notes 7}
    $w.tools.transpose add radiobutton -label "seventh up" -font $df\
         -command {process_buffer shift_all_notes 6}
    $w.tools.transpose add radiobutton -label "sixth up" -font $df\
         -command {process_buffer shift_all_notes 5}
    $w.tools.transpose add radiobutton -label "fifth up" -font $df\
         -command {process_buffer shift_all_notes 4}
    $w.tools.transpose add radiobutton -label "fourth up" -font $df\
         -command {process_buffer shift_all_notes 3}
    $w.tools.transpose add radiobutton -label "third up" -font $df\
         -command {process_buffer shift_all_notes 2}
    $w.tools.transpose add radiobutton -label "second up" -font $df\
         -command {process_buffer shift_all_notes 1}
    $w.tools.transpose add radiobutton -label "second down" -font $df\
         -command {process_buffer shift_all_notes -1}
    $w.tools.transpose add radiobutton -label "third down" -font $df\
         -command {process_buffer shift_all_notes -2}
    $w.tools.transpose add radiobutton -label "fourth down" -font $df\
         -command {process_buffer shift_all_notes -3}
    $w.tools.transpose add radiobutton -label "fifth down" -font $df\
         -command {process_buffer shift_all_notes -4}
    $w.tools.transpose add radiobutton -label "sixth down" -font $df\
         -command {process_buffer shift_all_notes -5}
    $w.tools.transpose add radiobutton -label "seventh down" -font $df\
         -command {process_buffer shift_all_notes -6}
    $w.tools.transpose add radiobutton -label "octave down" -font $df\
         -command {process_buffer shift_all_notes -7}
    $w.tools.transpose add radiobutton -label "help" -font $df\
         -command {show_message_page $hlp_transpose word
                   focus .abc
                   raise .abc .abcedit
                  }

    menu $w.tools.chords -tearoff 0
    $w.tools.chords add radiobutton -label "2nd" -font $df\
         -command {process_buffer notes2chords -1}
    $w.tools.chords add radiobutton -label "3rd" -font $df\
         -command {process_buffer notes2chords -2}
    $w.tools.chords add radiobutton -label "4th" -font $df\
         -command {process_buffer notes2chords -3}
    $w.tools.chords add radiobutton -label "5th" -font $df\
         -command {process_buffer notes2chords -2}
    $w.tools.chords add radiobutton -label "6th" -font $df\
         -command {process_buffer notes2chords -3}
    $w.tools.chords add radiobutton -label "help" -font $df\
         -command {show_message_page $hlp_chords word
                   focus .abc
                   raise .abc .abcedit
                  }
 
    button $w.find -text "find" -font $df -relief flat -command find_frame

    button $w.display -text "display" -font $df -relief flat -command "display_tunes  $abcfile \"\" "
 
    button $w.help  -text help -font $df -relief flat 
 
    scrollbar .abcedit.ysbar -orient vertical -command {.abcedit.t yview}
    scrollbar .abcedit.xsbar -orient horizontal -command {.abcedit.t xview}
    pack .abcedit.file -side top -anchor w
    pack $w.file $w.clean $w.undo $w.play \
      $w.find $w.tools $w.display $w.help -side left -anchor w
    pack $w -side top -anchor w
    pack .abcedit.ysbar -side right   -fill y -in .abcedit
    pack .abcedit.xsbar -side bottom  -fill x -in .abcedit
    pack .abcedit.t -fill both -expand y -in .abcedit
    set edit_handle [open $abcfile r]
    if {[eof $edit_handle] !=1} {gets $edit_handle line}
    .abcedit.t insert_bypass_undo end $line\n 
    while {[eof $edit_handle] != 1} {
	gets $edit_handle line
	.abcedit.t insert_bypass_undo end $line\n 
    }
    close $edit_handle

    
    bind .abcedit.func.help <Button> {show_message_page $hlp_editor word
                                      focus .abc
				      raise .abc .abcedit}
    bind .abcedit <Alt-f> shift_to_next_bar
    bind .abcedit <Alt-b> shift_to_previous_bar
    bind .abcedit <Alt-t> transfer_editor_buffer_to_abcmatch
    focus .abcedit
    tag_text
}


proc find_forwards {} {
global findstring
set point [.abcedit.t index insert]
set lastline [.abcedit.t index end]
if {[.abcedit.t compare $point >= $lastline]} {
  set point 1.0} else {
  set point "$point+1 chars"}
set point [.abcedit.t search -forwards $findstring $point]
if {[llength $point]>0} {
  .abcedit.t mark set insert $point
  .abcedit.t see $point
   focus .abcedit.t
   }
}


proc find_backwards {} {
global findstring
set point [.abcedit.t index insert]
set lastline [.abcedit.t index end]
if {[.abcedit.t compare $point >= $lastline]} {
   set point 1.0}
set point [.abcedit.t search -backwards $findstring $point]
if {[llength $point]>0} {
  .abcedit.t mark set insert $point
  .abcedit.t see $point
  focus .abcedit.t
  }
}


proc locate_barnum {} {
#returns the position of the findbarnum barline
global findbarnum body_start body_end
set p {\|+}
set point [.abcedit.t index insert]
set selstart [lindex [split $point .] 0]
if {$selstart > $body_end} {
   set point $body_start.0
   set selstart [lindex [split $point .] 0]
   }
set selstart $selstart.0
set start [.abcedit.t search -backwards X: $selstart]
set start [lindex [split $start .] 0]
set linend [.abcedit.t index "end -1 char"]
set linend [lindex [split $linend .] 0]
incr start
set bcount 1;
for {set index $start} {$index < $linend} {incr index} {
    set value [.abcedit.t get $index.0 $index.end]
    if {[string length $value] < 2} continue
    set initial [string index $value 0]
    set next [string index $value 1]
    if {[string first $initial ABCDEFGHIKLMNOPQRSTUVWwXZ]
           >= 0 && $next == ":" } {
              if {[string equal $initial X] == 1} break else continue
              }
    set begin 0
    set result 1
    while {$result} {
      set result [regexp -start $begin -indices $p $value loc]
      if {$result} {set begin [lindex $loc 1]
                   if {$bcount == $findbarnum} {return $index.$begin}
                   incr begin
                   incr bcount
                   }
      }
    }
return $index.$begin
}


proc find_barnum {} {
set point [locate_barnum]
.abcedit.t mark set insert $point
.abcedit.t see $point
focus .abcedit.t
}


proc find_frame {} {
global findstring df
global findbarnum
set findbarnum 1
set p .abcedit.find
if {[winfo exists .abcedit.find] == 0} {
   frame $p 
   entry $p.en -textvariable findstring -width 24 -font $df
   button $p.forwards -text forwards -font $df
   button $p.backwards -text backwards -font $df
   button $p.bar -text barline -font $df
   entry  $p.barnum -textvariable findbarnum -width 3 -font $df
   button $p.close -text close -font $df \
     -command {destroy .abcedit.find}
   pack $p.en $p.forwards $p.backwards $p.bar $p.barnum $p.close -side left -anchor w
   pack .abcedit.find -after .abcedit.func -anchor w
   bind .abcedit.find.en  <Return> find_forwards
   bind .abcedit.find.forwards <Button> find_forwards 
   bind .abcedit.find.backwards <Button> find_backwards 
   bind .abcedit.find.bar <Button> find_barnum
   } else {
  focus .abcedit.find
  }
}

proc shift_to_next_bar {} {
set nextbar [.abcedit.t search | insert]
.abcedit.t mark set insert "$nextbar+1 char"
}

proc shift_to_previous_bar {} {
set nextbar [.abcedit.t search -backwards | insert]
.abcedit.t mark set insert "$nextbar-1 char"
set nextbar [.abcedit.t search -backwards | insert]
.abcedit.t mark set insert "$nextbar+1 char"
}

proc erase_all {} {
.abcedit.t delete 1.0 end
}

proc set_abc_save {} {
global midi types
set filedir [file dirname $midi(abc_save)]
set midi(abc_save) [tk_getSaveFile -initialdir $filedir -filetypes $types]
}


proc Text_Dump {t {start 1.0} {end end}} {
global midi

  set outhandle [open $midi(abc_save) w]
  foreach {key value index} [$t dump $start $end] {
    if {$key == "text"} {
      puts -nonewline $outhandle $value}
    }
  close $outhandle
}


proc tag_text {} {
global body_start body_end midi
set t .abcedit.t
set body_start {}
$t configure -selectbackground $midi(edit_selectbackground)
$t tag delete "field" "body" "comment"
$t  tag configure "body"  -foreground $midi(edit_body_colour)
$t  tag configure "field" -foreground $midi(edit_field_colour) 
$t  tag configure "comment" -foreground $midi(edit_comment_colour)
set index [$t index "end -1 char"]
set lines [lindex [split $index .] 0]
  for {set index 1} {$index < $lines} {incr index} {
    set value [$t get $index.0 $index.end]
    if {[string length $value] < 2} continue
    set initial [string index $value 0]
    set next [string index $value 1]
    if {[string first $initial ABCDEFGHIKLMNOPQRSTUVWwXZ]
           >= 0 && $next == ":" } {
        $t tag add "field" $index.0 $index.end
      } elseif {$initial == "%"} {
        $t tag add "comment" $index.0 $index.end} else {
      $t tag add "body" $index.0 $index.end
      if {[llength $body_start] == 0} {set body_start $index}
      set body_end $index
      }
    }
}


proc play_from_edit_window {mode} {
# The function is designed to play a short extract from
# which has been selected from the editor. It searches
# for the X: code preceding the selected region and
# copies all the field commands preceding into the
# X.tmp file. The rest of the body is ignored and
# the selected portion of pasted in. abc2midi is
# executed to produce a midi file X1.mid which is then
# using the selected protocol played.
# We check that the user has selected a region or
# indicated an insert point in the body; otherwise
# we just play the first line in the body.
global midi exec_out files
global body_start body_end
set dir "[pwd]/$midi(midi_dir)"
set files ""
lappend files $dir/X1.mid
set cmd "file delete [glob -nocomplain $dir/X*.mid]"
catch {eval $cmd}
set out_fd [open $midi(midi_dir)/X.tmp w]

if {[info exists body_end] == 0} tag_text

if {[string compare $mode line] == 0} {
  set point [.abcedit.t index insert]
  set selstart [lindex [split $point .] 0]
  if {$selstart > $body_end} {
    set point $body_start.0
    set selstart [lindex [split $point .] 0]
    }
  set selend $selstart.end
  set selstart $selstart.0
  .abcedit.t tag add sel $selstart $selend 
  update
  } else { 
  set selrange [.abcedit.t tag ranges sel]
  if {[llength $selrange] < 2} {
    set selrange [list $body_start.0 $body_start.end]
    set selstart [lindex $selrange 0]
    set selend [lindex $selrange 1]
    .abcedit.t tag add sel $selstart $selend 
    update
    } else {
    set selstart [lindex $selrange 0]
    set selend [lindex $selrange 1]
    }
  }
set linend [lindex [split $selend .] 0]
set start [.abcedit.t search -backwards X: $selstart]
set start [lindex [split $start .] 0]
puts $out_fd "X: 1"
write_midi_codes $out_fd
for {set index $start} {$index < $linend} {incr index} {
    set value [.abcedit.t get $index.0 $index.end]
    set initial [string index $value 0]
    set next [string index $value 1]
    if {[string first $initial ABCDEFGHIKLMNOPQRSTUVWwXZ]
           > 0 && $next == ":" } {
       if {[string compare $initial "X"] != 0} {puts $out_fd $value} 
      } elseif {$initial == "%"} {
        puts $out_fd $value}  
    }
set value [.abcedit.t get $selstart $selend]
puts $out_fd $value
puts $out_fd "\n\n"
close $out_fd
set cmd "exec $midi(path_abc2midi) $midi(midi_dir)/X.tmp"
catch {eval $cmd} exec_out
set exec_out $cmd\n\n$exec_out
play_midis 1
}


#guitar.tcl

set keyorder CDEFGABCDEFGAB
global keyorder


proc setkeymap {sf} {
# work out accidentals to be applied to each note 
# sf; /* number of sharps in key signature -7 to +7 
global keymap
array set keymap {C C D D E E F F G G A A B B}
  if {$sf >= 1} {set keymap(F) F# } 
  if {$sf >= 2} {set keymap(C) C# } 
  if {$sf >= 3} {set keymap(G) G# }
  if {$sf >= 4} {set keymap(D) D# }
  if {$sf >= 5} {set keymap(A) A# }
  if {$sf >= 6} {set keymap(E) E# }
  if {$sf >= 7} {set keymap(B) B# }
  if {$sf <= -1} {set keymap(B) Bb }
  if {$sf <= -2} {set keymap(E) Eb } 
  if {$sf <= -3} {set keymap(A) Ab }
  if {$sf <= -4} {set keymap(D) Db } 
  if {$sf <= -5} {set keymap(G) Gb }
  if {$sf <= -6} {set keymap(C) Cb }
  if {$sf <= -7} {set keymap(F) Fb }
}


proc showkeys {from} {
global keyorder keymap
  for {set i 0} {$i < 8} {incr i} {
   set key [string index $keyorder [expr $i +$from]]
   set key $keymap($key)
   puts $key
   }
}


proc keytosf {keysig} {
#from key signature computes number of sharps/flats, mode
#and first key in scale.
global mode tonic
set s [regexp {([A-G])(#*|b*)(.*)} $keysig match tonic sub2 sub3]
if {$s < 1} {puts "can't understand key signature $keysig"
             return}
#puts "$tonic $sub2 $sub3"
set sf [expr [string first $tonic FCGDAEB] -1]
if {$sub2 == "#"} {set sf [expr $sf + 7]}
if {$sub2 == "b"} {set sf [expr $sf - 7]}
if {[string length $sub3] > 0} {set sub3 [string tolower $sub3]}
if {[string length $sub3] > 3} {set sub3 [string range $sub3 0 2 ]}
set mode [lsearch -exact "maj min m aeo loc ion dor phr lyd mix" $sub3]
if {$mode == -1} {set mode 0}
#puts "mode = $mode"
if {$mode >= 0} {set sf \
  [expr $sf - [lindex "0 3 3 3 5 0 2 4 -1 1" $mode]]}
#puts $sf
}

proc guitar_tool_bar {keysig} {
global df
global keyorder keymap
global tonic mode
# thanks to James Allwright for writing the original code in C
# (parsekey and event_key see parseabc.c and store.c).
set chordtype {0 1 1 0 0 1 1 0 1 1 0 0 1 1}
set modeloc {0 5 5 5 6 0 1 2 3 4}
set sf  [keytosf $keysig]
setkeymap $sf
set i1 [string first $tonic $keyorder]
set i2 [lindex $modeloc $mode]
if {[winfo exists .abcedit.guitar]} {destroy .abcedit.guitar}
frame .abcedit.guitar
pack .abcedit.guitar -after .abcedit.func -anchor w
for {set i 0} {$i < 7} {incr i} {
  set k [string index $keyorder [expr $i1+$i]]
  set majmin ""
  if {[lindex $chordtype [expr $i2+$i]] == 1} {set majmin m}
  set k $keymap($k)
  button .abcedit.guitar.$i -text $k$majmin -width 5 \
     -font $df -command "insert_chord $k$majmin"
#  puts "$k$majmin"
  }
  set g .abcedit.guitar
  button .abcedit.guitar.help -text help -width 5 -font $df\
   -command {show_message_page $hlp_guitar word
                   focus .abc
                   raise .abc .abcedit
                  }
  button .abcedit.guitar.close -text close -width 5 -font $df \
   -command {destroy .abcedit.guitar}
  grid $g.0 $g.1 $g.2 $g.3 $g.4 $g.5 $g.6 $g.help $g.close
}
   

proc insert_chord {chord} {
#puts or replaces a chord in the my selection editor text. 
#is there a guitar code already there?
set nextbar [.abcedit.t search | insert]
if {[info exists nextbar] == 0} {
 .abcedit.t insert insert "\"$chord\" "
 return}
#puts $nextbar
set chordbegin [.abcedit.t search \"  insert $nextbar]
#puts "chordbegin = $chordbegin"
if {$chordbegin == ""} {
 set lastinsert [.abcedit.t index insert]
 .abcedit.t insert insert "\"$chord\"" 
 .abcedit.t mark set insert $lastinsert
 return}
set chordend [.abcedit.t search \" "$chordbegin+1 char" $nextbar]
puts $chordend
.abcedit.t delete $chordbegin "$chordend +1 char"
.abcedit.t mark set insert $chordbegin
.abcedit.t insert insert "\"$chord\""
.abcedit.t mark set insert $chordbegin
}

set hlp_guitar "The tool bar assists you in inserting guitar chords \
(eg. \"Cm\") into the tune. When you create the tool bar, the program \
examines the preceding key signature and displays some of the guitar \
chords that are appropriate for this key signature. The program recognizes \
the major and minor key signatures as well as the various modes (eg. Ddor).\n\n\
Instructions: to insert a guitar chord, position the insert \
point of the editor where the chord is to be inserted \
and then click the button of the desired chord. To replace an existing guitar \
chord, position the insert before the guitar chord and click the button \
of the desired chord. You can also move the insert point one bar at a time \
using the <alt-f> and <alt-b> keys on your keyboard. You can listen to \
the chord by using the <alt-p>  <alt-l> keys.\n\n\Note: if the key signature \
changes and you wish to update the tool bar, merely position the insert \
marker after the key change (indicated by the K: field command), and \
click menu item tools/guitar again. The current guitar tool bar will be \
replaced with a new one."

#end of guitar.tcl


proc guitar_chord {} {
#identifies key signature and puts up appropriate chord toolbar.
global body_start body_end
set point [.abcedit.t index insert]
if {$point > $body_end} {set point $body_start.0}
set kfield [.abcedit.t search -backwards K: $point]
#puts $kfield
set kfield [.abcedit.t get $kfield  "$kfield lineend"]
#puts $kfield
set keysig [string range $kfield 2 end]
guitar_tool_bar $keysig
}


#transpose.tcl

set notekey "CDEFGABcdefgab"

proc shift_all_notes {line shift} {
#scans line for musical notes and shifts it up or down.
#In order to continue from where we left off, we strip
#off the part of string that we have already scanned,
#and keep track of the amount we stripped in the variable
#offset plus any expansion or contraction of note (eg. b to c').
  set note {[A-G]\,*|[a-g]\'*}
  set tmp $line
  set success 1
  set offset 0
  while {$success} {
    set success [regexp -indices $note $tmp match]
    if {!$success} break
    set pos1 [lindex $match 0]
    set pos2 [lindex $match 1]
    set key [string range $tmp  $pos1 $pos2]
    set newkey [shift_note $key $shift]
#    puts "$key $newkey" 
    set tmp [string range $tmp [expr $pos2 +1] end]
    set apos1 [expr $pos1 + $offset]
    set apos2 [expr $pos2 + $offset]
    set line [string replace $line $apos1 $apos2 $newkey]
    set offset [expr $offset + $pos1 + [string length $newkey]]
#    puts "$offset $pos1 $pos2"
    }
  return $line
  }
  

proc shift_note {note shift} {
# shift note by a given musical interval,up or down.
# 1 or -1 is second
# 2 or -2 is third...
global notekey
set n [string first [string index $note 0] $notekey]
set m [expr [string length $note] - 1]
if {$n < 7 && $m >0} {set m [expr -$m] }
set n [expr $n + $shift]
if {$n>13} {set n [expr $n - 7]
            incr m
  } elseif {$n<0} {set n [expr $n + 7]
           incr m -1
  }
if {$n < 7 && $m>0} {
            set n [expr $n + 7]
            incr m -1
   } elseif {$n > 6 && $m<0} {
            set n [expr $n - 7]
            incr m
   }
set result [string index $notekey $n]
if {$m > 0} {set suffix [string repeat "'" $m]
             set result $result$suffix
 } elseif {$m < 0} { set m [expr -$m]
                     set suffix [string repeat "," $m]
                     set result $result$suffix
                    }
return $result
}


set hlp_transpose "The purpose of this function is to assist you in \
creating a harmony line. First copy the music body and make it into \
a separate voice. Then select a section of the voice by highlighting \
it and then transpose it a certain number of intervals up or down. The \
colour of the transposed section will be turned to black to make \
it stand out. If you click the retag window it will be back to normal.\n\n\
Note this function is rather dumb and will change anything which \
looks like a musical note including comments, guitar chords etc. To get\
back to where you were before you need two undo's."

set hlp_clean "	Clean functions\n\n\
retag:   color the comments, field comments and body of the edit window.\n\
erase all:  clear the contents of the edit window.\n\n\
The next two function require you to first select a region in the \
body of the abc file (usually displayed as red).\n\
remove redundant guitar chords: repeated guitar chords are deleted\
in the selected area.\n\
remove all guitar chords: all guitar chords enclosed in double quotes \
are deleted in the selected area.\n\
remove all grace notes: all ornaments and grace notes enclosed in \
curly braces are removed.\n\n
Caution: guitar chords are recognized as pairs of double quotes. If your \
selected area begins in the middle of a guitar chord, the function will \
probably remove everything accept the guitar chords. The same also applies \
to grace notes"

proc remove_guitar_chords {line dummy} {
regsub -all  {"[^"]*"} $line "" result
return $result
}

proc remove_redundant_guitar_chords {line dummy} {
  set pat {"[^"]*"}
  set tmp $line
  set success 1
  set offset 0
  set oldguitarchord ""
  while {$success} {
    set success [regexp -indices $pat $tmp match]
    if {!$success} break
    set pos1 [lindex $match 0]
    set pos2 [lindex $match 1]
    set guitarchord [string range $tmp  $pos1 $pos2]
    puts $guitarchord 
    set tmp [string range $tmp [expr $pos2 +1] end]
    set apos1 [expr $pos1 + $offset]
    set apos2 [expr $pos2 + $offset]
    if {[string compare $guitarchord $oldguitarchord] == 0} {
       set line [string replace $line $apos1 $apos2 ""]
       set offset [expr $offset - [string length $guitarchord]]} 
   set offset [expr $offset + $pos2 +1]
   set oldguitarchord $guitarchord
   }
  return $line
}

proc process_buffer {action param} {
  set selrange [.abcedit.t tag ranges sel]
  if {[llength $selrange] < 2} {
     messages "Please select an area in the body of the tune \
and then try again. To select a region, hold the left mouse button \
down and sweep an area." 
     return
    } else {
    set selstart [lindex $selrange 0]
    set selend [lindex $selrange 1]
    set value [.abcedit.t get $selstart $selend]
    set cmd [list $action $value $param]
    set value [eval $cmd]
    .abcedit.t delete $selstart $selend
    .abcedit.t insert $selstart $value
    }
}
# end of transpose.tcl

# grace.tcl

proc grace_toolbar {} {
global df
if {[info exist .abcedit.grace]} {destroy .abcedit.grace}
set g .abcedit.grace
frame $g
button   $g.1 -text appoggiatura -font $df\
         -command {grace_note "1"}
button   $g.2 -text acciaccatura -font $df\
         -command {grace_note "-1"}
button   $g.3 -text mordent -font $df\
         -command {grace_note "0 1"}
button   $g.4 -text "reverse mordent" -font $df\
         -command {grace_note "0 -1"}
button   $g.5 -text slide -font $df\
         -command {grace_note "-3 -2 -1 0"}
button   $g.6 -text "trill 4" -font $df\
         -command {grace_note "0 1 0 1"}
button   $g.7 -text "trill 6" -font $df\
         -command {grace_note "0 1 0 1 0 1"}
button   $g.close -text close -width 5 -font $df \
   -command {destroy .abcedit.grace}
button   $g.help -text help -width 5 -font $df\
   -command {show_message_page $hlp_grace word
                   focus .abc
                   raise .abc .abcedit}
grid $g.1 $g.2 $g.3 $g.4 $g.5 $g.6 $g.7 $g.help $g.close
pack $g -after .abcedit.func -anchor w
}


proc make_grace {base sequence} {
set res ""
foreach action $sequence {
  set note [shift_note $base $action]
  set res $res$note
  }
return $res
}
  

proc grace_note {sequence} {
  set note {[A-G]|[a-g]}
  set loc [.abcedit.t search  -regexp  -- $note insert] 
#  puts "loc = $loc"
  set base [.abcedit.t get $loc]
# check for transpose
  set next [.abcedit.t index "$loc+1 char"]
#  puts "next = $next"
  set t [.abcedit.t get $next]
  if {$t == "," | $t == "'"} {set base $base$t}
  set grace [make_grace $base $sequence]
  .abcedit.t insert insert "\{$grace\}"
}

proc remove_grace_notes {line dummy} {
regsub -all  {\{[^\}]*\}} $line "" result
return $result
}
  

set hlp_grace "This toolbar is used for creating and inserting a grace \
note sequence before a specific note in the music body. For example, for \
the note C, clicking 'trill 4' will insert {DCDC} at the cursor position. \n\n\
Instructions: place the insert marker just before the note and click \
the desired ornament you wish to insert. If you change your mind, use \
<cntl-z> or undo to remove it.\n\nThe program looks at the following \
note and computes the grace sequence appropriate for this note. If the \
note is preceded by an accidental, it may be necessary for you to edit \
the grace sequence.\n\nCaution: if you are not careful in placing the \
insert marker, you can create a syntacticly meaningless abc file."


# end of grace.tcl


#source chords.tcl

proc notes2chords {line shift} {
#scans line for musical notes and shifts it up or down.
#In order to continue from where we left off, we strip
#off the part of string that we have already scanned,
#and keep track of the amount we stripped in the variable
#offset plus any expansion or contraction of note (eg. b to c').
  set note {(\^*|_*|=?)([A-G]\,*|[a-g]\'*)(/?[0-9]*|[0-9]*/*[0-9])}
  set tmp $line
  set success 1
  set offset 0
  while {$success} {
    set success [regexp -indices $note $tmp match]
    if {!$success} break
    set pos1 [lindex $match 0]
    set pos2 [lindex $match 1]
    set keystr [string range $tmp  $pos1 $pos2]
    regexp $note $keystr match acc key len
    #puts "$keystr $acc $key $len"
    set newkey [shift_note $key $shift]
    set chord "\[$acc$key$len$acc$newkey$len\]"
    #puts $chord
    set tmp [string range $tmp [expr $pos2 +1] end]
    set apos1 [expr $pos1 + $offset]
    set apos2 [expr $pos2 + $offset]
    set line [string replace $line $apos1 $apos2 $chord]
    set offset [expr $offset + $pos1 + [string length $chord]]
#    puts "$offset $pos1 $pos2"
    }
  return $line
  }
  
set hlp_chords "The function replaces the notes in the selected \
area with chords, going a specified interval down from the given note. \
Thus G2 would be replaced with \[G2E2\] if a third is requested.\
Like the transpose function, you should be careful that the selected \
area does not include anything else other than the musical notes. Also \
note that abc2midi does not handle the syntax \[AC\] > \[GB\] as you \
would expect. You should spell out C>B to C3/2B/ before applying the \
chord operator."


#end chords.tcl

proc show_tmpfile {} {
global midi df
set p .tmpfile
set num 0
if [winfo exist $p] {destroy $p}
toplevel $p
text $p.t -height 15 -width 80 -wrap char \
 -font $df -yscrollcommand ".tmpfile.ysbar set"
scrollbar $p.ysbar -orient vertical -command {.tmpfile.t yview}
pack $p.ysbar -side right -fill y -in $p 
pack $p.t -in $p -expand y -fill both
set handle [open $midi(midi_dir)/X.tmp]
while {[eof $handle] != 1} {
	gets $handle line
        incr num
	$p.t insert end "$num: $line\n"
    }
close $handle
}

proc run_abc2abc {} {
global midi df
global abc2abc_m  abc2abc_u  abc2abc_d  abc2abc_t abc2abc_n abc2abc_V
global abc2abc_e abc2abc_n_val  abc2abc_t_val abc2abc_V_val
set abc2abc_tofile 1
set abc2abc_opt ""
if {$abc2abc_m} {append abc2abc_opt "-m "}
if {$abc2abc_e} {append abc2abc_opt "-e "}
if {$abc2abc_u} {append abc2abc_opt "-u "}
if {$abc2abc_d} {append abc2abc_opt "-d "}
if {$abc2abc_t} {append abc2abc_opt "-t $abc2abc_t_val "}
if {$abc2abc_n} {append abc2abc_opt "-n $abc2abc_n_val "}
if {$abc2abc_V} {append abc2abc_opt "-V $abc2abc_V_val "}
copy_selection_to_file [title_selected] $midi(abc_open) $midi(midi_dir)/X.tmp
if {$midi(no_clipboard)} {
    set cmd "exec $midi(path_abc2abc) \
 $midi(midi_dir)/X.tmp $abc2abc_opt >$midi(abc_default_file)"
    catch {eval $cmd} exec_out
    tcl_abc_edit $midi(abc_default_file)
    .abc.abc2abc.10 configure -font $df -text "$cmd\n$exec_out"
    } else {
    set cmd "exec $midi(path_abc2abc) $midi(midi_dir)/X.tmp $abc2abc_opt"
    catch {eval $cmd} exec_out
    clipboard clear
    clipboard append $exec_out
    .abc.abc2abc.10 configure -font $df -text "$cmd\nThe results are in the clipboard."
    }
}


proc show_message_page {text wrapmode} {
global active_sheet df
remove_old_sheet
set p .abc.notice
if [winfo exist .abc.notice] {
   $p.t configure -state normal -font $df
   $p.t delete 1.0 end
   $p.t insert end $text
   $p.t configure -state disabled -wrap $wrapmode
   } else {
   frame $p
   text $p.t -height 15 -width 50 -wrap $wrapmode -font $df -yscrollcommand {.abc.notice.ysbar set}
   scrollbar $p.ysbar -orient vertical -command {.abc.notice.t yview}
   pack $p.ysbar -side right -fill y -in $p 
   pack $p.t -in $p
   $p.t insert end $text
   $p.t configure -state disabled
   }
set active_sheet notice
pack $p
}




    ######################
    #   Property Sheets  #
    ######################


set active_sheet none

proc remove_old_sheet {} {
global active_sheet
    switch -- $active_sheet {
      none   {pack forget .abc.titles}
      midi   {remove_midi_page} 
      voice  {pack forget .abc.voice}
      psstyle {pack forget .abc.psstyle}
      psform {pack forget .abc.psform}
      extract {remove_extract_page}
      yaps   {pack forget .abc.yaps}
      config {remove_cfg_page}
      titles {pack forget .abc.titles}
      notice {pack forget .abc.notice}
      abc2abc {pack forget .abc.abc2abc}
    }
}

proc show_ps_page {type} {
global active_sheet midi

   remove_old_sheet
    if {$active_sheet == $type} {
      set active_sheet "none"
   } else {
      if {$midi(ps_creator)=="yaps"} {
         pack .abc.yaps -side left
         set active_sheet yaps 
      } else {
         pack .abc.$type
         set active_sheet $type
         }
      }
}

proc show_abc2abc_page {} {
global active_sheet
remove_old_sheet
if {$active_sheet == "abc2abc"} {
  set active_sheet none
  }  else {
  pack .abc.abc2abc -side left
  set active_sheet abc2abc
  .abc.abc2abc.10 configure -text ""
  }
}


set midi_subsection 0

proc show_midi_page {subsection} {
global active_sheet midi_subsection

    remove_old_sheet
    if {$active_sheet == "midi" && $subsection == $midi_subsection} {
      set active_sheet "none"
      set midi_subsection 0
    } else {
      switch -- $subsection {
        1 { set w_list {1 2 3 4} }
        4 { set w_list {5} }
        5 { set w_list {6 10} }
        6 { set w_list {7 9} }
        7 { set w_list {8} }
       10 { set w_list {11} }
       12 { set w_list {12 13 14 15}}
	default { set w_list "" }
        }
      foreach i $w_list {
         pack .abc.midi1.$i -side top 
      }
      set midi_subsection $subsection
      pack .abc.midi1 
      set active_sheet midi 
    }
}


proc remove_midi_page {} {
global midi_subsection

  switch -- $midi_subsection {
    1 { set w_list {1 2 3 4} }
    4 { set w_list {5} }
    5 { set w_list {6 10} }
    6 { set w_list {7 9} }
    7 { set w_list {8} }
   10 { set w_list {11 } }
   12 { set w_list {12 13 14 15}}
    default { set w_list "" }
  }
  foreach i $w_list {
    pack forget .abc.midi1.$i 
  }
  pack forget .abc.midi1
}


proc show_voice_page {} {
global active_sheet

    remove_old_sheet
    if {$active_sheet == "voice"} {
      set active_sheet "none"
    } else {
      pack .abc.voice 
      set f .abc.voice.canvas.f 
      set child $f.pan1
      tkwait visibility $child
      set bbox [grid bbox $f 0 0]
#      puts $bbox
      set width [winfo reqwidth .abc]
      set height [winfo reqheight $f]
      set incr [lindex $bbox 3]
      .abc.voice.canvas config -scrollregion "0 0 $width $height"
      .abc.voice.canvas config -yscrollincrement $incr
      set height [expr $incr * 6]
      .abc.voice.canvas  config -width $width -height $height
      set active_sheet voice 
    }
}


proc show_titles_page {} {
global active_sheet

    remove_old_sheet
    if {$active_sheet == "titles"} {
       set active_sheet "none"
    } else {
       pack .abc.titles -expand y -fill both
       set active_sheet titles 
    }
}


set cfg_subsection 0

proc show_config_page {subsection} {
global active_sheet
global cfg_subsection

    remove_old_sheet
    if {$active_sheet == "config" && $subsection == $cfg_subsection} {
      set active_sheet "none"
      set cfg_subsection 0
    } else {
      switch -- $subsection {
        1 { set w_list {1 20 2 21 3 4 10 22 11 12} }
        2 { set w_list {5 6 13 19} }
        3 { set w_list {7 8 14 19} }
        4 { set w_list {9 18 15 16 17 } }
	default { set w_list "" }
        }
      foreach i $w_list {
         pack .abc.cfg.$i -side top 
      }
      set cfg_subsection $subsection
      pack .abc.cfg 
      set active_sheet config 
    }
}

proc show_extract_page {} {
global active_sheet

    remove_old_sheet
    if {$active_sheet == "extract"} {
      set active_sheet "none"
    } else {
	pack .abc.extract.1 .abc.extract.2 .abc.extract.3 -side top   
        pack .abc.extract 
        set active_sheet extract }
}


proc remove_cfg_page {} {
global cfg_subsection

  switch -- $cfg_subsection {
    1 { set w_list {1 20 2 21 3 4 10 22 11 12} }
    2 { set w_list {5 6 13 19} }
    3 { set w_list {7 8 14 19} }
    4 { set w_list {9 18 15 16 17 } }
    default { set w_list "" }
  }
  foreach i $w_list {
    pack forget .abc.cfg.$i 
  }
  pack forget .abc.cfg
}

proc remove_extract_page {} {
    pack forget .abc.extract.1
    pack forget .abc.extract.2     
    pack forget .abc.extract.3 
    pack forget .abc.extract
}

proc setpath {path_var} {
global midi

    set filedir [file dirname $midi($path_var)]
    set openfile [tk_getOpenFile -initialdir $filedir] 
    if {[string length $openfile] > 0} {
      set midi($path_var) $openfile
      update
    }
}


#	Abc2ps Property Sheet

set w .abc.psform
frame $w
label $w.heading -text "abc2ps/abcm2ps options" -font $df
grid $w.heading 

radiobutton $w.pretty1 -text "pretty 1" \
  -variable midi(ps_fmt_flag) -value 1 -command show_my_style -font $df
radiobutton $w.pretty2 -text "pretty 2" \
  -variable midi(ps_fmt_flag) -value 2 -command show_my_style -font $df
radiobutton $w.myown -text "use my own style" \
  -variable midi(ps_fmt_flag) -value 0  -command show_my_style -font $df
radiobutton $w.ulayout -text "use layout file" \
   -variable midi(ps_fmt_flag) -value 3 -command show_my_style -font $df
radiobutton $w.landscape -text "landscape" \
   -variable midi(ps_fmt_flag) -value 4 -command show_my_style -font $df
entry $w.layoutfile -width 16 -textvariable midi(ps_fmt_file) -font $df

label $w.glue -text glue  -font $df
set glmenu [tk_optionMenu $w.gluemenu midi(ps_glue) shrink space stretch fill]
$w.gluemenu configure -font $df
$glmenu configure -font $df
label $w.scalebut -text scale  -font $df
entry $w.scaleent -width 10 -relief sunken -textvariable midi(ps_scale) -font $df
label $w.widthbut -text width -font $df
entry $w.widthent -width 10 -relief sunken -textvariable midi(ps_width) -font $df
label $w.lmargbut -text "left margin" -font $df
entry $w.lmargent -width 10 -relief sunken -textvariable midi(ps_lmargin) -font $df
label $w.shrinkbut -text shrinkage -font $df
entry $w.shrinkent -width 10 -relief sunken -textvariable midi(ps_shrink) -font $df
label $w.staffseplab -text "staff separation" -font $df
entry $w.staffsepent -width 10 -relief sunken -textvariable midi(ps_staffsep) -font $df
grid $w.pretty1 $w.pretty2   -sticky w
grid $w.myown   $w.landscape -sticky w
grid $w.ulayout $w.layoutfile -sticky w


set w .abc.psstyle
frame $w
checkbutton $w.bref -text "reference numbers" -variable midi(ps_bxref) -font $df
checkbutton $w.bar  -text "number bar lines"  -variable midi(ps_bbar) -font $df
checkbutton $w.hist -text "show history"      -variable midi(ps_bhist) -font $df 
checkbutton $w.bnumb -text "page numbers"     -variable midi(ps_bnumb) -font $df
checkbutton $w.bppage -text "one tune/page"   -variable midi(ps_bppage) -font $df
checkbutton $w.nolyric -text "no lyrics"      -variable midi(ps_nolyric) -font $df
checkbutton $w.noslur -text "no slurs"        -variable midi(ps_noslur) -font $df

label $w.maxvlab -text "Maximum voices" -font $df
label $w.maxslab -text "Maximum symbols" -font $df
entry $w.maxvent -width 10 -relief sunken -textvariable midi(ps_maxvent) -font $df
entry $w.maxsent -width 10 -relief sunken -textvariable midi(ps_maxsent) -font $df
checkbutton $w.ps_c -text "ignore line ends" -variable midi(ps_c) -font $df
checkbutton $w.voices -text "select voices" -variable midi(bpsvoice) -font $df
entry $w.voicesent -width 10 -textvariable midi(psvoice) -font $df

grid $w.bref    $w.bar       -sticky w
grid $w.ps_c    $w.hist      -sticky w
grid $w.bnumb   $w.bppage    -sticky w
grid $w.nolyric $w.noslur    -sticky w
#grid $w.voices  $w.voicesent -sticky w
#grid $w.maxvlab $w.maxvent   -sticky w
#grid $w.maxslab $w.maxsent   -sticky w


#	extract property sheet

set w .abc.extract
frame $w
frame $w.1
frame $w.2
frame $w.3
set extr_v 1
set extr_t 0
set extr_x 1
label $w.1.0 -text "Tune:" -font $df
entry $w.1.1 -width 2 -relief sunken -textvariable extr_x -font $df
label $w.1.2 -text "Voice:" -font $df
entry $w.1.3 -width 2 -relief sunken -textvariable extr_v -font $df
label $w.1.4 -text "Transpose:" -font $df
entry $w.1.5 -width 2 -relief sunken -textvariable extr_t -font $df
pack $w.1.0 $w.1.1 $w.1.2 $w.1.3 $w.1.4 $w.1.5 -padx 3 -pady 3 -side left

checkbutton $w.2.0 -text "Reduce Accidentals" -variable midi(red_acc) -font $df
pack $w.2.0 -padx 3 -pady 3 -side left

button $w.3.0 -text "Display" -command extractd_action -font $df -borderwidth 3
button $w.3.1 -text "Save to file" -command extracts_action -font $df -borderwidth 3 
pack $w.3.0 $w.3.1  -padx 15 -pady 3 -side left 


#	yaps property sheet

set yaps_size {"Letter 8.5 x 11 in" "Tabloid 11 x 17 in" "Ledger 17 x 11 in" \
       "Legal 8.5 x 14 in" "Statement 5.5 x 8.5 in" "Executive 7.5 x 10 in" \
       "A3 297 x 420 mm" "A4 210 x 297 mm" "A5 148 x 210 mm" "B4 257 x 364 mm" \
       "B5 182 x 257 mm" "Folio 8.5 x 13 mm" "Quarto 8.5 x 10.8 in" \
       "10 x 14 10 x14 in"}

set w .abc.yaps
frame $w
label $w.heading -text "yaps options" -font $df
grid $w.heading  -sticky w
label $w.scalebut -text scale  -font $df
entry $w.scaleent -width 10 -relief sunken -textvariable midi(yaps_scale) -font $df
grid $w.scalebut $w.scaleent -sticky w
label $w.lmargbut -text "left margin" -font $df
entry $w.lmargent -width 10 -relief sunken -textvariable midi(yaps_lmargin) -font $df
label $w.lmargunits -text "points" -font $df
grid $w.lmargbut $w.lmargent $w.lmargunits -sticky w
label $w.tmargbut -text "top margin" -font $df
entry $w.tmargent -width 10 -relief sunken -textvariable midi(yaps_tmargin) -font $df
label $w.tmargunits -text "points" -font $df
grid $w.tmargbut $w.tmargent $w.tmargunits -sticky w
label $w.papersizelab -text "paper size" -font $df
set v $w.papersizemenu 
menubutton $v -text [lindex $yaps_size $midi(papersize)]   \
 -relief raised  -menu $v.type -font $df
menu $v.type -tearoff 0
set i 0
foreach item $yaps_size {
  $v.type add radiobutton -label $item -command "yaps_papersize $i" -font $df
  incr i
} 
grid $w.papersizelab $w.papersizemenu -sticky w
checkbutton $w.voicechk -text "separate voices" -font $df \
-variable midi(yaps_voice)

checkbutton $w.xchk -text "X tune number" -font $df \
-variable midi(yapsx)
grid $w.voicechk $w.xchk

checkbutton $w.land -text "Landscape" -font $df \
-variable midi(yaps_landscape)
grid $w.land -sticky w

proc yaps_papersize {size} {
global midi yaps_size df

    .abc.yaps.papersizemenu config -text [lindex $yaps_size $size] -font $df
    set midi(papersize) $size
}


proc show_my_style {} {
global midi

  set w .abc.psform
  if {$midi(ps_fmt_flag) == 0} {
     grid $w.scalebut $w.scaleent -sticky w
     grid $w.widthbut $w.widthent -sticky w
     grid $w.lmargbut $w.lmargent -sticky w
     grid $w.shrinkbut $w.shrinkent -sticky w
     grid $w.staffseplab $w.staffsepent -sticky w
     if {$midi(ps_creator) == "abc2ps"} {
       grid $w.glue $w.gluemenu -sticky w}
  } else {
     grid forget $w.scalebut $w.scaleent
     grid forget $w.widthbut $w.widthent
     grid forget $w.lmargbut $w.lmargent
     grid forget $w.shrinkbut $w.shrinkent
     grid forget $w.staffseplab $w.staffsepent
     grid forget $w.glue $w.gluemenu
  }
}

show_my_style

proc change_font {} {
global midi df
font configure $df -family $midi(font_family) -size $midi(font_size) \
     -weight $midi(font_weight) 
.abc.titles.t configure -font "$midi(font_family_toc) $midi(font_size) $midi(font_weight)"
}


#	Midi Property Sheet

set w .abc.midi1
frame $w
frame $w.1 
frame $w.2
frame $w.5

label $w.1.0 -text tempo -width 8 -font $df
scale $w.1.1 -from 0 -to 400 -length 240  \
  -width 10 -orient horizontal  -showvalue true \
  -variable midi(tempo)   -font $df
pack $w.1.0 $w.1.1  -side left

label $w.2.0 -text transpose -font $df
scale $w.2.1 -from -24 -to 24 -length 240 -width 10 \
   -orient horizontal  -showvalue true -variable midi(transpose)   -font $df
pack $w.2.0 $w.2.1 -side left 


checkbutton $w.3 -text "override tempo indications" \
  -variable midi(ignoreQ)  -font $df

checkbutton $w.4 -text "override all midi indications" \
  -variable midi(ignoremidi) -font $df

label $w.5.lab -text "midi player" -font $df
radiobutton $w.5.b0 -text "player 1"  -variable midi(player) \
	    -relief flat -value 0  -font $df
radiobutton $w.5.b1 -text "player 2"  -variable midi(player) \
	    -relief flat -value 1  -font $df
pack  $w.5.lab $w.5.b0 $w.5.b1 -side left

frame $w.6
label $w.6.basslab -text "bass volume"  -font $df
scale $w.6.basscal -from 0 -to 127 -length 200 -width 10 \
  -orient horizontal -showvalue true -variable midi(bassvol)  -font $df 
pack $w.6.basslab $w.6.basscal -side left

frame $w.7
label $w.7.chordlab -text "chord volume" -font  $df
scale $w.7.chordscal -from 0 -to 127 -length 200  -width 10\
  -orient horizontal -showvalue true -variable midi(chordvol)  -font $df
pack $w.7.chordlab $w.7.chordscal -side left

set melody 1
set chords 1

frame $w.8
frame $w.9
frame $w.10

checkbutton $w.8.melodychk -text "melody on" -variable midi(melody)  -font $df
checkbutton $w.9.chordchk -text "bass/chords on"  -variable midi(chords)  -font $df
checkbutton $w.10.basschk -text "bass/chords on"  -variable midi(chords)  -font $df


# program property page

set m(1) {"Acoustic Grand" "Bright Acoustic" "Electric Grand" \
"Honky-Tonk" "Electric Piano 1" "Electric Piano 2" Harpsichord \
Clav }

set m(2) {Celesta Glockenspiel "Music Box" Vibraphone Marimba \
Xylophone "Tubular Bells" Dulcimer}

set m(3) {"Drawbar Organ" "Percussive Organ" "Rock Organ" \
"Church Organ" "Reed Organ" Accordian Harmonica "Tango Accordian"}

set m(4) { "Acoustic Guitar (nylon)" "Acoustic Guitar (steel)" \
"Electric Guitar (jazz)" "Electric Guitar (clean)" \
 "Electric Guitar (muted)" "Overdriven Guitar" \
"Distortion Guitar" "Guitar Harmonics"} 

set m(5) {"Acoustic Bass" "Electric Bass (finger)" \
"Electric Bass (pick)" "Fretless Bass" "Slap Bass 1" \
"Slap Bass 2" "Synth Bass 1" "Synth Bass 2" }

set m(6) { Violin Viola Cello Contrabass "Tremolo Strings" \
"Pizzicato Strings" "Orchestral Strings" Timpani }

set m(7) { "String Ensemble 1" "String Ensemble 2" "SynthStrings 1" \
"SynthStrings 2" "Choir Aahs" "Voice Oohs" "Synth Voice" "Orchestra Hit" }

set m(8) { Trumpet Trombone Tuba "Muted Trumpet" "French Horn" \
"Brass Section" "SynthBrass 1" "SynthBrass 2"}

set m(9) { "Soprano Sax" "Alto Sax" "Tenor Sax" "Baritone Sax" \
 Oboe "English Horn" Bassoon Clarinet }

set m(10) { Piccolo Flute Recorder "Pan Flute" "Blown Bottle" \
Skakuhachi Whistle Ocarina }

set m(11) { "Lead 1 (square)" "Lead 2 (sawtooth)" "Lead 3 (calliope)" \
"Lead 4 (chiff)" "Lead 5 (charang)" "Lead 6 (voice)" \
"Lead 7 (fifths)" "Lead 8 (bass+lead)"} 

set m(12) { "Pad 1 (new age)" "Pad 2 (warm)" "Pad 3 (polysynth)" \
"Pad 4 (choir)" "Pad 5 (bowed)" "Pad 6 (metallic)" "Pad 7 (halo)" \
"Pad 8 (sweep)" }

set m(13) { "FX 1 (rain)" "(soundtrack)" "FX 3 (crystal)" \
"FX 4 (atmosphere)" "FX 5 (brightness)" "FX 6 (goblins)" \
"FX 7 (echoes)" "FX 8 (sci-fi)" }

set m(14) { Sitar Banjo Shamisen Koto Kalimba \
 Bagpipe Fiddle Shanai} 

set m(15) { "Tinkle Bell" Agogo "Steel Drums" Woodblock \
"Taiko Drum" "Melodic Tom" "Synth Drum" "Reverse Cymbal" }

set m(16) { "Guitar Fret Noise" "Breath Noise" Seashore \
"Bird Tweet" "Telephone ring" Helicopter Applause Gunshot }




set i1 [expr int(1 + $midi(program)/8)] 
set i2 [expr $midi(program)  % 8 ]
button $w.8.melodybut -text [lindex $m($i1) $i2]  -font $df
set i1 [expr int(1 + $midi(chordprog)/8)] 
set i2 [expr $midi(chordprog) % 8 ]
button $w.9.chordbut -text [lindex $m($i1) $i2]  -font $df
set i1 [expr int(1 + $midi(bassprog)/8)] 
set i2 [expr $midi(bassprog)  % 8 ]
button $w.10.bassbut -text [lindex $m($i1) $i2]  -font $df

pack $w.8.melodychk $w.8.melodybut -side left 
pack $w.9.chordchk $w.9.chordbut  -side left
pack $w.10.basschk $w.10.bassbut -side left 

if {$midi(melody) != 0} {$w.8.melodychk select}
if {$midi(chords) != 0} {$w.9.chordchk select; $w.10.basschk select}

frame $w.11
checkbutton $w.11.mychordchk -text "my gchord" -variable midi(bmychord) -font $df

set c2_4 {f2z2 fzfz f2f2 c2c2}
set c3_4 {f2z4 f2f2f2 c2c2c2}
set c6_8 {f3z3 f3f3 c3c3}
set c7_8 {f2z5 f2f2c3 f2f2f3 f3c2c2 f3f2f2}
set c4_4 {f4f4 c4c4}
set c9_8 {f2z6 f3f3f3 c3c3c3}
set c3_2 {f4c4c4 f4f4f4 f2z2c2z2c2z2 c4c4c4}

set w .abc.midi1.11.gchord
menubutton $w -text gchord -menu $w.type -relief raised -font $df
menu $w.type -tearoff 0
$w.type add cascade  -label "2/4" -menu $w.type.1 -font $df
$w.type add cascade  -label "3/4" -menu $w.type.2 -font $df
$w.type add cascade  -label "6/8" -menu $w.type.3 -font $df
$w.type add cascade  -label "7/8" -menu $w.type.4 -font $df
$w.type add cascade  -label "4/4" -menu $w.type.5 -font $df
$w.type add cascade  -label "9/8" -menu $w.type.6 -font $df
$w.type add cascade  -label "3/2" -menu $w.type.7 -font $df

set w .abc.midi1.11.gchord.type

menu $w.1 -tearoff 0
set i 0
foreach inst $c2_4 {
  $w.1 add radiobutton -label $inst -command "gchord_select 1 $i" -font $df
  incr i
}

menu $w.2 -tearoff 0
set i 0
foreach inst $c3_4 {
  $w.2 add radiobutton -label $inst -command "gchord_select 2 $i" -font $df
  incr i
}

menu $w.3 -tearoff 0
set i 0
foreach inst $c6_8 {
  $w.3 add radiobutton -label $inst -command "gchord_select 3 $i" -font $df
  incr i
}

menu $w.4 -tearoff 0
set i 0
foreach inst $c7_8 {
  $w.4 add radiobutton -label $inst -command "gchord_select 4 $i" -font $df
  incr i
}

menu $w.5 -tearoff 0
set i 0
foreach inst $c4_4 {
  $w.5 add radiobutton -label $inst -command "gchord_select 5 $i" -font $df
  incr i
}

menu $w.6 -tearoff 0
set i 0
foreach inst $c9_8 {
  $w.6 add radiobutton -label $inst -command "gchord_select 6 $i" -font $df
  incr i
}

menu $w.7 -tearoff 0
set i 0
foreach inst $c3_2 {
  $w.7 add radiobutton -label $inst -command "gchord_select 7 $i" -font $df
  incr i
}


proc gchord_select {p1 p2} {
global midi df

   set name [.abc.midi1.11.gchord.type.$p1 entrycget $p2 -label]
   .abc.midi1.11.gchord configure -text $name  -font $df
   set midi(mychord) $name
}

grid .abc.midi1.11.mychordchk .abc.midi1.11.gchord -sticky w


# advanced midi features
set w .abc.midi1.12
frame $w
label $w.beat -text "beat   a b c n" -font $df
entry $w.a -width 3 -textvariable midi(beat_a) -font $df
entry $w.b -width 3 -textvariable midi(beat_b) -font $df
entry $w.c -width 3 -textvariable midi(beat_c) -font $df
entry $w.n -width 3 -textvariable midi(beat_n) -font $df
pack $w.beat $w.a $w.b $w.c $w.n -side left 

set w .abc.midi1.13
frame $w
label $w.grace -text "grace a/b" -font $df
entry $w.graceval -width 5 -font $df -textvariable midi(grace)
pack  $w.grace $w.graceval -side left 

set w .abc.midi1.14
frame $w
label $w.lab -text "broken rhythm ratio" -font $df
entry $w.ent1 -width 1 -font $df -textvariable midi(ratio_n)
entry $w.ent2 -width 1 -font $df -textvariable midi(ratio_m)
pack $w.lab $w.ent1 $w.ent2 -side left -anchor w

set w .abc.midi1.15
frame $w
button $w.default -text default -font $df -command reset_advanced_midi
button $w.help    -text help    -font $df \
  -command {show_message_page $hlp_advancedmidi word}
pack $w.default $w.help -side left -anchor w

proc reset_advanced_midi {} {
global midi
set midi(beat_a) 127
set midi(beat_b) 120
set midi(beat_c) 90
set midi(beat_n) 4
set midi(grace) 1/2
set midi(ratio_n) 2
set midi(ratio_m) 1
}

set hlp_advancedmidi "Beat a b c n controls the way note velocities \
are selected. The first note in a bar has a velocity a. Other strong \
notes have velocity b and all the rest have velocity c. a, b, c must \
be in the range 0-127. The parameter n determines which notes are \
strong. If the time signature is x/y then each note is given a position \
0,1,2,..x-1. If k is a multiple of n, then the note is strong.\n\n\Grace\
sets the fraction of the next note that the grace notes will take \
up.\n\n
Broken rhythm ratio adjusts the times for > and < indications \
in the music body (eg. B > c). The default is 2 to 1, meaning the notes \
are played as B4/3 c2/3 even though it is printed as B3/2 c1/2. If you want \
it to be played as written then you should set the ratio to 3 to 1.\n\nThe\
default button resets all these values to their initial settings.\n\n
Comment: the beat feature does not seem to work correctly \
in abc2midi when guitar chords are running."

# voice Property Sheet

frame .abc.voice
canvas .abc.voice.canvas -width 80 -height 20 \
   -yscrollcommand [list .abc.voice.yscroll set]
scrollbar .abc.voice.yscroll -orient vertical \
   -command [list .abc.voice.canvas yview]
pack  .abc.voice.yscroll -side right -fill y
pack  .abc.voice.canvas -side left -fill both -expand true

set w [frame .abc.voice.canvas.f -bd 0]
.abc.voice.canvas create window 0 0 -anchor nw -window $w

for {set i 1} {$i <17} {incr i} {
   label $w.lab$i -text "voice $i" -font $df
   set i1 [expr int(1 + $midi(voice$i)/8)] 
   set i2 [expr $midi(voice$i) % 8 ]
   button $w.prog$i -text [lindex $m($i1) $i2]  -font $df
   eval {bind $w.prog$i <Button> [list voice_button $i %X %Y]}
   scale $w.pan$i -from 0 -to 127 -length 128  \
    -width 10 -orient horizontal  -showvalue true \
    -variable midi(pvoice$i)   -font $df -label "pan left/right"
   scale $w.vol$i -from 0 -to 127 -length 128  \
    -width 10 -orient horizontal  -showvalue true \
    -variable midi(lvoice$i)   -font $df -label "volume level"
}

proc voice_button {num X Y} {
global window chan
set window .abc.voice.canvas.f.prog$num
set chan voice$num
my_popup $X $Y}


grid   $w.lab1  $w.prog1 -sticky w
grid   $w.vol1  $w.pan1 -sticky w
grid   $w.lab2  $w.prog2 -sticky w
grid   $w.vol2  $w.pan2 -sticky w
grid   $w.lab3  $w.prog3 -sticky w
grid   $w.vol3  $w.pan3 -sticky w
grid   $w.lab4  $w.prog4 -sticky w
grid   $w.vol4  $w.pan4 -sticky w
grid   $w.lab5  $w.prog5 -sticky w
grid   $w.vol5  $w.pan5 -sticky w
grid   $w.lab6  $w.prog6 -sticky w
grid   $w.vol6  $w.pan6 -sticky w
grid   $w.lab7  $w.prog7 -sticky w
grid   $w.vol7  $w.pan7 -sticky w
grid   $w.lab8  $w.prog8 -sticky w
grid   $w.vol8  $w.pan8 -sticky w
grid   $w.lab9  $w.prog9 -sticky w
grid   $w.vol9  $w.pan9 -sticky w
grid   $w.lab10 $w.prog10 -sticky w
grid   $w.vol10 $w.pan10 -sticky w
grid   $w.lab11 $w.prog11 -sticky w
grid   $w.vol11 $w.pan11 -sticky w
grid   $w.lab12 $w.prog12 -sticky w
grid   $w.vol12 $w.pan12 -sticky w
grid   $w.lab13 $w.prog13 -sticky w
grid   $w.vol13 $w.pan13 -sticky w
grid   $w.lab14 $w.prog14 -sticky w
grid   $w.vol14 $w.pan14 -sticky w
grid   $w.lab15 $w.prog15 -sticky w
grid   $w.vol15 $w.pan15 -sticky w
grid   $w.lab16 $w.prog16 -sticky w
grid   $w.vol16 $w.pan16 -sticky w



proc my_popup {rootx rooty} {
global m df

   if {![winfo exists .patchmap]} {
      set instrum_family {piano "chrom percussion" organ guitar bass \
      strings ensemble brass reed pipe "synth lead" "synth pad" \
      "synth effects" ethnic percussive "sound effects"}
      set w .patchmap
      menu $w -tearoff 0
      set i 1
      foreach class $instrum_family {
        $w add cascade  -label $class -menu $w.$i -font $df
        set w2 .patchmap.$i
        menu $w2 -tearoff 0
        set j 0
        foreach inst $m($i) {
          $w2 add radiobutton -label $inst \
              -command "program_select  $i $j " -font $df
          incr j
          }
        incr i
      }
   }
   tk_popup .patchmap $rootx $rooty 
}

proc program_select {p1 p2} {
global midi chan window

#  puts "program_select $p1 $p2"
  set midi($chan) [expr ($p1-1)*8 + $p2]
  set name [.patchmap.$p1 entrycget $p2 -label]
  $window configure -text $name 
}


bind .abc.midi1.8.melodybut <Button> {set window .abc.midi1.8.melodybut
  set chan program
  my_popup %X %Y} 
bind .abc.midi1.9.chordbut <Button> {set window .abc.midi1.9.chordbut
  set chan chordprog
  my_popup %X %Y} 
bind .abc.midi1.10.bassbut <Button> {set window .abc.midi1.10.bassbut
  set chan bassprog 
  my_popup %X %Y} 


# abc2abc property page
set abc2abc_e 1
set w .abc.abc2abc
frame $w
frame $w.1
checkbutton $w.1.1 -text "new spacing" -variable abc2abc_m  -font $df
checkbutton $w.1.2 -text "no error report" -variable abc2abc_e -font $df
pack $w.1.1 $w.1.2 -anchor w -side left
checkbutton $w.2 -text "use [] () for chords and slurs" -font $df -variable abc2abc_u
checkbutton $w.3 -text "double note length" -font $df -variable abc2abc_d
checkbutton $w.4 -text "halve note length" -font $df -variable abc2abc_v
frame $w.5
checkbutton $w.5.1 -text "transpose" -font $df -variable abc2abc_t
entry $w.5.2 -width 2 -textvariable abc2abc_t_val -font $df
label $w.5.3 -text semitones -font $df
pack $w.5.1 $w.5.2 $w.5.3 -side left
frame $w.6
checkbutton $w.6.1 -text "linebreaks every " -font $df -variable abc2abc_n
entry $w.6.2 -width 2 -textvariable abc2abc_n_val
label $w.6.3 -text "bars" -font $df
pack $w.6.1 $w.6.2 $w.6.3 -side left
frame $w.7
checkbutton $w.7.1 -text "extract only voice " -font $df -variable abc2abc_V
entry $w.7.2 -width 2 -textvariable abc2abc_V_val -font $df
pack $w.7.1 $w.7.2 -side left
frame $w.8
radiobutton $w.8.0 -text "output to clipboard"  -variable midi(no_clipboard) \
	    -relief flat -value 0  -font $df 
radiobutton $w.8.1 -text "output to editor"     -variable midi(no_clipboard) \
	    -relief flat -value 1  -font $df
pack $w.8.0 $w.8.1 -side left
frame $w.9
button $w.9.1 -text "abc2abc" -font $df -command run_abc2abc
button $w.9.2 -text "help" -font $df -command contexthelp 
pack $w.9.1 $w.9.2 -side left
label $w.10 -text "" -font $df
pack $w.1 $w.2 $w.3 $w.4 $w.5 $w.6 $w.7 $w.8 $w.9 $w.10 -side top -anchor w




# Config property page

set w .abc.cfg
frame $w
for {set i 1} {$i <= 22} {incr i} {
  frame $w.$i
}
button $w.1.0 -text abc2ps -width 14 -command {setpath path_abc2ps}  -font $df
entry $w.1.1 -width 30 -relief sunken -textvariable midi(path_abc2ps) -font $df
pack $w.1.0 $w.1.1  -side left

button $w.20.0 -text abcm2ps -width 14 -command {setpath path_abcm2ps}  -font $df
entry $w.20.1 -width 30 -relief sunken -textvariable midi(path_abcm2ps) -font $df
pack $w.20.0 $w.20.1  -side left

button $w.21.0 -text abc2abc -width 14 -command {setpath path_abc2abc}  -font $df
entry $w.21.1 -width 30 -relief sunken -textvariable midi(path_abc2abc) -font $df
pack $w.21.0 $w.21.1  -side left

button $w.22.0 -text abcmatch -width 14 -command {setpath path_abcmatch}  -font $df
entry $w.22.1 -width 30 -relief sunken -textvariable midi(path_abcmatch) -font $df
pack $w.22.0 $w.22.1  -side left

button $w.2.0 -text yaps -width 14  -command {setpath path_yaps} -font $df
entry $w.2.1 -width 30 -relief sunken -textvariable midi(path_yaps) -font $df
pack $w.2.0 $w.2.1  -side left

button $w.3.0 -text abc2midi -width 14  -command {setpath path_abc2midi} -font $df
entry $w.3.1 -width 30 -relief sunken -textvariable midi(path_abc2midi) -font $df
pack $w.3.0 $w.3.1  -side left

button $w.4.0 -text ghostview -width 14  -command {setpath path_gs} -font $df
entry $w.4.1 -width 30 -relief sunken -textvariable midi(path_gs) -font $df
pack $w.4.0 $w.4.1  -side left

button $w.5.0 -text "midiplayer 1" -width 14  -command {setpath path_midiplay} -font $df
entry $w.5.1 -width 36 -relief sunken -textvariable midi(path_midiplay) -font $df
pack $w.5.0 $w.5.1  -side left

button $w.6.0 -text "player 1 opts" -width 14 -command {}  -font $df
entry $w.6.1 -width 36 -relief sunken -textvariable midi(midiplay_options) -font $df
pack $w.6.0 $w.6.1  -side left

button $w.7.0 -text "midiplayer 2" -width 14 -command {setpath alt_path_midiplay} -font $df
entry $w.7.1 -width 36 -relief sunken -textvariable midi(alt_path_midiplay) -font $df
pack $w.7.0 $w.7.1  -side left

button $w.8.0 -text "player 2 options" -width 14 -command {} -font $df
entry $w.8.1 -width 36 -relief sunken -textvariable midi(alt_midi_options) -font $df
pack $w.8.0 $w.8.1  -side left

label $w.9.0 -text "gui font" -width 10  -font $df
entry $w.9.1 -width 24 -relief sunken -textvariable midi(font_family) -font $df
pack $w.9.0 $w.9.1 -side left

button $w.10.0 -text editor -width 14  -command {setpath path_editor} -font $df
entry $w.10.1 -width 30 -relief sunken -textvariable midi(path_editor) -font $df
pack $w.10.0 $w.10.1  -side left

checkbutton $w.11.0 -text "ignore blank lines" -variable midi(blank_lines)  -font $df
pack $w.11.0 -side left

label $w.12.lab -text use -font $df
radiobutton $w.12.b0 -text abc2ps  -variable midi(ps_creator) \
	    -relief flat -value abc2ps  -font $df -command switch_ps_button
radiobutton $w.12.b1 -text abcm2ps  -variable midi(ps_creator) \
	    -relief flat -value abcm2ps  -font $df -command switch_ps_button
radiobutton $w.12.b2 -text yaps  -variable midi(ps_creator) \
	    -relief flat -value yaps  -font $df -command switch_ps_button
pack $w.12.lab $w.12.b0 $w.12.b1 $w.12.b2 -side left


label $w.13.lab -text "transfer protocol 1" -font $df
radiobutton $w.13.b0 -text "one file"  -variable midi(transfer_prot_1) \
	    -relief flat -value 0  -font $df \
            -command {protocol_message 0}
radiobutton $w.13.b1 -text "file list"  -variable midi(transfer_prot_1) \
	    -relief flat -value 1  -font $df \
            -command {protocol_message 1}
radiobutton $w.13.b2 -text "folder"  -variable midi(transfer_prot_1) \
	    -relief flat -value 2  -font $df \
            -command {protocol_message 2}
pack $w.13.lab $w.13.b0 $w.13.b1 $w.13.b2 -side left

label $w.14.lab -text "transfer protocol 2" -font $df
radiobutton $w.14.b0 -text "one file"  -variable midi(transfer_prot_2) \
	    -relief flat -value 0  -font $df \
            -command {protocol_message 0}
radiobutton $w.14.b1 -text "file list"  -variable midi(transfer_prot_2) \
	    -relief flat -value 1  -font $df \
            -command {protocol_message 1}
radiobutton $w.14.b2 -text "folder"  -variable midi(transfer_prot_2) \
	    -relief flat -value 2  -font $df \
            -command {protocol_message 2}
pack $w.14.lab $w.14.b0 $w.14.b1 $w.14.b2 -side left

label $w.15.lab -text size -font $df
radiobutton $w.15.b0 -text 7  -variable midi(font_size) \
	    -relief flat -value 7  -command change_font -font $df
radiobutton $w.15.b1 -text 8  -variable midi(font_size) \
	    -relief flat -value 8  -command change_font -font $df
radiobutton $w.15.b2 -text 10  -variable midi(font_size) \
	    -relief flat -value 10  -command change_font -font $df
radiobutton $w.15.b3 -text 12  -variable midi(font_size) \
	    -relief flat -value 12  -command change_font -font $df
radiobutton $w.15.b4 -text 15  -variable midi(font_size) \
	    -relief flat -value 15  -command change_font -font $df
radiobutton $w.15.b5 -text 18  -variable midi(font_size) \
	    -relief flat -value 18  -command change_font -font $df
pack $w.15.lab $w.15.b0 $w.15.b1 $w.15.b2 $w.15.b3 $w.15.b4 $w.15.b5 -side left

label $w.16.lab -text weight -font $df
radiobutton $w.16.b1 -text normal -font $df -variable midi(font_weight) -value normal -command change_font
radiobutton $w.16.b2 -text bold   -font $df -variable midi(font_weight) -value bold   -command change_font
pack $w.16.lab $w.16.b1 $w.16.b2 -side left

button $w.17.1 -text apply -command change_font -font $df
pack $w.17.1 

label $w.18.0 -text "toc font" -width 10  -font $df
entry $w.18.1 -width 24 -relief sunken -textvariable midi(font_family_toc) -font $df
pack $w.18.0 $w.18.1 -side left

message $w.19.0 -aspect 300 -font $df
pack $w.19.0


proc protocol_message {value} {
global midi
if {$value == 2} {
 .abc.cfg.19.0 config -text "The options entry box should contain the path \
to the folder [pwd]/$midi(midi_dir) or \$dir for short. If you change the protocol \
back to the other modes, you should delete this path from the options entry box. If \
there are any problems during play, press the console button to view the messages \
sent to the midi player. It is necessary to do this manually in order to accommodate \
most midi players."} else {.abc.cfg.19.0  config -text ""}
} 

proc switch_ps_button {} {
global midi
if {$midi(ps_creator)=="yaps"} {
   pack forget .abc.auxfunc.abc2ps
   pack .abc.auxfunc.yaps -after .abc.auxfunc.midimenu -side left
   } else {
   pack forget .abc.auxfunc.yaps 
   pack .abc.auxfunc.abc2ps -after .abc.auxfunc.midimenu -side left
   if {$midi(ps_creator)=="abcm2ps"} {
     .abc.auxfunc.abc2ps config -text abcm2ps
     grid forget .abc.psstyle.maxvlab .abc.psstyle.maxvent
     grid forget .abc.psstyle.maxslab .abc.psstyle.maxsent
     grid forget .abc.psstyle.voices  .abc.psstyle.voicesent
     grid .abc.psstyle.nolyric .abc.psstyle.noslur -sticky w
     if {$midi(ps_fmt_flag) == 0} {
        grid forget .abc.psform.glue .abc.psform.gluemenu}
     } else {
     .abc.auxfunc.abc2ps config -text abc2ps
     grid forget .abc.psstyle.nolyric .abc.psstyle.noslur
     grid .abc.psstyle.voices .abc.psstyle.voicesent -sticky w
     grid .abc.psstyle.maxvlab .abc.psstyle.maxvent  -sticky w
     grid .abc.psstyle.maxslab .abc.psstyle.maxsent  -sticky w
     if {$midi(ps_fmt_flag) == 0} {
        grid .abc.psform.glue .abc.psform.gluemenu -sticky w}
     }
   }
}

set hlp_overview "You are running runabc.tcl version $runabc_version $runabc_date\n\n\
This is a graphical user interface to abc2ps, yaps, abc2midi, a \
PostScript file viewer and midi file player. Either yaps \
or abc2ps is required. If you are running \
the program for the first time, you need to specify the path name \
to these executables in the configuration property sheet. \
Click the config button and then the help button for further instructions.\n\n\
Once you have properly configured the program, use the 'file' \
button to select the input abc file. The abc file may consist of \
compilation of many tunes. The list of tune titles will be displayed \
in the listbox below. Select one or tunes and then click one \
of the buttons described below. To select several tunes, either sweep \
the mouse cursor over the tunes holding the left button down, or else \
click on each tune while holding either the shift or control key down.\n\n\
quit	Will write the runabc.ini to disk and exit the program.\n\n\
play	Will run abc2midi on the selected tune and play the resulting \
midi file.\n\n\
display	Will run abc2ps on the selected tune and display the output \
PostScript file.\n\n\
edit	Will allow you to edit the selected tune and save only this tune \
in a separate  file.\n\n\
console	Will display any error messages originating from abc2ps or \
abc2midi.\n\n\
find	Will provide a tool for searching for abc tunes with specific words \
in the title.\n\n\
The remaining buttons 'toc', 'abc2midi options', 'abc2ps options' and \
'config' will replace the current sheet with a property sheet that allows \
you to modify how the midi file or postscript file is created. There are \
separate help pages for each button. Click the button once for the page \
to appear. Click the same button again to remove that sheet. \
Clicking a different button will bring up a different property page. \
There are two possible property pages associated with the 'postscript' \
button. The appropriate page will displayed depending on whether abc2ps \
or yaps was selected in the 'config' property page. \
Click the help button for further instructions when \
one of these property pages is in view.\n\n
Other bindings\n\n
The arrow, page up/down, home, end keys allow you to scroll up and\
down the table of contents. The <cntl>-slash and <cntl>-backslash allow\
you to select or deselect all titles in the table of contents. In addition\
the p key  or <space> will play the current selection and the d key will \
display this selection.\n\n\
Seymour.Shlien@crc.ca, 624 Courtenay Ave,  Ottawa, K2A 3B5, Canada, \
 Sept 9 2001."


set hlp_config_1 "Configuration Property Sheet\n\n\This page is\
used to specify the file path names to the various executables called \
by this program. It is probably most convenient to put these \
executables (except for ghostview) in the same folder as runabc. \
When you start up runabc.tcl for the first time, the entry boxes will \
be configured for my own computer for Windows 98. This is probably not the \
way things are set up on your own computer but it at least provides you \
with an example.\n\n\
For convenience, clicking one of the left buttons on this property \
sheet will pop up a file browser, which will allow you to find and \
indicate the location of the desired executable. Once you have located \
this file, click on 'open' to save this path name in the entry box. The \
contents of the entry box can also be modified by clicking to the \
position in the box using the regular keys on your keyboard. The left \
and right arrow keys will move the entry cursor. Though Tcl/Tk supports \
long file names it may get confused if the path name includes blanks or \
the backward slash. If the filename or folder where the executable is \
found contains embedded spaces, it may be necessary to substitute the \
DOS filename instead.\n\n\
If you are going to play your midi files on a midi player you should \
also go to the config/player 1 configuration page. Go to this page \
and click the help button for more instructions.\n\n"

set hlp_config_2 "This configuration page is used to specify the path name \
to the midi player, any required run time options and the protocol for \
passing the midi files to the midi player. All midi players handle single \
files.  Others can accommodate a list of midi files, while some will even \
accept a folder of midi files. The manner of passing the midi file(s) \
to the player is specified by the protocol. You may use any of the three \
protocols even if you are only passing one midi file to the player, provided \
the player supports this method. If you are passing a \
folder then the folder name should be given in the options entry \
box and the 'folder' radio button should be selected. It is necessary \
to do this manually since some midi players require the folder name \
to be preceded by a flag. Note that in some operating systems there may \
be a limit in the length of the string containing the list of midi files. \n\n\
When you click the play button, runabc creates \
a file called X.tmp containing the selected abc tunes and puts it \
in a tmp folder in the same folder from which runabc.tcl is \
invoked. (If  you wish to use a different subdirectory name instead of \
tmp, you should edit the contents of the midi_dir parameter \
in the runabc.ini file.)  Prior to running abc2midi, any midi files \
beginning with the letter X are removed from the tmp directory. Then \
abc2midi is executed with the file X.tmp and it creates a new set of \
midi files. These midi files are sent to a designated midi player using one \
of three protocols. You are allowed to designate up to two possible midi \
players which have their own option entry box and protocol.\
The selected midi player is indicated in the midi options page.\n\n\
A convenient way for viewing the X.tmp file is to click the edit/view X.tmp \
menu item. Line numbers are added so it is easy to review the error \
messages from abc2midi."


set hlp_config_3 "This feature allows you to specify the \
default font, size and style used by the program. This is a useful \
feature if your monitor is running in a high resolution and you find the \
text difficult to read. Increasing the font size will cause Tcl/Tk to \
automatically readjust the size of all the buttons, entry boxes and \
other widgets.  There are numerous possible fonts available. \
To see a list, run your Tcl/Tk interpreter (usually called 'wish') and \
type 'font families' without the quotes.\n\n If you run into problems and \
all the controls become unreadable or not usable, you can return to the \
default font by clearing the family entry box and clicking the apply \
button. In the worst case, edit the runabc.ini file and delete the \
offending font name associated with the variable font_family. (In \
other words, leave it blank.)\n\nClicking any radio button causes an \
immediate action; however if you change the font names in the entry box, \
it is necessary to click the apply button in order to make the new \
font immediately effective.\n\nThere is a separate font family assigned \
to the table of contents (toc). The table of contents (toc) will proper \
alignment if you use a fixed spacing font (eg Terminal which exists on \
both Windows and Unix systems) rather than a  variable width spacing.\n\n\
On Windows system, I prefer the System or Fixedsys font while on Linux \
I use the fixed font."


set hlp_ps "Abc2ps and Abcm2ps Property Sheet\n\n\
This property page is used to select the options for the program \
abc2ps which converts the designated tune in the abc file into a \
PostScript file and then displays the resulting file. The first three \
check boxes allow you to indicate whether you want the X: reference \
numbers, numbered bar numbers and historic notes appearing in the \
output score. Abc2ps usually uses the layout of the abc file for \
determining the number of bars in a line; however, if you check the box \
'ignore line ends', then it will fit in as many bars as possible.\n\ 
Normally, abc2ps will display all voices included in the abc file. \
It is possible to choose a subset of these voices by ticking the 'select \
voices' check box and indicating in the entry box which voices are \
desired (eg. 1-2, 5). The next time the 'display' button is clicked \
only those voices will be presented. Note: abc2ps will complain of an \
error and refuse to run if the selection box is ticked but there is nothing \
in the entry box.\n\n\
For best results, it is recommended that you use a layout file like \
letter.fmt which is provided with the distribution. This is a text \
file which can be edited and which allows you to control the font and size of \
many of the headings. Alternatively, you can try to control the layout \
using the other options provided by abc2ps which become available when \
you untick the check box labeled 'use layout file'. The factory \
settings were chosen for a low resolution screen. See the abc2ps.readme \
file which comes with the abc2ps distribution for more details. I \
recommend that you use the 'fill' 'glue' mode, but there may be cases \
where the other modes may be preferable.\n\n\
Some of the new abc files, in particular the multivoiced files, may be \
too large for abc2ps in its standard settings. Since abc2ps must store the \
entire score in memory, enough dynamic memory needs to be \
allocated for all the voices and symbols. Abc2ps normally returns a \
message (which you can view when you press the 'console' button) when it \
runs out of space. You can increase the amount of dynamic memory \
allocated using the entry boxes provided.\n\n\
Abcm2ps does not use or support all of the available options. In particular, \
it is not necessary to specify the maximum number of voices and symbols. \
The voice selection is not supported."

set hlp_yaps "Yaps Property Sheet\n\n
The page sizes, correspond to the choices available in \
gsview (ghostview). For multivoice abc tunes, you can choose between \
combining the voices on an array of staves or displaying each voice \
separately."

set hlp_midi "Midi Property Sheet\n\n\
Most abc files do not have any tempo or midi program indications, so \
it does not matter whether this program ignores or uses these \
indications. This also provides complete freedom to \
change the tempo or instrumentation. However, there are a few abc \
files (in particular the multivoice files generated by midi2abc) \
which preserve these indications. If you wish to ignore these \
indications, check the appropriate boxes in this property page.\n\n\
The tempo slider allows you to control the tempo in quarter \
notes per minute. The transpose slider allows you to transpose all \
channels up or down by a number of semitones.\n\n\
If you have specified an alternate midi player in the configuration \
property page, you can select the desired midi player to use here.\n\n\
Note: for these changes to take effect, the play mode must be aborted and \
the play button must be clicked again.\n\n\
If guitar chords are included in the abc score, this property page \
allows you to change the level and instrumentation of the chords. \
You can specify both the bass and chordal accompaniment as well as \
the program for the melody channel. By ticking or unticking the \
check boxes, you can turn the accompaniment on or off. Alternatively, \
you can turn off the melody and play only the accompaniment.\n\n\
Abc2midi choses the chordal accompaniment based on the meter of the \
piece. (See %%MIDI ghcord in the user's guide to writing abc files for \
abc2midi.) There may be some reasons where a different chordal accompaniment \
may be preferable. To access the other choices, tick the my gchord \
check box and choose the desired pattern from the gchord menu. The 'f','c' \
and 'z' indicate bass (fundamental), chords and rest respectively. The \
number following the letter indicates duration in 1/8 notes. Do not \
forget to untick the 'my gchord' check box when you move on to another \
tune with a different meter.\n\n\
Abc2midi has a quirk that it sometimes fails to recognize the %%MIDI bassprog \
and %%MIDI chordprog commands. This occurs when these commands occur in \
the tune but there is no corresponding guitar indication anywhere in the \
body of the tune. As runabc.tcl does not know whether guitar indications \
are present in the body, it always places these commands in the output \
X.tmp for abc2midi."

set hlp_voice "Voice Selection Property Sheet\n\n
There are now several multivoice abc files available on the web.\
Furthermore, you can generate your own abc files from a midi file using the \
program abc2midi. This property page allows you to turn selected \
voices on or off when they are converted into a midi file. You can \
also designate the programs (midi instruments) associated with the \
voice.\n\n Note: some abc files may already have midi indications \
embedded. You may need to tell this program to ignore these indications \
(in the midi property page) in order for these changes to be \
effective."

set hlp_abc2abc "Abc2abc Property Sheet\n\n\
Check the desired options and click the abc2abc button. The \
program will copy the selected files in the Table of Contents page (TOC) \
to the X.tmp file and will run abc2abc on this file. Depending on the\
radio button settings, the results will either be placed on a \
clipboard or recorded on the abc_default_file (e.g. edit.abc) which\
is automatically opened by an edit window. If you make the latter\
choice, the contents of the abc_default_file will automatically be\
overwritten each time you click the abc2abc button. If you use\
clipboard instead, then open a blank abc file (using edit/new file)\
and paste the contents of the clipboard using <cntl-v> on Windows\
operating system or <cntl-y> on Unix and Linux systems.\n\n\
The new spacing option, may introduce spaces between the notes in the \
abc file in order for the notes to line up on beats. If you tick the \
transpose box, you need to indicate the number of semitones to shift in \
the adjoining entry box. Positive and negative numbers are accepted by \
abc2abc. Note that if you have forgotten to tick the transpose box, \
this action will not be taken. Similarly for the linebreaks box. \n\n
If you are selecting particular voices from a multivoice file, then\
you need abc2abc version 2.18 or higher.\n\n\
See the README file in the abcMIDI.zip distribution for any additional \
documentations."

set hlp_extr "Voice: n1 Transpose: n2 \n\
(thanks to Luis Pablo Gasparotto for the suggestion and implementation)\n\n\
This feature allows you to extract single parts from a score.\
n1 is the selected voice and n2 is the amount of steps to be\
transposed. \n
When extracting voices the program also compress whole rests into\
one multimeasure rest (when it's posible) \
You can display the single part pressing the display buttom or\
to save as a new file.\n"


# Help
proc contexthelp {} {
global active_sheet cfg_subsection
global hlp_overview hlp_config_1 hlp_config_2
global hlp_config_3 hlp_ps
global hlp_midi hlp_voice
global hlp_yaps hlp_abc2abc
global hlp_extr

  switch -- $active_sheet {
    none   {show_message_page $hlp_overview word}
    titles {show_message_page $hlp_overview word}
    config {switch -- $cfg_subsection {
       1 {show_message_page $hlp_config_1 word}
       2 {show_message_page $hlp_config_2 word}
       3 {show_message_page $hlp_config_2 word}
       4 {show_message_page $hlp_config_3 word}
       }
      }
    psstyle {show_message_page $hlp_ps word}
    psform {show_message_page $hlp_ps word}
    yaps   {show_message_page $hlp_yaps word}
    midi   {show_message_page $hlp_midi word}
    voice  {show_message_page $hlp_voice word}
    abc2abc {show_message_page $hlp_abc2abc word}
    extract {show_message_page $hlp_extr word}
  }
}

#source drumedit.tcl

set drumpatches {
{35	B,,,	{Acoustic Bass Drum}}
{36	C,,	{Bass Drum 1}}
{37	^C,,	{Side Stick}}
{38	D,,	{Acoustic Snare}}
{39	^D,,	{Hand Clap}}
{40	E,,	{Electric Snare}}
{41	F,,	{Low Floor Tom}}
{42	^F,,	{Closed Hi Hat}}
{43	G,,	{High Floor Tom}}
{44	^G,,	{Pedal Hi-Hat}}
{45	A,,	{Low Tom}}
{46	^A,,	{Open Hi-Hat}}
{47	B,,	{Low-Mid Tom}} 
{48	C,	{Hi Mid Tom}} 
{49	^C,	{Crash Cymbal 1}}
{50	D,	{High Tom}}
{51	^D,	{Ride Cymbal 1}}
{52	E,	{Chinese Cymbal}}
{53	F,	{Ride Bell}}
{54	^F,	Tambourine}
{55	G,	{Splash Cymbal}}
{56	^G,	Cowbell}	
{57	A,	{Crash Cymbal 2}}
{58	^A,	Vibraslap}
{59	B,	{Ride Cymbal 2}}
{60	C	{Hi Bongo}}
{61	^C	{Low Bongo}}
{62	D	{Mute Hi Conga}}
{63	^D	{Open Hi Conga}}
{64 	E	{Low Conga}}
{65 	F	{High Timbale}}
{66	^F	{Low Timbale}}
{67	G	{High Agogo}}
{68	^G	{Low Agogo}}
{69	A	Cabasa}
{70	^A	Maracas}
{71	B	{Short Whistle}}
{72	c	{Long Whistle}}
{73	^c	{Short Guiro}}
{74	d	{Long Guiro}}
{75	^d	{Claves}}
{76	e	{Hi Wood Block}}
{77	f	{Low Wood Block}}
{78	^f	{Mute Cuica}}
{79	g	{Open Cuica}}
{80	^g	{Mute Triangle}}
{81	a	{Open Triangle}}
}




proc drum_editor {} {
  global drumpatches df drumprog
  if {[winfo exists .drums] != 0} {focus .drums
                                  return}
  toplevel .drums
  array unset drumprog
  frame .drums.window
  frame .drums.input
  text .drums.window.txt -height 1 -width 30 -font $df
  pack .drums.window.txt -side left
  pack .drums.window
  .drums.window.txt tag configure b -foreground blue 
  .drums.window.txt configure -state disable
  entry .drums.input.ent -width 20 -textvariable drumentry -font $df
  button .drums.input.enter -text enter -font $df \
    -command {break_drumentry $drumentry}
  button .drums.input.play -command play_drum_test -text play -font $df
  button .drums.input.paste -command paste_drumstring -text paste -font $df
  button .drums.input.help -text help -font $df
  pack .drums.input.ent .drums.input.enter .drums.input.play \
    .drums.input.paste .drums.input.help -anchor w -side left
  pack .drums.input -anchor w
  label .drums.prog -width 35 -relief sunken -bg #f4ece0 -font $df
  pack .drums.prog 

  bind .drums.input.help <Button> {show_message_page $hlp_drums word
                                      focus .abc
				      raise .abc .drums}


  listbox .drums.list -width 20 -borderwidth 2 -bg #f4ece0 \
    -selectmode single -yscrollcommand {.drums.ysbar set} \
    -height 8 -font $df -exportselection 0
  scrollbar .drums.ysbar -bd 2 -command {.drums.list yview}
  listbox .drums.list2 -width 20 -borderwidth 2 -bg #f4ece0 \
    -selectmode single -yscrollcommand {.drums.ysbar2 set} \
    -height 8 -font $df -exportselection 0
  scrollbar .drums.ysbar2 -bd 2 -command {.drums.list2 yview}
  pack .drums.ysbar2 .drums.list2 .drums.list .drums.ysbar -side left -fill y 
  foreach e $drumpatches {
    set item [format "%2d   %s   %s" [lindex $e 0] [lindex $e 1] [lindex $e 2]]
   .drums.list insert end $item}
  .drums.list selection set 0 
  bind .drums.input.ent <Return> {break_drumentry $drumentry}
  

  global drumprog ndrums


# This function and others are nested in proc drum_editor
  proc parse_drum_string {input} {
   set charlist [split $input ""]
   set result {}
   set item ""
   foreach letter $charlist {
   switch -regexp $letter {
   d {
     set result [concat $result $item]
     set item d}
   z {
     set result [concat $result $item]
     set item z}
   [0-9]
     {set item [append item $letter]}
   }
 }
set result [concat $result $item]
}


  proc break_drumentry {drumentry} {
   global ndrums  drumpatches drumprog
   .drums.window.txt configure -state normal 
   .drums.window.txt delete 1.0 end
   set drumliststring [parse_drum_string $drumentry] 
   set ndrums 0
   set index [.drums.list curselection]
   .drums.list2 delete 0 end
   foreach item $drumliststring {
    if {[string first "d" $item  ] >= 0} {
     .drums.window.txt insert end "$item" b
     set drumprog($ndrums) [expr $index + 35]
     set name [lindex [lindex $drumpatches $index] 2]
     .drums.list2 insert end "$item $name"
     incr ndrums
     } else {
      set drumprog($ndrums) none
     .drums.window.txt insert end "$item"
     .drums.list2 insert end "$item "
     incr ndrums}
    }
   .drums.window.txt configure -state disable
   }


  proc changeprog {} {
  global drumprog drumpatches
  set index [.drums.list curselection]
  set index2 [.drums.list2 curselection]
  set name [lindex [lindex $drumpatches $index] 2]
  .drums.prog configure -text $name 
  if {[llength $index2] < 1} return
  set content [lindex [.drums.list2 get $index2] 0]
  if {[string first "d" $content] >= 0} {
#   puts $name
   set drumprog($index2) [lindex [lindex $drumpatches $index] 0]
#   puts "drumprog $index2 $drumprog($index2)"
   .drums.list2 delete $index2
   .drums.list2 insert $index2 "$content $name"
   .drums.list2 selection set $index2} 
  }

  proc make_drumstring {} {
    global drumentry drumprog ndrums
    set drumstring [.drums.window.txt get 1.0 end] 
    for {set i 0} {$i <$ndrums} {incr i} {
      if {$drumprog($i) != "none"} {
      set drumstring [concat $drumstring $drumprog($i)]} 
     }
   return $drumstring
  }


  proc make_trial_abc_file {} {
    global midi
    set drumstring [make_drumstring]
    set outfd [open $midi(midi_dir)/X.tmp w]
    puts $outfd "X: 1\nT: drum trial\nM: 2/4\nL: 1/8\nK: G"
    puts $outfd "%%MIDI drum $drumstring"
    puts $outfd "!drum! z4|z4|z4|z4|z4|z4|z4|z4\n\n"
    close $outfd
 }

  proc play_drum_test {} {
    global midi exec_out drumentry
    make_trial_abc_file
    set dir "[pwd]/$midi(midi_dir)"
    set cmd "file delete [glob -nocomplain $dir/X*.mid]"
    catch {eval $cmd}
    set cmd "exec $midi(path_abc2midi) $midi(midi_dir)/X.tmp"
    catch {eval $cmd} exec_out
    set exec_out $cmd\n\n$exec_out
    play_midis 1
  }

  proc paste_drumstring {} {
    set drumstring [make_drumstring]
    set point [.abcedit.t index insert]
    .abcedit.t insert $point  "%%MIDI drum $drumstring\n"
  }

  bind .drums.list <ButtonRelease-1> changeprog

  set  drumentry d2dz2d
  break_drumentry $drumentry
  changeprog
}

set hlp_drums "This drum tool is designed to help you create a \
drum string which is used by abc2midi. For a good explanation of \
the drum string, you should refer to the file abcguide.txt \
that is included with the abcMIDI.zip distribution.\n\n\
The top  text line window shows the default drum string d2dz2d. \
To change it, click on the entry box in the line below and enter \
your desired string. You must complete the string with a \
carriage return or click the enter button. \
\n\nThere are two scrolled listboxes that are used to map the \
individual drum hits (eg d2) to particular drum patches. The left \
box shows the current mapping for each drum hit. \
To change, the mapping select a particular drum \
hit in the left listbox and then click on a desired patch \
shown in the right list box. The mapping should change immediately. \
The beige label immediately above the list box shows the \
last selected patch. Whenever you click the enter button, all the drum \
hits will be changed to that patch. \n\nThe play button will allow \
you to listen to the particular drumstring. It will create and play \
a midi file with 8 bars of this drumstring. The paste button will \
paste the drumstring in your file being edited in the abcedit window. \
The %%MIDI drum command will be pasted at the position of the insert \
cursor. (Don't forget to insert the !drum! command in the body of the \
music.)"
#end of drumedit.tcl

#source abcsearch.tcl




# abcsearch.tcl
# The program searches for all abc tunes whose
# title contains a specific string. 
# The search is started from the given directory
# and all abc files found in this directory or
# subdirectories are scanned.
#
#

proc find_window {} {
set p .abcsearch
global df midi
if {[winfo exist $p]} return
toplevel $p
frame $p.1
frame $p.2
label $p.lab -width 30 -text "" -font $df
label $p.1.dirlab -text "top directory" -font $df
entry $p.1.dirbox -width 30 -textvariable midi(searchdir) -font $df
button $p.1.help -text help -font $df
label $p.2.strlab -text "search string" -font $df
entry $p.2.strbox -width 30 -textvariable searchstring -font $df
bind $p.2.strbox <Return> title_match
button $p.2.but -text search -command title_match -font $df
listbox $p.list -width 50 -bg #f4ece0 \
     -yscrollcommand {.abcsearch.ysbar set} \
     -xscrollcommand {.abcsearch.xsbar set} \
     -font $df -exportselection false
scrollbar $p.ysbar -orient vertical -command {.abcsearch.list yview}
scrollbar $p.xsbar -orient horizontal -command {.abcsearch.list xview}
   
  
pack $p.1.dirlab $p.1.dirbox $p.1.help -side left
pack $p.2.strlab $p.2.strbox $p.2.but -side left
pack $p.1
pack $p.2
pack $p.ysbar -side right   -fill y -in $p 
pack $p.xsbar -side bottom  -fill x -in $p
pack $p.list -fill both -expand y -in $p
  
bind .abcsearch.list <Button> {show_file [.abcsearch.list nearest %y]}
bind .abcsearch.1.help <Button> {show_message_page $hlp_find word
                                      focus .abc
				      raise .abc .abcsearch}

}

set hlp_find "This tool is used for searching for tunes whose title\
contains a specific string (eg. word or sequence of words). The search\
is performed on all abc files contained in a specific directory including\
all its subdirectories. This tool is useful if you have a large collection\
of abc files some of which are compilations of many tunes. Before using\
tool you should enter the path to this directory in the top entry box.\
This directory will be stored in the runabc.ini file, so you will not\
have to type it in each time you run this tool. Next enter a specific\
string that you think is in the title. The program does not distinguish\
upper and lower case letters. Avoid punctuation marks and other special\
symbols as this string is actually interpreted as a regular Tcl expression\
using the regexp function. (You can actually specify fairly complex\
patterns in the title if you know about the regexp function.) \n\n\
After you enter a carriage return or click the search button, the\
program will cd to this directory and locate all abc files using a\
recursive glob.  For each file with an abc extension, the title line\
of every tune will be scanned for the specified string. Whenever a\
match is found, the title and source will be added to the listbox below.\
If you feel you have enough matches, you can stop the search by clicking\
on the stop button.\n\n\ All the identified titles in the listbox are\
mouse sensitive. If you click on one of the titles, the selected file\
will be automatically opened, the table of contents of that file will\
appear and the listbox will automatically scroll so that the selected\
tune will be visible.  Note that some tunes may have more\
than one title. The match is done on all titles, but only the first\
title appears in the table of contents. If it selects a title past the\
one you wanted, it probably indicates that some tunes are missing a T:\
field in the abc file. Correct the abc file so you do not miss some tunes."



# This function lists the X: index number, titles and key signature
# of every tune found in the specified abcfile. The function expects
# every tune to begin with an X: index specifier and the title to
# found in the T: field command. More than one title may belong to
# a specific tune.

# Method: search for an X: field and then all the associated T:
# fields. When a K: field is encountered, grab the key signature
# and output all the title, key information to the open output file
# referenced by the outhandle variable;  ignore everything else
# that follows until the next X: field is found. Stop when an
# eof is encountered.
proc search_string_in_title {abcfile filenum} {
    global searchstring
    global listbox_line
    global file_index locator
    global stopsearch
    set srch X 
    set blank_lines 0
    set titlehandle [open $abcfile r]
    set loc -1
    while {[gets $titlehandle line] >= 0 && $stopsearch == 0} {
	if {!$blank_lines && [string length $line] < 1} {set srch X}
        set initialchar [string index $line 0]
	switch -- $srch {
	X {
		if { $initialchar == "X"} {
                    set number [string range $line 2 10]
		    set srch T
                    incr loc
		}
	    }
	T {
		if { $initialchar == "T"} {
		    set name [string range $line 2 end]
		    set name [string trim $name]
                     if {[regexp -nocase  $searchstring $name] > 0} {
                      set match   [format "%s %s   %s" $number $name $abcfile]
                      .abcsearch.list insert end $match
                      update
                      set file_index($listbox_line) $filenum
                      set locator($listbox_line) $loc
                      .abcsearch.list see $listbox_line
                      incr listbox_line
		    } elseif {$initialchar == "K"} {
#		    set keysig [string range $line 2 end]
#		    set keysig [string trim $keysig]
		    set srch X
		    }
	    }
    }
  }
 }
  close $titlehandle
}


proc show_file {loc} {
global midi
global file_index gfiles locator
global globesel
if {$loc < 0} return
open_abc_file $midi(searchdir)/$gfiles($file_index($loc))
#focus .abc
#raise .abc .abcsearch
.abc.titles.t selection clear 0 end
.abc.titles.t selection set $locator($loc) $locator($loc)
.abc.titles.t see $locator($loc)
set globsel $locator($loc)
update
}


#
#---------------------------------------------------------------------------
#  rglob, Performs a recursive glob
#
#  Copyright (C) 2000 Agnar Renolen
#     email: agnar@organizer.net
#
#  This module is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  .................
#---------------------------------------------------------------------------

##
#   rglob - recursive glob
#
# SYNOPSIS
#   [rglob <pattern>]
#
# DESCRIPTION
#   Executes a recursive /glob/ returning a list of all files matching
#   a <pattern>.  If the search is to be started in a particular
#   directory, this is obtained by prefixing the <pattern> with the
#   directory name as e.g., [rglob "c:/develop/tclproject/*.tcl"].
# 
#   The globbing is executed using the [-nocomplain] option, so if
#   [rglob] finds no matches, an empty list is returned.
##

proc rglob {pattern} {
    set fileList {}
    foreach fname [glob -nocomplain $pattern] {
	if {![file isdirectory $fname]} {
	    lappend fileList $fname
	}
    }
    foreach fname [glob -nocomplain -types d "[file dirname $pattern]/*"] {
	set fileList [concat $fileList [rglob "$fname/[file tail $pattern]"]]
    }
    return $fileList
}


proc title_match {} {
global midi 
global gfiles
global file_index
global listbox_line
global locator
global stopsearch
global searchstring
if {[string length $searchstring] < 1} return
set home [pwd]
cd $midi(searchdir)
pack .abcsearch.lab
.abcsearch.2.but configure -text stop -command stopsearching 
.abcsearch.list delete 0 end
set file_list [rglob *.abc]
set i 0
set listbox_line 0
if {[info exist file_index]} {unset file_index}
if {[info exist locator]} {unset locator}
set stopsearch 0
foreach filename $file_list {
    if {$stopsearch} break;
    set gfiles($i) $filename
    .abcsearch.lab configure -text $filename
    update
    search_string_in_title $filename $i
    incr i
}
cd $home
.abcsearch.2.but configure -text search -command title_match
}

proc stopsearching {} {
global stopsearch
set stopsearch 1
.abcsearch.2.but configure -text search -command title_match
}
#end of abcsearch.tcl




#start of abcmatch.tcl

proc check_abcmatch_version {version} {
global midi
global exec_out
if {[file exist $midi(path_abcmatch)] != 1} {
  messages "runabc requires the executable abcmatch.exe to run this \
function. You need to specify the path to this executable using the \
config/executables menu item."
  return}
set cmd "exec [list $midi(path_abcmatch)]  -ver"
catch {eval $cmd} result
set exec_out $cmd\n$result
if {$result < [expr $version + 0.1] && $result >= $version} return
#version difference of 0.1 implies new version not compatible with runabc
messages "runabc was expecting abcmatch version $version. Instead it \
found version $result"
}


proc match_window {} {
global midi df hlp_match
set f .matcher
if {[winfo exist $f]} return
toplevel $f
frame $f.1
label $f.1.inputfilelab -text "path" -font $df
entry $f.1.inputfilevar -width 40 -textvariable midi(searchdir) -font $df
button $f.1.filebrowse -text browse -command match_file_browser -font $df
frame $f.2
label $f.2.keylab -text K: -font $df
label $f.2.meterlab -text M: -font $df
label $f.2.lengthlab -text L: -font $df
entry $f.2.keyent -width 5 -textvariable midi(match_keysig) -font $df
entry $f.2.meterent -width 5 -textvariable midi(match_timesig) -font $df
entry $f.2.lengthent -width 5 -textvariable midi(match_length) -font $df
button $f.2.start -text match -font $df -command {create_match.abc
                                         start_matcher}
button $f.2.play -text play -font $df -command play_match.abc
button $f.2.transfer -text transfer -font $df -command transfer_editor_buffer_to_abcmatch
button $f.2.help -text help -font $df \
 -command {show_message_page $hlp_match word}
pack  $f.2.keylab $f.2.keyent $f.2.meterlab $f.2.meterent $f.2.lengthlab \
$f.2.lengthent $f.2.start $f.2.play $f.2.transfer $f.2.help  -side left -anchor w

frame $f.3
label $f.3.bodylab -text body -font $df
entry $f.3.bodyent -width 64 -textvariable midi(match_body) -font $df
pack  $f.1.inputfilelab  $f.1.inputfilevar $f.1.filebrowse -side left -anchor w
pack  $f.2.keylab $f.2.keyent $f.2.meterlab $f.2.meterent $f.2.lengthlab $f.2.lengthent $f.2.start  -side left -anchor w
pack  $f.3.bodylab $f.3.bodyent  -side left -anchor w
pack $f.1 $f.2 $f.3  -side top -anchor w

frame $f.options 
label $f.options.reslab -text resolution -font $df
entry $f.options.resent -width 3 -textvariable midi(match_resolution) -font $df
set p $f.options.resbut
menubutton $p -menu $p.type -text "1/8 note" -relief raised -font $df
menu $p.type -tearoff 0
$p.type add radiobutton -label "exact"  -command "setres 0" -font $df
$p.type add radiobutton -label "1/16 note" -command "setres 6" -font $df
$p.type add radiobutton -label "1/8 note"  -command "setres 12" -font $df
$p.type add radiobutton -label "1/4 note"  -command "setres 24" -font $df
$p.type add radiobutton -label "3/8 note"  -command "setres 36" -font $df
$p.type add radiobutton -label "1/2 note"  -command "setres 48" -font $df
$p.type add radiobutton -label "3/4 note"  -command "setres 72" -font $df
$p.type add radiobutton -label "whole note" -command "setres 96" -font $df

radiobutton $f.options.any1 -variable midi(match_any) -value 1 -text "any bar" -font $df
radiobutton $f.options.any2 -variable midi(match_any) -value 0 -text "all bars" -font $df
checkbutton $f.options.contour -variable midi(match_contour) -text "contour match" -font $df


frame $f.msg
label $f.msg.lab -width 40 -text "" -font $df
setres $midi(match_resolution)
pack  $f.options.reslab $f.options.resent $f.options.resbut -side left -anchor w
pack  $f.options.any1 $f.options.any2 $f.options.contour -side left -anchor w
pack  $f.options -side top -anchor w
pack $f.msg.lab -side left
pack $f.msg -side top
check_abcmatch_version {1.0}
}


proc setres {value} {
global midi
set w .matcher.options.resbut
switch $value {
  0 {set midi(match_resolution) 0
     $w configure -text exact } 
  6 {set midi(match_resolution) 6
     $w configure -text "1/16 note" } 
  12 {set midi(match_resolution) 12
     $w configure -text "1/8 note" } 
  24 {set midi(match_resolution) 24 
     $w configure -text "1/4 note" } 
  36 {set midi(match_resolution) 36 
     $w configure -text "3/8 note" } 
  48 {set midi(match_resolution) 48 
     $w configure -text "1/2 note" } 
  72 {set midi(match_resolution) 72 
     $w configure -text "3/4 note" } 
  96 {set midi(match_resolution) 96 
     $w configure -text "whole note" } 
  }
}
 
proc show_matcher_results {text} {
pop_message_page 
.matcher.notice.t insert end $text
}

proc pop_matcher_results {} {
global df
set p .matcher.notice
if [winfo exist .matcher.notice] {
   $p.t configure -state normal 
   $p.t delete 1.0 end
   } else {
   frame $p
   text $p.t -height 25 -width 70 -wrap word -yscrollcommand {.matcher.notice.ysbar set} -font $df
   scrollbar $p.ysbar -orient vertical -command {.matcher.notice.t yview}
   pack $p.ysbar -side right -fill y -in $p
   pack $p.t -in $p
   pack $p}
}

       

proc create_match.abc {} {
global midi
set handle [open match.abc w]
puts $handle  X:1
puts $handle  T:match_template
puts $handle  M:$midi(match_timesig)
puts $handle  L:$midi(match_length)
puts $handle  K:$midi(match_keysig)
puts $handle  $midi(match_body)
close $handle
set last [string index $midi(match_body) [expr [string length $midi(match_body)] -1]]
if {[string equal $last |] != 1} {
 messages "body should end with a bar line"}
}

proc start_matcher {} {
global midi
global stopsearch
global match_index
if {[file exist $midi(searchdir)] != 1} {
   messages "The file or directory $midi(searchdir) could not be \
found. Searching was aborted."
   return
   }
set match_index 0
set stopsearch 0
.matcher.2.start configure -text stop -command stop_matcher
update
if {[file isdirectory $midi(searchdir)]} run_glob_matcher else run_matcher
}


proc stop_matcher {} {
global stopsearch
.matcher.2.start configure -text match -command {create_match.abc
						start_matcher}
set stopsearch 1
}

proc run_matcher {} {
global matchlist
global midi
global exec_out
set cmd "exec [list $midi(path_abcmatch)]  [list $midi(searchdir)] -r $midi(match_resolution)"
if {$midi(match_any)} {set cmd [concat $cmd -a]}
if {$midi(match_contour)} {set cmd [concat $cmd -con]}
catch {eval $cmd} result
set exec_out $cmd\n$result
set matchlist [split $result \n]
pop_matcher_results
record_matched_tunes $midi(searchdir)
stop_matcher
}


proc run_glob_matcher {} {
global midi
global matchlist
global exec_out
global stopsearch
set home [pwd]
cd $midi(searchdir)
set file_list [rglob *.abc]
cd $home
set stopsearch 0
pop_matcher_results
set i 0
foreach filename $file_list {
    .matcher.msg.lab configure -text $filename"
    update
     set filepath [file join $midi(searchdir) $filename]
     set cmd "exec [list $midi(path_abcmatch)] [list $filepath] \
        -r $midi(match_resolution)"
    if {$midi(match_any)} {set cmd [concat $cmd -a]}
    if {$midi(match_contour)} {set cmd [concat $cmd -con]}
    catch {eval $cmd} result
    set exec_out $cmd\n$result
    set matchlist [split $result \n"]
    record_matched_tunes [file join $midi(searchdir) $filename]
    if {$stopsearch} break;
    set gfiles($i) $filename
    incr i
    update
    }
stop_matcher
}


proc record_matched_tunes {filename} {
global matchlist mlocator
set lastxref -1
set abchandle [open $filename r]
foreach matchpair $matchlist {
  set matches [lrange $matchpair 2 [llength $matchpair]]
  set xref [lindex $matchpair 1]
  set mlocator [lindex $matchpair 0]
  if {$xref != $lastxref} {
     find_xref_in_file $abchandle $xref $filename
     highlight_selected_bars $matches
     set lastxref $xref
     
  }
 }
close $abchandle
}




proc copy_tune {abchandle} {
global barloc df
set line [gets $abchandle]
set bcount 1;
.matcher.notice.t tag configure bl  -foreground red
.matcher.notice.t insert end $line\n
set p {\|+}
if {[info exist barloc]} {unset barloc}
while {[string length $line] > 0 } {
       set line  [gets $abchandle]
       if {[string index $line 0] == "X"} break;
       .matcher.notice.t insert end $line 

       set initial [string index $line 0]
       set next [string index $line 1]
       if {[string first $initial ABCDEFGHIKLMNOPQRSTUVWwXZ]
            >= 0 && $next == ":" } {
               .matcher.notice.t insert end \n 
               continue
               }

       set begin 0
       set result 1
       set index [.matcher.notice.t index "end -1 char"]  
       set linenum [lindex [split $index .] 0]
       while {$result} {
       set result [regexp -start $begin -indices $p $line loc]
       if {$result} {set begin [lindex $loc 1]
                   set barloc($bcount) $linenum.$begin 
                   if {$bcount==1} {set barloc(0) $linenum.0} 
                   incr begin
                   incr bcount
                   }
          }
       .matcher.notice.t insert end \n
       }
}


proc highlight_selected_bars {bars} {
global barloc
.matcher.notice.t tag configure bl  -foreground red
foreach bar $bars {
 if {[string is integer $bar] == 0} break;
 set bar2 [expr $bar + 1]
 if {[info exist barloc($bar2)]} {
      .matcher.notice.t tag add bl $barloc($bar) $barloc($bar2)
       }
  }
}




proc find_xref_in_file {abchandle xref filename} {
global match_index
global mlocator
while {[gets $abchandle line] >= 0 } {
set initialchar [string index $line 0]
 if {[string equal $initialchar X]} {
    set number [string range $line 2 10]
    if {$number == $xref} {
       .matcher.notice.t tag configure m$match_index -foreground darkblue
       .matcher.notice.t insert end  $filename\n m$match_index
       .matcher.notice.t tag bind m$match_index <1> "show_match_file $filename $mlocator"
       incr match_index
       .matcher.notice.t insert end \n$line\n
       copy_tune $abchandle
       break}
    }
 }
}


#set types {{{abc files} {*.abc}}
#           {{all} {*}}}

proc match_file_browser {} {
global types midi
set filename [tk_getOpenFile -filetypes $types]
if {[string length $filename] > 0} {set midi(searchdir) $filename}
}


proc show_match_file {filename locator} {
global midi
global globesel
if {$locator < 0} return
open_abc_file $filename
.abc.titles.t selection clear 0 end
.abc.titles.t selection set $locator $locator
.abc.titles.t see $locator
set globsel $locator
update
}


proc transfer_editor_buffer_to_abcmatch {} {
global midi
global body_start body_end
if {[winfo  exist .abcedit] == 0} {
   messages "You need to run my selection editor\
and highlight several complete bars of the music body\
of a specific tune before using this function."
return
}

  set selrange [.abcedit.t tag ranges sel]
  if {[llength $selrange] < 2} {
     messages "Please select an area in the body of the tune \
and then try again. To select a region, hold the left mouse button \
down and sweep an area." 
     return
    } else {
    set selstart [lindex $selrange 0]
    set selend [lindex $selrange 1]
    set value [.abcedit.t get $selstart $selend]
    set value [remove_guitar_chords $value 0]
    set value [remove_grace_notes $value 0]
    set midi(match_body) $value
 
   set point [.abcedit.t index insert]
   if {$point > $body_end} {set point $body_start.0}
   set kfield [.abcedit.t search -backwards K: $point]
   set kfield [.abcedit.t get $kfield  "$kfield lineend"]
   set midi(match_keysig) [string range $kfield 2 end]
   set lfield [.abcedit.t search -backwards L: $point]
   if {[string length $lfield] > 0} {
     set lfield [.abcedit.t get $lfield  "$lfield lineend"]
     set midi(match_length) [string range $lfield 2 end]
   } else {
     set midi(match_length) 1/8}

   
   set mfield [.abcedit.t search -backwards M: $point]
   set mfield [.abcedit.t get $mfield  "$mfield lineend"]
   set midi(match_timesig) [string range $mfield 2 end]
   }
}



proc play_match.abc {} {
global midi exec_out
global files
set dir "[pwd]/$midi(midi_dir)"
set cmd "file delete [glob -nocomplain $dir/X*.mid]"
catch {eval $cmd}
set out_fd [open $midi(midi_dir)/X.tmp w]
puts $out_fd "X: 1"
write_midi_codes $out_fd
puts $out_fd "M: $midi(match_timesig)"
puts $out_fd "L: $midi(match_length)"
puts $out_fd "K: $midi(match_keysig)"
puts $out_fd "$midi(match_body)\n"
close $out_fd
set cmd "exec $midi(path_abc2midi) $midi(midi_dir)/X.tmp"
catch {eval $cmd} exec_out
set exec_out $cmd\n\n$exec_out
set files $midi(midi_dir)/X1.mid
play_midis 1
}

set hlp_match \
"This function searches for any tune having music bars with\
a specific pattern. The search is performed on either a specific\
abc file (which is usually a compilation of many tunes) or on a\
folder containing abc files. The file or folder is specified in\
the path entry box. The browse button allows you to use the tcl/tk\
 file browser to search for the file, but unfortunately it does not\
allow you to specify  a folder. (It is necessary to edit the\
contents of the path entry box to specify a folder.) If you\
specify a folder, the program will search recursively for all abc\
files starting from this folder.\n\nTo specify the music pattern\
that you are searching for, you edit the contents of the K: M: L:\
and body entry boxes. The contents of the first three boxes are\
identical to the corresponding field entries in an abc formated file.\
The body should contain 1 or a few complete bars. It is not necessary\ 
to start a bar with a bar line, but the last bar must end with a\
bar line.\n\nClicking the `play' button allows you to hear the bar or\
bars that you have specified. The button labeled `transfer' allows\
you to fill in the K: M: L: and body fields rapidly, assuming that\
the specific bar or bars is already highlighted in a the tcl abc\
editor (called my selection editor). Thus you are able to search for\
other tunes which include those highlighted bars. When you transfer\
these highlighted bars, guitar chords and grace notes are automatically\
removed. The searching function ignores this information as well as\
decorations, slurs and rolls. Instead of clicking the transfer button\
you may also use the <Alt-t> key when the focus is on the abcedit\
window.\n\nTo start the matching process click\
the button labeled `match'. Matches will appear in a scrolled text\
window appearing below and all bars that match the criterion will be\
displayed in red. In addition the file where the match occurred will\
appear above in blue. If you click on the blue file name, then the\
selected tune will be loaded into the table of contents window and\
the specific tune will be selected and centered in that window. This\
allows you to get to that tune rapidly, assuming you wish to either\
display the music notation or play the tune.\n\nOther parameters such\
as resolution, and other radio and check buttons, control the matching\
process. They allow you to perform an exact or loose match and are\
explained in detail in the documentation included with the abcmatch.zip\
distribution."



#end of abcmatch.tcl

#source diag.tcl
#Check configuration of runabc

bind . <Alt-s> {runabc_diagnostic}
bind . <Alt-S> {runabc_diagnostic}

proc runabc_diagnostic {} {
global midi
global tcl_platform
set diag_output [open  runabc.out   w ]
puts $diag_output "runabc version $midi(version)"
set plat $tcl_platform(platform)
set os $tcl_platform(os)
set osver $tcl_platform(osVersion)
set machine $tcl_platform(machine)
set msg [format "platform $plat\noperating system $os\nversion $osver\
\nmachine $machine\n"]
puts $diag_output $msg
set msg "\nThe following verifies the existence of an executable at
the specific location specified in the config/executables property
sheet. Note that executable may still be accessible using the
PATH environment variable if only the filename is given.\n\n" 
puts $diag_output $msg

foreach path {path_abc2abc path_abc2midi path_abc2ps path_abcm2ps \
	path_yaps path_gs path_midiplay path_abcmatch} {
 set msg [check_file $path]
 puts $diag_output $msg
 }

#check out abc2midi

  puts $diag_output "\nchecking out abc2midi\n"
  set cmd "exec [list $midi(path_abc2midi)]"
  catch {eval $cmd} exec_out
  set msg $cmd\n$exec_out
  puts $diag_output $msg\n
 
  set sel [title_selected]
  set sel [tune_picked $sel $midi(abc_open)]
  set cmd "exec [list $midi(path_abc2midi)] [list $midi(midi_dir)/X.tmp] -v"
  catch {eval $cmd} exec_out
  set exec_out $cmd\n$exec_out
  puts $diag_output $exec_out

close $diag_output
messages "Results of the sanity check are recorded in the\
file runabc.out. You may view these results using any text\
editor or word processor."
}

proc check_file {execname} {
global midi
if {[string length $midi($execname)] < 1} {
	set msg [format "$execname not defined"]} else {
  if {[file exist $midi($execname)]} {
	set msg [format "executable $midi($execname) for $execname was found"]
  } else {
	set msg [format "cannot find executable $midi($execname) for $execname"]}
  }
return $msg
}
#end of diag.tcl


###source lpgextra.tcl


# open last used file
if [file exists $midi(abc_open)] { 
   title_index $midi(abc_open)
   show_titles_page  
}

# create tmp directory if it doesn't exists
file mkdir "[pwd]/$midi(midi_dir)"
# create abc_work_folder if it doesn't exists
file mkdir "[pwd]/$midi(abc_work_folder)"


switch_ps_button
focus .abc

proc abcmr {} {
global type
#originally coded by Luis Pablo Gasparotto
#reorganized and documented by Seymour Shlien.

proc condense {wr_length} {
# Starting from the column'th character in music_string where
# the first whole_rest was found, the procedure continues
# scanning all the following measures in music_string
# until it encounters a situation which breaks the
# sequence of whole bar rests. The function counts
# the number of whole rest bars, stores it in the variable
# measures and then issues # a Zn or "n"z8 rest to cover these bars.
# The output is put into music_string2.
# wr_length is the number of characters in the whole bar
# rest (eg. z16 -- either 2 or 3) 
global music_string
global line
global column
global whole_rest
global music_string2
global type
global temp_file
global file2 outfile_flag

# initialize local variables
set measures 1 ;# counts number of bars of whole rests
set measures2 0 ; #used to signal that the end of music_string was encountered
set multimeasures 1 ;# signals that a break in a sequence of rests occurred.
set column3 [expr $column + $wr_length]
set barline "nb"

#We need to remember the type of bar line which broke the sequence,
#since we will be issueing that bar line.
while { $multimeasures != 0 } {
    set multimeasures 0

    if { [string index $music_string $column3] == "|"} {
	set multimeasures 1
       }

    set token [string range $music_string $column3 [expr $column3 +1]]
    # Ending if :| , |: or || detected
    if { $token == ":|" } {
	set barline "rc"
	set multimeasures 0
    }
    if { $token == "|:" } {
	set barline "ro"
	set multimeasures 0
    }    	
    if { $token == "||"  } {
	set barline "db"
	set multimeasures 0
    }
    if { $token == "|]" } {
	set barline "fb"
	set multimeasures 0
    }
    # Check if there's a whole_rest at the current position. If so, increment
    # measures, the number of measures in the sequence.    
    if { [string range $music_string $column3 [expr $column3+$wr_length-1]] == $whole_rest } {
	incr measures
	set multimeasures $wr_length
    }
    if { [string index $music_string $column3] == " " } {
	set multimeasures 1
    }
    # Check for line breaks
    if { [string range $music_string $column3 $column3] == "\\" } {
	set multimeasures 1
    }
    if { [string is control [string range $music_string $column3 $column3]] == 1 } {
# if we get to the end of the line, (i.e. encounter a newline), we set music_string
# to the following line.
    set measures2 $measures
    incr line
    set music_string $temp_file($line)
    set column3 -1
    set multimeasures 1
    }
set column3 [expr $column3+$multimeasures]
}


set suma 0;  #used to skip over character in music_string
if { $measures != 1 } {
    # The following line add the multimeasure
    if { $type == "z" } {
	set music_string2 "$music_string2\"$measures\"$whole_rest"
	set incr [expr $wr_length+3]
    } else {
	set music_string2 "$music_string2\Z$measures"
	set incr 2
    }
    if { $measures > 9 } {
    } else {
    }
    # Adding a barline closing the multimeasure rest
    if { $barline == "rc" } {
    	set music_string2 "$music_string2 :|"
	set suma 1
    } elseif { $barline == "ro" } {
    	set music_string2 "$music_string2 |:"
	set suma 1    
    } elseif { $barline == "db" } {
    	set music_string2 "$music_string2 ||"
    	set suma 1
    } elseif { $barline == "nb" } {
    	set music_string2 "$music_string2 |"
	if { $column3 >= 1 } {
	    set column3 [expr $column3-1]
	}
    } elseif { $barline == "fb" } {
    	set music_string2 "$music_string2 |]"
	set suma 1
    }
    if { $measures == $measures2 } {
    puts $file2 "$music_string2"
    set column3 -1
    set suma 0
    set music_string2 ""
    }
    set column [expr $column3+$suma]
} elseif { $measures2 > 0 } {
    set line [expr $line-1]
    set music_string $temp_file($line)
    set music_string2 "$music_string2[string range $music_string $column $column]"
    } else {
    set music_string2 "$music_string2[string range $music_string $column $column]"
    }
}
    
proc abcmr_load_file {} {

  global temp_file compl
  global file file2 midi midi_dir
  set file [open "$midi(midi_dir)/temp2.abc" r]
  set file2 [open "$midi(midi_dir)/temp3.abc" w]
  set body 0

# Copy input file into temp_file variable deleting any comment lines which are
# not pseudocomments and inline V: indications.
  while { [gets $file temp_file2] != -1 } {
    # Removing tab char
    regsub -all \t $temp_file2 { } temp_file2
    # Removing \ char if it's necessary
    # regsub -all {\\} $temp_file2 { } temp_file2
    if { $body == 1 } {
    	regsub -all {(\[)( )*(V:)( )*([0-9]+)( )*(\])} $temp_file2 {} temp_file2

	# Looks for double flats and double sharps
    
    }
    if { [string match V:* $temp_file2] == 1 } {
	set body 1
    }
    
    # deleting comment lines not pseudocomment
    if { [string match *%* $temp_file2] == 1 } {
	if { [string match *%%* $temp_file2] == 1 } {
	    set temp_file($compl) $temp_file2
	    incr compl
	} 
    } else { 
	set temp_file($compl) $temp_file2
	incr compl ; #number of lines in temp_file array.
    }
  }
}



proc abcmr_get_whole_rest {} {
# This procedure  checks if it's on the header and skips over all
# all field lines except L: and M: which are used to determine
# the length of a  whole rest. At the same time, the
# procedure outputs all header stuff to stdout.

  global temp_file line wr_length 
  global file2 outfile_flag
  set pattern1 {(M:)([0-9]+)*([/])*([0-9]+)}
  set pattern2 {(L:)([0-9]+)*([/])*([0-9]+)}
  set body 0
  set num 0
  set den 0
  set num2 0
  set den2 0
  while { $body != 1 } { 
    set music_string $temp_file($line)
    if { [string match V:* $music_string] == 1 } {
	set body 1
    }
    puts $file2 "$music_string"
    if { [regexp $pattern1 $music_string meas_line meas1 num meas2 den] == 1 } { 
    }

    if { [regexp $pattern2 $music_string beat_line beat1 num2 beat2 den2] == 1 } { 
    }

    incr line
   }


	# Determine the whole rest unit

# if no L: or M: was encountered assume 4/4 time and L:1/8
  if { $den2 == 0 } {
      set num2 1
      set den2 8
     }
  if { $den == 0 } {
      set num 4
      set den 4
     }

  set duration [expr $den2*$num/$num2/$den]
  set whole_rest "z$duration"
# Determine the $whole_rest length
  if { $duration >= 10 } {
    set wr_length 3
  } else {
    set wr_length 2
  }
  return $whole_rest
}



proc abcmr_pass_1 {} {
  global temp_file line compl type whole_rest
  global file2 outfile_flag
  set pattern1 {(Z)([0-9]+)}
  set pattern2 {(\")([0-9]+)(\")(z)([0-9]+)}
  set line2 $line; # save position of beginning of body.

# If there are any multimeasure rest indications already in the file
# we make sure that they conform to the desired format specified by the $type
# variable. We replace them directly in the input music_string.
  while { $line2 != $compl } {
    set music_string $temp_file($line2)

    if { $type == "z" } {
	if { [regexp $pattern1 $music_string mr_string meas_type meas_num] == 1 } { 
	    regsub {(Z)([0-9]+)} $music_string "\"$meas_num\"$whole_rest" music_string
	    set temp_file($line2) $music_string
	} else {
	    incr line2
	}
    }
    if { $type == "Z" } { 
	if { [regexp $pattern2 $music_string mr_string sign1 meas_num sign2 sign3 sign4] == 1 } { 
	    if { $sign4 == $duration } {
		regsub {(\")([0-9]+)(\")(z)([0-9]+)} $music_string "Z$meas_num" music_string
		set temp_file($line2) $music_string
	    } else {
		set temp_file($line2) $music_string
		incr line2
	    }	
	} else {
	    incr line2
	}
    }
  }
}


proc abcmr_pass_2 {} {
  global line compl whole_rest music_string music_string2 column temp_file
  global wr_length
  global file2 

# Second and final pass. We match on whole rests and call the procedure
# condense to do the rest. Note that the procedure condense can modify
# the global variables music_string, music_string2, line, and column

  while { $line < $compl } {
    set music_string $temp_file($line)

    if { [string match *$whole_rest* $music_string] != 1 } {   ;#nothing to do 
       puts $file2 "$music_string"
       set music_string2 ""
       incr line
       continue
       }

#  scan music_string character by character; if this is no whole_rest, just
#  append the character to music_string2, otherwise call condense to process the rest.

    while { [string is control [string index $music_string $column]] != 1 } {
        if { [string range $music_string $column [expr $wr_length+$column-1]] == $whole_rest \
              &&  \
	     [string index $music_string [expr $column-1] ] != "\""  && \
	     [string index $music_string [expr $column-1] ] != "!" } { condense $wr_length
	} else  {
	      set music_string2 $music_string2[string range $music_string $column $column]
        }
        incr column 
    }
      puts $file2 "$music_string2"
   incr line
   set music_string2 "" 
   set column 0
   }
}


###################### Main Program
global compl line type whole_rest column midi midi_dir file file2
set compl 0
abcmr_load_file

#Process contents of temp_file array.

set temp_file($compl) " "
set line 0
set line2 0
set column 0



set whole_rest [abcmr_get_whole_rest] 

# At this point, we are processing the body of the file.
# All header field lines have already been outputted.

abcmr_pass_1

set music_string2 ""

abcmr_pass_2

close $file2
close $file
}

proc redacc_load_file {} {
  global infile outfile input_line pattern2 clef_accidentals notes key flats sharps naturals_s naturals_f file2
  set file [open "$infile" r]
  set file2 [open "$outfile" w]
  set pattern1 {(K:)\w*}
  set pattern2 {(K:)\w*(\])}
  set compl 0
  set body 0
  set key "C"  
    # Set all notes
    
    set notes(0) "C,,," 
    set notes(1) "D,,,"     
    set notes(2) "E,,,"     
    set notes(3) "F,,,"     
    set notes(4) "G,,," 
    set notes(5) "A,,," 
    set notes(6) "B,,," 
    set notes(7) "c'''" 
    set notes(8) "d'''" 
    set notes(9) "e'''" 
    set notes(10) "f'''" 
    set notes(11) "g'''" 
    set notes(12) "a'''" 
    set notes(13) "b'''" 
    set notes(14) "C,," 
    set notes(15) "D,," 
    set notes(16) "E,," 
    set notes(17) "F,," 
    set notes(18) "G,," 
    set notes(19) "A,," 
    set notes(20) "c''" 
    set notes(21) "d''" 
    set notes(22) "e''" 
    set notes(23) "f''" 
    set notes(24) "g''" 
    set notes(25) "a''" 
    set notes(26) "b''" 
    set notes(27) "C," 
    set notes(28) "D," 
    set notes(29) "E," 
    set notes(30) "F," 
    set notes(31) "G," 
    set notes(32) "A," 
    set notes(33) "B," 
    set notes(34) "c'" 
    set notes(35) "d'" 
    set notes(36) "e'" 
    set notes(37) "f'" 
    set notes(38) "g'" 
    set notes(39) "a'" 
    set notes(40) "b'" 
    set notes(41) "C" 
    set notes(42) "D" 
    set notes(43) "E" 
    set notes(44) "F" 
    set notes(45) "G" 
    set notes(46) "A" 
    set notes(47) "B" 
    set notes(48) "c" 
    set notes(49) "d" 
    set notes(50) "e" 
    set notes(51) "f" 
    set notes(52) "g" 
    set notes(53) "a" 
    set notes(54) "b" 

    # Set accidentals for each clef
    
    # C clef
    set clef_accidentals(CC) "=" 
    set clef_accidentals(CD) "=" 
    set clef_accidentals(CE) "="
    set clef_accidentals(CF) "="
    set clef_accidentals(CG) "=" 
    set clef_accidentals(CA) "=" 
    set clef_accidentals(CB) "="
    set clef_accidentals(Cc) "=" 
    set clef_accidentals(Cd) "=" 
    set clef_accidentals(Ce) "="
    set clef_accidentals(Cf) "=" 
    set clef_accidentals(Cg) "=" 
    set clef_accidentals(Ca) "="
    set clef_accidentals(Cb) "=" 

    # C# clef
    set clef_accidentals(C#C) "^" 
    set clef_accidentals(C#D) "^" 
    set clef_accidentals(C#F) "^"
    set clef_accidentals(C#E) "^"
    set clef_accidentals(C#G) "^" 
    set clef_accidentals(C#A) "^" 
    set clef_accidentals(C#B) "^"
    set clef_accidentals(C#c) "^" 
    set clef_accidentals(C#d) "^" 
    set clef_accidentals(C#e) "^"
    set clef_accidentals(C#f) "^" 
    set clef_accidentals(C#g) "^" 
    set clef_accidentals(C#a) "^"
    set clef_accidentals(C#b) "^" 
    
    # Db clef
    set clef_accidentals(DbC) "=" 
    set clef_accidentals(DbD) "_" 
    set clef_accidentals(DbE) "_"
    set clef_accidentals(DbF) "="
    set clef_accidentals(DbG) "_" 
    set clef_accidentals(DbA) "_" 
    set clef_accidentals(DbB) "_"
    set clef_accidentals(Dbc) "=" 
    set clef_accidentals(Dbd) "_" 
    set clef_accidentals(Dbe) "_"
    set clef_accidentals(Dbf) "=" 
    set clef_accidentals(Dbg) "_" 
    set clef_accidentals(Dba) "_"
    set clef_accidentals(Dbb) "_" 
    
    # D clef
    set clef_accidentals(DC) "^" 
    set clef_accidentals(DD) "=" 
    set clef_accidentals(DE) "="
    set clef_accidentals(DF) "^"
    set clef_accidentals(DG) "=" 
    set clef_accidentals(DA) "=" 
    set clef_accidentals(DB) "="
    set clef_accidentals(Dc) "^" 
    set clef_accidentals(Dd) "=" 
    set clef_accidentals(De) "="
    set clef_accidentals(Df) "^" 
    set clef_accidentals(Dg) "=" 
    set clef_accidentals(Da) "="
    set clef_accidentals(Db) "=" 

    # Eb clef
    set clef_accidentals(EbC) "=" 
    set clef_accidentals(EbD) "=" 
    set clef_accidentals(EbE) "_"
    set clef_accidentals(EbF) "="
    set clef_accidentals(EbG) "=" 
    set clef_accidentals(EbA) "_" 
    set clef_accidentals(EbB) "_"
    set clef_accidentals(Ebc) "=" 
    set clef_accidentals(Ebd) "=" 
    set clef_accidentals(Ebe) "_"
    set clef_accidentals(Ebf) "=" 
    set clef_accidentals(Ebg) "=" 
    set clef_accidentals(Eba) "_"
    set clef_accidentals(Ebb) "_" 

    # E clef
    set clef_accidentals(EC) "^" 
    set clef_accidentals(ED) "^" 
    set clef_accidentals(EE) "="
    set clef_accidentals(EF) "^"
    set clef_accidentals(EG) "^" 
    set clef_accidentals(EA) "=" 
    set clef_accidentals(EB) "="
    set clef_accidentals(Ec) "^" 
    set clef_accidentals(Ed) "^" 
    set clef_accidentals(Ee) "="
    set clef_accidentals(Ef) "^" 
    set clef_accidentals(Eg) "^" 
    set clef_accidentals(Ea) "="
    set clef_accidentals(Eb) "=" 

    # F clef
    set clef_accidentals(FC) "=" 
    set clef_accidentals(FD) "=" 
    set clef_accidentals(FE) "="
    set clef_accidentals(FF) "="
    set clef_accidentals(FG) "=" 
    set clef_accidentals(FA) "=" 
    set clef_accidentals(FB) "_"
    set clef_accidentals(Fc) "=" 
    set clef_accidentals(Fd) "=" 
    set clef_accidentals(Fe) "="
    set clef_accidentals(Ff) "=" 
    set clef_accidentals(Fg) "=" 
    set clef_accidentals(Fa) "="
    set clef_accidentals(Fb) "_" 

    # F# clef
    set clef_accidentals(F#C) "^" 
    set clef_accidentals(F#D) "^" 
    set clef_accidentals(F#F) "^"
    set clef_accidentals(F#E) "^"
    set clef_accidentals(F#G) "^" 
    set clef_accidentals(F#A) "^" 
    set clef_accidentals(F#B) "="
    set clef_accidentals(F#c) "^" 
    set clef_accidentals(F#d) "^" 
    set clef_accidentals(F#e) "^"
    set clef_accidentals(F#f) "^" 
    set clef_accidentals(F#g) "^" 
    set clef_accidentals(F#a) "^"
    set clef_accidentals(F#b) "=" 

    # Gb clef
    set clef_accidentals(GbC) "_" 
    set clef_accidentals(GbD) "_" 
    set clef_accidentals(GbE) "_"
    set clef_accidentals(GbF) "="
    set clef_accidentals(GbG) "_" 
    set clef_accidentals(GbA) "_" 
    set clef_accidentals(GbB) "_"
    set clef_accidentals(Gbc) "_" 
    set clef_accidentals(Gbd) "_" 
    set clef_accidentals(Gbe) "_"
    set clef_accidentals(Gbf) "=" 
    set clef_accidentals(Gbg) "_" 
    set clef_accidentals(Gba) "_"
    set clef_accidentals(Gbb) "_" 

    # G clef
    set clef_accidentals(GC) "=" 
    set clef_accidentals(GD) "=" 
    set clef_accidentals(GE) "="
    set clef_accidentals(GF) "^"
    set clef_accidentals(GG) "=" 
    set clef_accidentals(GA) "=" 
    set clef_accidentals(GB) "="
    set clef_accidentals(Gc) "=" 
    set clef_accidentals(Gd) "=" 
    set clef_accidentals(Ge) "="
    set clef_accidentals(Gf) "^" 
    set clef_accidentals(Gg) "=" 
    set clef_accidentals(Ga) "="
    set clef_accidentals(Gb) "=" 

    # Ab clef
    set clef_accidentals(AbC) "=" 
    set clef_accidentals(AbD) "_" 
    set clef_accidentals(AbE) "_"
    set clef_accidentals(AbF) "="
    set clef_accidentals(AbG) "=" 
    set clef_accidentals(AbA) "_" 
    set clef_accidentals(AbB) "_"
    set clef_accidentals(Abc) "=" 
    set clef_accidentals(Abd) "_" 
    set clef_accidentals(Abe) "_"
    set clef_accidentals(Abf) "=" 
    set clef_accidentals(Abg) "=" 
    set clef_accidentals(Aba) "_"
    set clef_accidentals(Abb) "_" 

    # A clef
    set clef_accidentals(AC) "^" 
    set clef_accidentals(AD) "=" 
    set clef_accidentals(AE) "="
    set clef_accidentals(AF) "^"
    set clef_accidentals(AG) "^" 
    set clef_accidentals(AA) "=" 
    set clef_accidentals(AB) "="
    set clef_accidentals(Ac) "^" 
    set clef_accidentals(Ad) "=" 
    set clef_accidentals(Ae) "="
    set clef_accidentals(Af) "^" 
    set clef_accidentals(Ag) "^" 
    set clef_accidentals(Aa) "="
    set clef_accidentals(Ab) "=" 

    # Bb clef
    set clef_accidentals(BbC) "=" 
    set clef_accidentals(BbD) "=" 
    set clef_accidentals(BbE) "_"
    set clef_accidentals(BbF) "="
    set clef_accidentals(BbG) "=" 
    set clef_accidentals(BbA) "=" 
    set clef_accidentals(BbB) "_"
    set clef_accidentals(Bbc) "=" 
    set clef_accidentals(Bbd) "=" 
    set clef_accidentals(Bbe) "_"
    set clef_accidentals(Bbf) "=" 
    set clef_accidentals(Bbg) "=" 
    set clef_accidentals(Bba) "="
    set clef_accidentals(Bbb) "_" 

    # B clef
    set clef_accidentals(BC) "^" 
    set clef_accidentals(BD) "^" 
    set clef_accidentals(BE) "="
    set clef_accidentals(BF) "^"
    set clef_accidentals(BG) "^" 
    set clef_accidentals(BA) "^" 
    set clef_accidentals(BB) "="
    set clef_accidentals(Bc) "^" 
    set clef_accidentals(Bd) "^" 
    set clef_accidentals(Be) "="
    set clef_accidentals(Bf) "^" 
    set clef_accidentals(Bg) "^" 
    set clef_accidentals(Ba) "^"
    set clef_accidentals(Bb) "=" 
    
    # Gb clef
    set clef_accidentals(GbC) "_" 
    set clef_accidentals(GbD) "_" 
    set clef_accidentals(GbE) "_"
    set clef_accidentals(GbF) "_"
    set clef_accidentals(GbG) "_" 
    set clef_accidentals(GbA) "_" 
    set clef_accidentals(GbB) "_"
    set clef_accidentals(Gbc) "_" 
    set clef_accidentals(Gbd) "_" 
    set clef_accidentals(Gbe) "_"
    set clef_accidentals(Gbf) "_" 
    set clef_accidentals(Gbg) "_" 
    set clef_accidentals(Gba) "_"
    set clef_accidentals(Gbb) "_" 

  set key "C"
  redacc_clef_setting

  while { [gets $file input_line] != -1 } {

	# Looks for double flats and double sharps
	if { [string match *__* $input_line] != 1 && \
	[string match *^^* $input_line] != 1 && \
	[string match *^B* $input_line] != 1 && \
	[string match *^b* $input_line] != 1 && \
	[string match *^E* $input_line] != 1 && \
	[string match *^e* $input_line] != 1 && \
	[string match *_C* $input_line] != 1 && \
	[string match *_c* $input_line] != 1 && \
	[string match *_F* $input_line] != 1 && \
	[string match *_f* $input_line] != 1 } {
	     puts $file2 "$input_line"
		 if { [regexp $pattern1 $input_line key] == 1 } { 
		    regsub  {(K:)} $key {} key
		    regsub -all {( )} $key {} key
		    redacc_clef_setting 
		    } elseif { [regexp $pattern2 $input_line key] == 1 } { 
		    regsub  {(\[)} $key {} key
		    regsub  {(\])} $key {} key
		    regsub  {(K:)} $key {} key
		    regsub -all {( )} $key {} key
		 redacc_clef_setting }
	 } else { redacc_reduce_accidentals }    
  }
}

proc redacc_reduce_accidentals {} {
global input_line pattern2 bar_content bar_content2 key file2

set modified_line ""
set bars [split $input_line |]
set bar_position 0

while { $bar_position <= [expr [llength $bars]-1] } {

set bar_number [expr $bar_position+1]
set bar_content [lindex $bars $bar_position]

if { [regexp $pattern2 $bar_content key] == 1 } { 
    regsub  {(\[)} $key {} key
    regsub  {(\])} $key {} key
    regsub  {(K:)} $key {} key
    regsub -all {( )} $key {} key
    redacc_clef_setting 
}
if { [string match *__* $bar_content] == 1 || \
[string match *^^* $bar_content] == 1 || \
[string match *^^* $bar_content] == 1 || \
[string match *^B* $bar_content] == 1 || \
[string match *^b* $bar_content] == 1 || \
[string match *^E* $bar_content] == 1 || \
[string match *^e* $bar_content] == 1 || \
[string match *_C* $bar_content] == 1 || \
[string match *_c* $bar_content] == 1 || \
[string match *_F* $bar_content] == 1 || \
[string match *_f* $bar_content] == 1 } {
    redacc_reduce_accidentals_pass_2 
    set bar_content $bar_content2
}
set modified_line "$modified_line$bar_content"

if { $bar_position < [expr [llength $bars]-1] } {
set modified_line "$modified_line|"
} 

incr bar_position
}

puts $file2 "$modified_line"

}
proc redacc_clef_setting {} {
    global key clef_accidentals notes accidentals accidentals2
    for { set notes_location 0 } { $notes_location <= 54 } { } { 
	regsub -all {(')} $notes($notes_location) {,} notes_no_octave
	regsub -all {(\,)} $notes_no_octave {} notes_no_octave
	set accidentals($notes($notes_location)) $clef_accidentals($key$notes_no_octave)
	set accidentals2($notes($notes_location)) $clef_accidentals($key$notes_no_octave)
	incr notes_location
    }
}
    
proc redacc_reduce_accidentals_pass_2 {} {
    global accidentals accidentals2 key bar_content notes flats sharps naturals_s naturals_f bar_content2
    redacc_clef_setting
	set bar_length [string length $bar_content]
	set bar_length [expr $bar_length-1]
	set bar_content2 ""
	set bar_location 0
	set replaced 0
	set quote_open1 0
	set quote_open2 0

	for { set bar_location 0 } { $bar_location <= $bar_length } { } { 
	    set bar_column [string range $bar_content $bar_location $bar_location]
	    if { $bar_column == "\"" } {
		if { $quote_open1 == 0 } {
		    set quote_open1 1 
		} else {
		    set quote_open1 0
		}
	    }
	    if { $bar_column == "!" } {
		if { $quote_open2 == 0 } {
		    set quote_open2 1 
		} else {
		    set quote_open2 0
		}
	    }
	    set detected_note ""
	    set incr_bar_location 1
	    if { [string match {[A-Za-z]} $bar_column ] == 1 && $quote_open1 == 0 && $quote_open2 == 0 } {
		set string_4_characters [string range $bar_content $bar_location [expr $bar_location + 3]]
		if { [array get accidentals $string_4_characters] == "" } {
		    set string_3_characters [string range $bar_content $bar_location [expr $bar_location + 2]]
		    if { [array get accidentals $string_3_characters] == "" } {
		        set string_2_characters [string range $bar_content $bar_location [expr $bar_location + 1]]
			if { [array get accidentals $string_2_characters] == "" } {
			    if { [array get accidentals $bar_column] == "" } {
				set replaced 0
	    		    } else { 
				set replaced 1
				set detected_note $bar_column
			    }
			} else { 
			    set replaced 1
			    set detected_note $string_2_characters 
			    set incr_bar_location 2
			}
		    } else {
			set replaced 1
			set detected_note $string_3_characters 
			set incr_bar_location 3
		    }
		} else {
		    set replaced 1
		    set detected_note $string_4_characters
		    set incr_bar_location 4
		}
	    } 
	
	if { $replaced == 0 } {
	    set bar_content2 "$bar_content2$bar_column"
	} else {
	    if { $bar_location > 0 } { 
		set bar_prev_column [string range $bar_content [expr $bar_location - 1] [expr $bar_location - 1]]
		if { $bar_prev_column == "_" || $bar_prev_column == "^" || $bar_prev_column == "=" } {
		    set bar_content2 "$bar_content2$detected_note"
		    set accidentals($detected_note) $bar_prev_column
		    if { $bar_location > 1 } {
			set bar_prev_column2 [string range $bar_content [expr $bar_location - 2] [expr $bar_location - 2]]
			if { $bar_prev_column2 == "_" || $bar_prev_column2 == "^" } { 
			    set accidentals($detected_note) "$bar_prev_column2$bar_prev_column"
			}
		    }
		} else { set bar_content2 "$bar_content2$accidentals($detected_note)$detected_note" }	
	    } else { set bar_content2 "$bar_content2$accidentals($detected_note)$detected_note" }
		
	set replaced 0
	
	}
	set bar_location [expr $bar_location + $incr_bar_location]
    }
# Replacing double accidentals for naturals

    regsub -all {\^\^c} $bar_content2 {=d} bar_content2
    regsub -all {\^\^C} $bar_content2 {=D} bar_content2
    regsub -all {\^\^d} $bar_content2 {=e} bar_content2
    regsub -all {\^\^D} $bar_content2 {=E} bar_content2
    regsub -all {\^\^e} $bar_content2 {^f} bar_content2
    regsub -all {\^e} $bar_content2 {=f} bar_content2
    regsub -all {\^\^E} $bar_content2 {^F} bar_content2
    regsub -all {\^E} $bar_content2 {=F} bar_content2
    regsub -all {\^\^f} $bar_content2 {=g} bar_content2
    regsub -all {\^\^F} $bar_content2 {=G} bar_content2
    regsub -all {\^\^g} $bar_content2 {=a} bar_content2
    regsub -all {\^\^G} $bar_content2 {=A} bar_content2
    regsub -all {\^\^a} $bar_content2 {=b} bar_content2
    regsub -all {\^\^A} $bar_content2 {=B} bar_content2
    regsub -all {\^\^B,,,} $bar_content2 {^C,,} bar_content2
    regsub -all {\^B,,,} $bar_content2 {=C,,} bar_content2    
    regsub -all {\^\^B,,} $bar_content2 {^C,} bar_content2
    regsub -all {\^B,,} $bar_content2 {=C,} bar_content2
    regsub -all {\^\^B,} $bar_content2 {^C} bar_content2
    regsub -all {\^B,} $bar_content2 {=C} bar_content2
    regsub -all {\^\^B} $bar_content2 {^c} bar_content2
    regsub -all {\^B} $bar_content2 {=c} bar_content2
    regsub -all {\^\^b} $bar_content2 {^c'} bar_content2
    regsub -all {\^b} $bar_content2 {=c'} bar_content2

    regsub -all {__c'''} $bar_content2 {_b''} bar_content2
    regsub -all {_c'''} $bar_content2 {=b''} bar_content2
    regsub -all {__c''} $bar_content2 {_b'} bar_content2
    regsub -all {_c''} $bar_content2 {=b'} bar_content2
    regsub -all {__c'} $bar_content2 {_b} bar_content2
    regsub -all {_c'} $bar_content2 {=b} bar_content2
    regsub -all {__c} $bar_content2 {_B} bar_content2
    regsub -all {_c} $bar_content2 {=B} bar_content2
    regsub -all {__C} $bar_content2 {_B,} bar_content2
    regsub -all {_C} $bar_content2 {=B,} bar_content2
    regsub -all {__d} $bar_content2 {=c} bar_content2
    regsub -all {__D} $bar_content2 {=C} bar_content2
    regsub -all {__e} $bar_content2 {=d} bar_content2
    regsub -all {__E} $bar_content2 {=D} bar_content2
    regsub -all {__f} $bar_content2 {_e} bar_content2
    regsub -all {_f} $bar_content2 {=e} bar_content2
    regsub -all {__F} $bar_content2 {_E} bar_content2
    regsub -all {_F} $bar_content2 {=E} bar_content2
    regsub -all {__g} $bar_content2 {=f} bar_content2
    regsub -all {__G} $bar_content2 {=F} bar_content2
    regsub -all {__a} $bar_content2 {=g} bar_content2
    regsub -all {__A} $bar_content2 {=G} bar_content2
    regsub -all {__b} $bar_content2 {=a} bar_content2
    regsub -all {__B} $bar_content2 {=A} bar_content2
    
# Eliminate the unnecessary accidentals

	set bar_length [string length $bar_content2]
	set bar_length [expr $bar_length-1]
	set bar_location 0
	set quote_open1 0
	set quote_open2 0

	for { set bar_location 0 } { $bar_location <= $bar_length } { } { 
	    set bar_column [string range $bar_content2 $bar_location $bar_location]
	    if { $bar_column == "\"" } {
		if { $quote_open1 == 0 } {
		    set quote_open1 1 
		} else {
		    set quote_open1 0
		}
	    }
	    if { $bar_column == "!" } {
		if { $quote_open2 == 0 } {
		    set quote_open2 1 
		} else {
		    set quote_open2 0
		}
	    }

	    set detected_note ""

	    if { [string match {[A-Za-z]} $bar_column ] == 1 && $quote_open1 == 0 && $quote_open2 == 0 } {
		set string_4_characters [string range $bar_content2 $bar_location [expr $bar_location + 3]]
		if { [array get accidentals $string_4_characters] == "" } {
		    set string_3_characters [string range $bar_content2 $bar_location [expr $bar_location + 2]]
		    if { [array get accidentals $string_3_characters] == "" } {
		        set string_2_characters [string range $bar_content2 $bar_location [expr $bar_location + 1]]
			if { [array get accidentals $string_2_characters] == "" } {
			    if { [array get accidentals $bar_column] == "" } {
				set replaced 0
	    		    } else { 
				set replaced 1
				set detected_note $bar_column
			    }
			} else { 
			    set replaced 1
			    set detected_note $string_2_characters 
			    set incr_bar_location 2
			}
		    } else {
			set replaced 1
			set detected_note $string_3_characters 
			set incr_bar_location 3
		    }
		} else {
		    set replaced 1
		    set detected_note $string_4_characters
		    set incr_bar_location 4
		}
	    } 
	
	if { $replaced == 0 } {
	    set incr_bar_location 1
	} else {
	    if { $bar_location > 0 } { 
		set bar_prev_column [string range $bar_content2 [expr $bar_location - 1] [expr $bar_location - 1]]
		if { $bar_prev_column == "_" || $bar_prev_column == "^" || $bar_prev_column == "=" } {
		    if { $bar_prev_column == $accidentals2($detected_note) } {
			set bar_content2 [string replace $bar_content2 [expr $bar_location - 1] [expr $bar_location - 1] ""]
		    } else { set accidentals2($detected_note) $bar_prev_column }
		} 
	    } 
		
	set replaced 0
	
	}
	set bar_location [expr $bar_location + $incr_bar_location]
    }
}

############# Main Program
proc abcra {} {
global input_line file file2
redacc_load_file
# close $file
close $file2
}


